@@ -2,7 +2,7 @@
 *                                                                      *
 *               This software is part of the ast package               *
 *          Copyright (c) 1982-2014 AT&T Intellectual Property          *
-*          Copyright (c) 2020-2022 Contributors to ksh 93u+m           *
+*          Copyright (c) 2020-2023 Contributors to ksh 93u+m           *
 *                      and is licensed under the                       *
 *                 Eclipse Public License, Version 2.0                  *
 *                                                                      *
@@ -13,6 +13,7 @@
 *                  David Korn <dgk@research.att.com>                   *
 *                  Martijn Dekker <martijn@inlv.org>                   *
 *            Johnothan King <johnothanking@protonmail.com>             *
+*                      Phi <phi.debian@gmail.com>                      *
 *                                                                      *
 ***********************************************************************/
 /*
@@ -27,7 +28,6 @@
 #include	"shopt.h"
 #include	"defs.h"
 #include	<error.h>
-#include	<stak.h>
 #include	"io.h"
 #include	"name.h"
 #include	"history.h"
@@ -56,6 +56,7 @@ struct printf
 	Sffmt_t		hdr;
 	int		argsize;
 	int		intvar;
+	char		**argv0; /* see reload() below */
 	char		**nextarg;
 	char		*lastarg;
 	char		cescape;
@@ -83,6 +84,7 @@ static const struct printmap  Pmap[] =
 
 static int		echolist(Sfio_t*, int, char**);
 static int		extend(Sfio_t*,void*, Sffmt_t*);
+static int		reload(int argn, char fmt, void* v, Sffmt_t* fe);
 static char		*genformat(char*);
 static int		fmtvecho(const char*, struct printf*);
 static ssize_t		fmtbase64(Sfio_t*, char*, int);
@@ -108,13 +110,13 @@ static int	exitval;
 	/* This mess is because /bin/echo on BSD is different */
 	if(!sh.universe)
 	{
-		register char *universe;
+		char *universe;
 		if(universe=astconf("UNIVERSE",0,0))
 			bsd_univ = (strcmp(universe,"ucb")==0);
 		sh.universe = 1;
 	}
 	if(!bsd_univ)
-		return(b_print(0,argv,(Shbltin_t*)&prdata));
+		return b_print(0,argv,(Shbltin_t*)&prdata);
 	prdata.options = sh_optecho;
 	prdata.raw = 1;
 	while(argv[1] && *argv[1]=='-')
@@ -134,7 +136,7 @@ static int	exitval;
 			break;
 		argv++;
 	}
-	return(b_print(0,argv,(Shbltin_t*)&prdata));
+	return b_print(0,argv,(Shbltin_t*)&prdata);
    }
 #endif /* SHOPT_ECHOPRINT */
 
@@ -145,7 +147,7 @@ int    b_printf(int argc, char *argv[],Shbltin_t *context)
 	NOT_USED(context);
 	memset(&prdata,0,sizeof(prdata));
 	prdata.options = sh_optprintf;
-	return(b_print(-1,argv,(Shbltin_t*)&prdata));
+	return b_print(-1,argv,(Shbltin_t*)&prdata);
 }
 
 static int infof(Opt_t* op, Sfio_t* sp, const char* s, Optdisc_t* dp)
@@ -154,7 +156,7 @@ static int infof(Opt_t* op, Sfio_t* sp, const char* s, Optdisc_t* dp)
 	char c='%';
 	for(pm=Pmap;pm->size>0;pm++)
 		sfprintf(sp, "[+%c(%s)q?Equivalent to %s.]",c,pm->name,pm->equivalent);
-	return(1);
+	return 1;
 }
 
 /*
@@ -163,11 +165,14 @@ static int infof(Opt_t* op, Sfio_t* sp, const char* s, Optdisc_t* dp)
  */
 int    b_print(int argc, char *argv[], Shbltin_t *context)
 {
-	register Sfio_t *outfile;
-	register int n, fd = 1;
+	Sfio_t *outfile;
+	int n, fd = 1;
 	const char *options, *msg = e_file+4;
 	char *format = 0;
-	int sflag = 0, nflag=0, rflag=0, vflag=0;
+#if !SHOPT_SCRIPTONLY
+	int sflag = 0;
+#endif /* !SHOPT_SCRIPTONLY */
+	int nflag=0, rflag=0, vflag=0;
 	Namval_t *vname=0;
 	Optdisc_t disc;
 	exitval = 0;
@@ -198,12 +203,14 @@ int    b_print(int argc, char *argv[], Shbltin_t *context)
 			nflag++;
 			break;
 		case 'p':
+		coprocess:
 			fd = sh.coutpipe;
 			msg = e_query;
 			break;
 		case 'f':
 			format = opt_info.arg;
 			break;
+#if !SHOPT_SCRIPTONLY
 		case 's':
 			/* print to history file */
 			if(!sh_histinit())
@@ -215,6 +222,7 @@ int    b_print(int argc, char *argv[], Shbltin_t *context)
 			sh_onstate(SH_HISTORY);
 			sflag++;
 			break;
+#endif /* !SHOPT_SCRIPTONLY */
 		case 'e':
 			rflag = 0;
 			break;
@@ -223,17 +231,17 @@ int    b_print(int argc, char *argv[], Shbltin_t *context)
 			break;
 		case 'u':
 			if(opt_info.arg[0]=='p' && opt_info.arg[1]==0)
-			{
-				fd = sh.coutpipe;
-				msg = e_query;
-				break;
-			}
+				goto coprocess;
 			fd = (int)strtol(opt_info.arg,&opt_info.arg,10);
 			if(*opt_info.arg)
 				fd = -1;
 			else if(!sh_iovalidfd(fd))
 				fd = -1;
+#if SHOPT_SCRIPTONLY
+			else if(!(sh.inuse_bits&(1<<fd)) && sh_inuse(fd))
+#else
 			else if(!(sh.inuse_bits&(1<<fd)) && (sh_inuse(fd) || (sh.hist_ptr && fd==sffileno(sh.hist_ptr->histfp))))
+#endif /* SHOPT_SCRIPTONLY */
 				fd = -1;
 			break;
 		case 'v':
@@ -282,7 +290,7 @@ int    b_print(int argc, char *argv[], Shbltin_t *context)
 	argv += opt_info.index;
 	if(error_info.errors || (argc<0 && !(format = *argv++)))
 	{
-		errormsg(SH_DICT,ERROR_usage(2),"%s",optusage((char*)0));
+		errormsg(SH_DICT,ERROR_usage(2),"%s",optusage(NULL));
 		UNREACHABLE();
 	}
 	if(vflag && format)
@@ -298,9 +306,10 @@ skip:
 		argv++;
 	if(vname)
 	{
-		if(!sh.strbuf2)
-			sh.strbuf2 = sfstropen();
-		outfile = sh.strbuf2;
+		static Sfio_t *vbuf;
+		if(!vbuf)
+			vbuf = sfstropen();
+		outfile = vbuf;
 		goto printf_v;
 	}
 skip2:
@@ -315,7 +324,7 @@ skip2:
 	{
 		/* don't print error message for stdout for compatibility */
 		if(fd==1)
-			return(1);
+			return 1;
 		errormsg(SH_DICT,ERROR_system(1),msg);
 		UNREACHABLE();
 	}
@@ -323,7 +332,7 @@ skip2:
 	{
 		sh_onstate(SH_NOTRACK);
 		n = SF_WRITE|((n&IOREAD)?SF_READ:0);
-		sh.sftable[fd] = outfile = sfnew(NIL(Sfio_t*),sh.outbuff,IOBSIZE,fd,n);
+		sh.sftable[fd] = outfile = sfnew(NULL,sh.outbuff,IOBSIZE,fd,n);
 		sh_offstate(SH_NOTRACK);
 		sfpool(outfile,sh.outpool,SF_WRITE);
 	}
@@ -338,18 +347,20 @@ printf_v:
 		memset(&pdata, 0, sizeof(pdata));
 		pdata.hdr.version = SFIO_VERSION;
 		pdata.hdr.extf = extend;
+		pdata.hdr.reloadf = reload;
 		pdata.nextarg = argv;
 		sh_offstate(SH_STOPOK);
-		pool=sfpool(sfstderr,NIL(Sfio_t*),SF_WRITE);
+		pool=sfpool(sfstderr,NULL,SF_WRITE);
 		do
 		{
+			pdata.argv0 = pdata.nextarg;
 			if(sh.trapnote&SH_SIGSET)
 				break;
 			pdata.hdr.form = format;
 			sfprintf(outfile,"%!",&pdata);
 		} while(*pdata.nextarg && pdata.nextarg!=argv);
 		if(pdata.nextarg == nullarg && pdata.argsize>0)
-			if(sfwrite(outfile,stakptr(staktell()),pdata.argsize) < 0)
+			if(sfwrite(outfile,stkptr(sh.stk,stktell(sh.stk)),pdata.argsize) < 0)
 				exitval = 1;
 		sfpool(sfstderr,pool,SF_WRITE);
 		if (pdata.err)
@@ -371,7 +382,7 @@ printf_v:
 		/* echo style print */
 		if(nflag && !argv[0])
 		{
-			if (sfsync((Sfio_t*)0) < 0)
+			if (sfsync(NULL) < 0)
 				exitval = 1;
 		}
 		else if(echolist(outfile,rflag,argv) && !nflag)
@@ -380,11 +391,13 @@ printf_v:
 	}
 	if(vname)
 		nv_putval(vname, sfstruse(outfile), 0);
+#if !SHOPT_SCRIPTONLY
 	else if(sflag)
 	{
 		hist_flush(sh.hist_ptr);
 		sh_offstate(SH_HISTORY);
 	}
+#endif /* !SHOPT_SCRIPTONLY */
 	else
 	{
 		if(n&SF_SHARE)
@@ -392,7 +405,7 @@ printf_v:
 		if (sfsync(outfile) < 0)
 			exitval = 1;
 	}
-	return(exitval);
+	return exitval;
 }
 
 /*
@@ -402,8 +415,8 @@ printf_v:
  */
 static int echolist(Sfio_t *outfile, int raw, char *argv[])
 {
-	register char	*cp;
-	register int	n;
+	char	*cp;
+	int	n;
 	struct printf pdata;
 	pdata.cescape = 0;
 	pdata.err = 0;
@@ -412,7 +425,7 @@ static int echolist(Sfio_t *outfile, int raw, char *argv[])
 		if(!raw  && (n=fmtvecho(cp,&pdata))>=0)
 		{
 			if(n)
-				if(sfwrite(outfile,stakptr(staktell()),n) < 0)
+				if(sfwrite(outfile,stkptr(sh.stk,stktell(sh.stk)),n) < 0)
 					exitval = 1;
 		}
 		else
@@ -423,7 +436,7 @@ static int echolist(Sfio_t *outfile, int raw, char *argv[])
 				exitval = 1;
 		sh_sigcheck();
 	}
-	return(!pdata.cescape);
+	return !pdata.cescape;
 }
 
 /*
@@ -431,10 +444,10 @@ static int echolist(Sfio_t *outfile, int raw, char *argv[])
  */
 static char strformat(char *s)
 {
-        register char*  t;
-        register int    c;
-        char*           b;
-        char*           p;
+        char*		t;
+        int		c;
+        char*		b;
+        char*		p;
 #if SHOPT_MULTIBYTE && defined(FMT_EXP_WIDE)
 	int		w;
 #endif
@@ -477,7 +490,7 @@ static char strformat(char *s)
                         break;
                     case 0:
                         *t = 0;
-                        return(t - b);
+                        return t - b;
                 }
                 *t++ = c;
         }
@@ -485,18 +498,18 @@ static char strformat(char *s)
 
 static char *genformat(char *format)
 {
-	register char *fp;
-	stakseek(0);
-	stakputs(format);
-	fp = (char*)stakfreeze(1);
+	char *fp;
+	stkseek(sh.stk,0);
+	sfputr(sh.stk,format,-1);
+	fp = stkfreeze(sh.stk,1);
 	strformat(fp);
-	return(fp);
+	return fp;
 }
 
 static char *fmthtml(const char *string, int flags)
 {
-	register const char *cp = string, *op;
-	register int c, offset = staktell();
+	const char *cp = string, *op;
+	int c, offset = stktell(sh.stk);
 	/*
 	 * The only multibyte locale ksh currently supports is UTF-8, which is a superset of ASCII. So, if we're on an
 	 * EBCDIC system, below we attempt to convert EBCDIC to ASCII only if we're not in a multibyte locale (mbwide()).
@@ -510,19 +523,19 @@ static char *fmthtml(const char *string, int flags)
 			if(!mbwide())
 				c = CCMAPC(c,CC_NATIVE,CC_ASCII);
 			if(mbwide() && c < 0)		/* invalid multibyte char */
-				stakputc('?');
+				sfputc(sh.stk,'?');
 			else if(c == 60)		/* < */
-				stakputs("&lt;");
+				sfputr(sh.stk,"&lt;",-1);
 			else if(c == 62)		/* > */
-				stakputs("&gt;");
+				sfputr(sh.stk,"&gt;",-1);
 			else if(c == 38)		/* & */
-				stakputs("&amp;");
+				sfputr(sh.stk,"&amp;",-1);
 			else if(c == 34)		/* " */
-				stakputs("&quot;");
+				sfputr(sh.stk,"&quot;",-1);
 			else if(c == 39)		/* ' (&apos; is not HTML) */
-				stakputs("&#39;");
+				sfputr(sh.stk,"&#39;",-1);
 			else
-				stakwrite(op, cp-op);
+				sfwrite(sh.stk, op, cp-op);
 		}
 	}
 	else
@@ -533,12 +546,12 @@ static char *fmthtml(const char *string, int flags)
 			while(op = cp, c = mbchar(cp))
 			{
 				if(c < 0)
-					stakputs("%3F");
+					sfputr(sh.stk,"%3F",-1);
 				else if(c < 128 && strchr(URI_RFC3986_UNRESERVED, c))
-					stakputc(c);
+					sfputc(sh.stk,c);
 				else
 					while(c = *(unsigned char*)op++, op <= cp)
-						sfprintf(stkstd, "%%%02X", c);
+						sfprintf(sh.stk, "%%%02X", c);
 			}
 		}
 		else
@@ -546,14 +559,14 @@ static char *fmthtml(const char *string, int flags)
 			while(c = *(unsigned char*)cp++)
 			{
 				if(strchr(URI_RFC3986_UNRESERVED, c))
-					stakputc(c);
+					sfputc(sh.stk,c);
 				else
-					sfprintf(stkstd, "%%%02X", CCMAPC(c, CC_NATIVE, CC_ASCII));
+					sfprintf(sh.stk, "%%%02X", CCMAPC(c, CC_NATIVE, CC_ASCII));
 			}
 		}
 	}
-	stakputc(0);
-	return(stakptr(offset));
+	sfputc(sh.stk,0);
+	return stkptr(sh.stk,offset);
 }
 
 static ssize_t fmtbase64(Sfio_t *iop, char *string, int alt)
@@ -561,7 +574,7 @@ static ssize_t fmtbase64(Sfio_t *iop, char *string, int alt)
 	char			*cp;
 	Sfdouble_t		d;
 	ssize_t			size;
-	Namval_t		*np = nv_open(string, NiL, NV_VARNAME|NV_NOADD);
+	Namval_t		*np = nv_open(string, NULL, NV_VARNAME|NV_NOADD);
 	Namarr_t		*ap;
 	static union types_t	number;
 	if(!np || nv_isnull(np))
@@ -571,7 +584,7 @@ static ssize_t fmtbase64(Sfio_t *iop, char *string, int alt)
 			errormsg(SH_DICT,ERROR_exit(1),e_notset,string);
 			UNREACHABLE();
 		}
-		return(0);
+		return 0;
 	}
 	if(nv_isattr(np,NV_INTEGER))
 	{
@@ -612,7 +625,7 @@ static ssize_t fmtbase64(Sfio_t *iop, char *string, int alt)
 				number.i = (int)d; 
 			}
 		}
-		return(sfwrite(iop, (void*)&number, size));
+		return sfwrite(iop, &number, size);
 	}
 	if(nv_isattr(np,NV_BINARY))
 	{
@@ -638,7 +651,7 @@ static ssize_t fmtbase64(Sfio_t *iop, char *string, int alt)
 			if((size = n)==0)
 				size = strlen(cp);
 			size = sfwrite(iop, cp, size);
-			return(n?n:size);
+			return n?n:size;
 		}
 	}
 	else if(nv_isarray(np) && (ap=nv_arrayptr(np)) && array_elem(ap) && (ap->nelem&(ARRAY_UNDEF|ARRAY_SCAN)))
@@ -646,7 +659,7 @@ static ssize_t fmtbase64(Sfio_t *iop, char *string, int alt)
 		nv_outnode(np,iop,(alt?-1:0),0);
 		if(sfputc(iop,')') < 0)
 			exitval = 1;
-		return(sftell(iop));
+		return sftell(iop);
 	}
 	else
 	{
@@ -658,15 +671,15 @@ static ssize_t fmtbase64(Sfio_t *iop, char *string, int alt)
 		if(alt)
 			nv_offattr(np,NV_EXPORT);
 		if(!cp)
-			return(0);
+			return 0;
 		size = strlen(cp);
-		return(sfwrite(iop,cp,size));
+		return sfwrite(iop,cp,size);
 	}
 }
 
 static int varname(const char *str, int n)
 {
-	register int c,dot=1,len=1;
+	int c,dot=1,len=1;
 	if(n < 0)
 	{
 		if(*str=='.')
@@ -687,7 +700,7 @@ static int varname(const char *str, int n)
 			break;
 		dot = (c=='.');
 	}
-	return(n==0);
+	return n==0;
 }
 
 static const char *mapformat(Sffmt_t *fe)
@@ -696,10 +709,10 @@ static const char *mapformat(Sffmt_t *fe)
 	while(pm->size>0)
 	{
 		if(pm->size==fe->n_str && strncmp(pm->name,fe->t_str,fe->n_str)==0)
-			return(pm->map);
+			return pm->map;
 		pm++;
 	}
-	return(0);
+	return NULL;
 }
 
 static int extend(Sfio_t* sp, void* v, Sffmt_t* fe)
@@ -713,7 +726,7 @@ static int extend(Sfio_t* sp, void* v, Sffmt_t* fe)
 	int		fold = fe->base;
 	union types_t*	value = (union types_t*)v;
 	struct printf*	pp = (struct printf*)fe;
-	register char*	argp = *pp->nextarg;
+	char*		argp = *pp->nextarg;
 	char		*w,*s;
 	if(fe->n_str>0 && (format=='T'||format=='Q') && varname(fe->t_str,fe->n_str) && (!argp || varname(argp,-1)))
 	{
@@ -723,7 +736,7 @@ static int extend(Sfio_t* sp, void* v, Sffmt_t* fe)
 			argp = pp->lastarg;
 		if(argp)
 		{
-			sfprintf(sh.strbuf,"%s.%.*s%c",argp,fe->n_str,fe->t_str,0);
+			sfprintf(sh.strbuf,"%s.%.*s",argp,fe->n_str,fe->t_str);
 			argp = sfstruse(sh.strbuf);
 		}
 	}
@@ -777,6 +790,10 @@ static int extend(Sfio_t* sp, void* v, Sffmt_t* fe)
 			fe->fmt = 'd';
 			value->ll = tmxgettime();
 			break;
+		case '.':
+			fe->fmt = 'd';
+			value->ll = 0;
+			break;
 		default:
 			if(!strchr("DdXxoUu",format))
 			{
@@ -891,7 +908,15 @@ static int extend(Sfio_t* sp, void* v, Sffmt_t* fe)
 				}
 				break;
 			default:
-				d = sh_strnum(argp,&lastchar,0);
+				if(sh.bltinfun==b_printf && sh_isoption(SH_POSIX))
+				{
+					/* POSIX requires evaluating a number here, not an arithmetic expression */
+					d = (Sfdouble_t)strtoll(argp,&lastchar,0);
+					if(*lastchar)
+						errormsg(SH_DICT,ERROR_exit(0),e_number,argp);
+				}
+				else
+					d = sh_strnum(argp,&lastchar,0);
 				if(d<longmin)
 				{
 					errormsg(SH_DICT,ERROR_warn(0),e_overflow,argp);
@@ -905,7 +930,7 @@ static int extend(Sfio_t* sp, void* v, Sffmt_t* fe)
 					d = longmax;
 				}
 				value->ll = (Sflong_t)d;
-				if(lastchar == *pp->nextarg)
+				if(lastchar == argp)
 				{
 					value->ll = *argp;
 					lastchar = "";
@@ -922,7 +947,6 @@ static int extend(Sfio_t* sp, void* v, Sffmt_t* fe)
 		case 'E':
 		case 'F':
 		case 'G':
-			d = sh_strnum(*pp->nextarg,&lastchar,0);
 			switch(*argp)
 			{
 			    case '\'':
@@ -935,7 +959,15 @@ static int extend(Sfio_t* sp, void* v, Sffmt_t* fe)
 				}
 				break;
 			    default:
-				d = sh_strnum(*pp->nextarg,&lastchar,0);
+				if(sh.bltinfun==b_printf && sh_isoption(SH_POSIX))
+				{
+					/* POSIX requires evaluating a number here, not an arithmetic expression */
+					d = strtold(argp,&lastchar);
+					if(*lastchar)
+						errormsg(SH_DICT,ERROR_exit(0),e_number,argp);
+				}
+				else
+					d = sh_strnum(argp,&lastchar,0);
 				break;
 			}
                         if(SFFMT_LDOUBLE)
@@ -950,10 +982,10 @@ static int extend(Sfio_t* sp, void* v, Sffmt_t* fe)
 			}
 			break;
 		case 'Q':
-			value->ll = (Sflong_t)strelapsed(*pp->nextarg,&lastchar,1);
+			value->ll = (Sflong_t)strelapsed(argp,&lastchar,1);
 			break;
 		case 'T':
-			value->ll = (Sflong_t)tmxdate(*pp->nextarg,&lastchar,TMX_NOW);
+			value->ll = (Sflong_t)tmxdate(argp,&lastchar,TMX_NOW);
 			break;
 		default:
 			value->ll = 0;
@@ -986,7 +1018,7 @@ static int extend(Sfio_t* sp, void* v, Sffmt_t* fe)
 				pp->argsize = n;
 				return -1;
 			}
-			value->s = stakptr(staktell());
+			value->s = stkptr(sh.stk,stktell(sh.stk));
 			fe->size = n;
 		}
 		break;
@@ -1042,7 +1074,7 @@ static int extend(Sfio_t* sp, void* v, Sffmt_t* fe)
 			value->s = fmttmx(fe->t_str, value->ll);
 			fe->t_str[fe->n_str] = n;
 		}
-		else value->s = fmttmx(NIL(char*), value->ll);
+		else value->s = fmttmx(NULL, value->ll);
 		fe->fmt = 's';
 		fe->size = -1;
 		break;
@@ -1050,6 +1082,52 @@ static int extend(Sfio_t* sp, void* v, Sffmt_t* fe)
 	return 0;
 }
 
+/*
+ * reload() is called when the caller wants to solve a fp[x] cache miss, i.e.,
+ * a type mismatch between the cached type/value and the desired new type.
+ * This keeps the fp[x] cache intact, it just fills up the new value (v)
+ * with a conversion of the fp[x] type/value to the new type/value.
+ * This is a single-use value; the conversion is not cached.
+ * The conversion is handled by the extend() function that uses the current
+ * nextarg (argv[]) and pushes it, i.e., *nextarg++.
+ * To trick extend(), we back up nextarg, set nextarg to argn, do extend()
+ * and restore nextarg.
+ *
+ * fmt==0 is a special case to handle indexed jumps like '%s $5s'.
+ * In that case, argv[0] and argv[4] are consumed and nextarg push
+ * to &argv[5] argv[1..3] is ignored.
+ */
+static int reload(int argn, char fmt, void* v, Sffmt_t* fe)
+{
+	union types_t*	value = (union types_t*)v;
+	struct printf*	pp = (struct printf*)fe;
+	int		r;
+	int		n;
+	if(fmt == 0)
+	{
+		/* Set nextarg */
+		n = 0;
+		if(pp->nextarg != nullarg)
+		{
+			n = pp->nextarg - pp->argv0;
+			pp->nextarg = pp->argv0;
+			while(argn && *pp->nextarg)
+				argn--, pp->nextarg++;
+		}
+		return n;
+	}
+	/*
+	 * fmt!=0 ==> Late conversion on type mismatch on fp[x], i.e., %1$s %1$d
+	 * fp[1-1].fmt='s' ==> %1$d wants an int, go convert.
+	 */
+	n = pp->nextarg - pp->argv0;
+	pp->nextarg = pp->argv0 + argn;
+	fe->fmt = fmt;
+	r = extend(0,v,fe);
+	pp->nextarg = pp->argv0 + n;
+	return r;
+}
+
 /*
  * construct System V echo string out of <cp>
  * If there are no escape sequences, returns -1
@@ -1058,9 +1136,9 @@ static int extend(Sfio_t* sp, void* v, Sffmt_t* fe)
  */
 static int fmtvecho(const char *string, struct printf *pp)
 {
-	register const char *cp = string, *cpmax;
-	register int c;
-	register int offset = staktell();
+	const char *cp = string, *cpmax;
+	int c;
+	int offset = stktell(sh.stk);
 	int chlen;
 	if(mbwide())
 	{
@@ -1077,15 +1155,15 @@ static int fmtvecho(const char *string, struct printf *pp)
 		while((c= *cp++) && (c!='\\'))
 			;
 	if(c==0)
-		return(-1);
+		return -1;
 	c = --cp - string;
 	if(c>0)
-		stakwrite((void*)string,c);
+		sfwrite(sh.stk,string,c);
 	for(; c= *cp; cp++)
 	{
 		if (mbwide() && ((chlen = mbsize(cp)) > 1))
 		{
-			stakwrite(cp,chlen);
+			sfwrite(sh.stk,cp,chlen);
 			cp +=  (chlen-1);
 			continue;
 		}
@@ -1134,11 +1212,11 @@ static int fmtvecho(const char *string, struct printf *pp)
 			default:
 				cp--;
 		}
-		stakputc(c);
+		sfputc(sh.stk,c);
 	}
 done:
-	c = staktell()-offset;
-	stakputc(0);
-	stakseek(offset);
-	return(c);
+	c = stktell(sh.stk)-offset;
+	sfputc(sh.stk,0);
+	stkseek(sh.stk,offset);
+	return c;
 }
