@@ -2,7 +2,7 @@
 *                                                                      *
 *               This software is part of the ast package               *
 *          Copyright (c) 1982-2012 AT&T Intellectual Property          *
-*          Copyright (c) 2020-2022 Contributors to ksh 93u+m           *
+*          Copyright (c) 2020-2023 Contributors to ksh 93u+m           *
 *                      and is licensed under the                       *
 *                 Eclipse Public License, Version 2.0                  *
 *                                                                      *
@@ -90,11 +90,11 @@ static void(*nullscan)(Namval_t*,void*);
 #endif
 int    b_readonly(int argc,char *argv[],Shbltin_t *context)
 {
-	register int flag;
+	int flag;
 	char *command = argv[0];
 	struct tdata tdata;
 	NOT_USED(argc);
-	memset((void*)&tdata,0,sizeof(tdata));
+	memset(&tdata,0,sizeof(tdata));
 	tdata.aflag = '-';
 	while((flag = optget(argv,*command=='e'?sh_optexport:sh_optreadonly))) switch(flag)
 	{
@@ -106,11 +106,11 @@ int    b_readonly(int argc,char *argv[],Shbltin_t *context)
 			break;
 		case '?':
 			errormsg(SH_DICT,ERROR_usage(0), "%s", opt_info.arg);
-			return(2);
+			return 2;
 	}
 	if(error_info.errors)
 	{
-		errormsg(SH_DICT,ERROR_usage(2),optusage(NIL(char*)));
+		errormsg(SH_DICT,ERROR_usage(2),optusage(NULL));
 		UNREACHABLE();
 	}
 	argv += (opt_info.index-1);
@@ -120,7 +120,7 @@ int    b_readonly(int argc,char *argv[],Shbltin_t *context)
 		flag = (NV_ASSIGN|NV_EXPORT|NV_IDENT);
 	if(!sh.prefix)
 		sh.prefix = Empty;
-	return(setall(argv,flag,sh.var_tree, &tdata));
+	return setall(argv,flag,sh.var_tree, &tdata);
 }
 
 /*
@@ -128,16 +128,16 @@ int    b_readonly(int argc,char *argv[],Shbltin_t *context)
  */
 #if 0
     /* for the dictionary generator */
-    int    b_hash(int argc,register char *argv[],Shbltin_t *context){}
+    int    b_hash(int argc,char *argv[],Shbltin_t *context){}
 #endif
-int    b_alias(int argc,register char *argv[],Shbltin_t *context)
+int    b_alias(int argc,char *argv[],Shbltin_t *context)
 {
-	register unsigned flag = NV_NOARRAY|NV_NOSCOPE|NV_ASSIGN;
-	register Dt_t *troot;
-	register int rflag=0, xflag=0, n;
+	unsigned flag = NV_NOARRAY|NV_NOSCOPE|NV_ASSIGN;
+	Dt_t *troot;
+	int rflag=0, xflag=0, n;
 	struct tdata tdata;
 	NOT_USED(argc);
-	memset((void*)&tdata,0,sizeof(tdata));
+	memset(&tdata,0,sizeof(tdata));
 	troot = sh.alias_tree;
 	if(*argv[0]=='h')
 		flag |= NV_TAGGED;
@@ -166,18 +166,18 @@ int    b_alias(int argc,register char *argv[],Shbltin_t *context)
 			break;
 		    case ':':
 			if(sh.shcomp)
-				return(2);  /* don't print usage info while shcomp is compiling */
+				return 2;  /* don't print usage info while shcomp is compiling */
 			errormsg(SH_DICT,2, "%s", opt_info.arg);
 			break;
 		    case '?':
 			if(sh.shcomp)
-				return(2);
+				return 2;
 			errormsg(SH_DICT,ERROR_usage(0), "%s", opt_info.arg);
-			return(2);
+			return 2;
 		}
 		if(error_info.errors)
 		{
-			errormsg(SH_DICT,ERROR_usage(2),"%s",optusage(NIL(char*)));
+			errormsg(SH_DICT,ERROR_usage(2),"%s",optusage(NULL));
 			UNREACHABLE();
 		}
 		argv += (opt_info.index-1);
@@ -186,7 +186,7 @@ int    b_alias(int argc,register char *argv[],Shbltin_t *context)
 	if(flag&NV_TAGGED)
 	{
 		if(xflag)
-			return(0);		/* do nothing for 'alias -tx' */
+			return 0;		/* do nothing for 'alias -tx' */
 		if(tdata.pflag)
 		{
 			troot = sh_subtracktree(0);	/* use existing hash table */
@@ -198,32 +198,32 @@ int    b_alias(int argc,register char *argv[],Shbltin_t *context)
 			tdata.aflag = '-';		/* make setall() treat 'hash' like 'alias -t' */
 		}
 		if(rflag)				/* hash -r: clear hash table */
-			nv_scan(troot,nv_rehash,(void*)0,NV_TAGGED,NV_TAGGED);
+			nv_scan(troot,nv_rehash,NULL,NV_TAGGED,NV_TAGGED);
 	}
-	return(setall(argv,flag,troot,&tdata));
+	return setall(argv,flag,troot,&tdata);
 }
 
 
 #if 0
     /* for the dictionary generator */
-    int    b_autoload(int argc,register char *argv[],Shbltin_t *context){}
-    int    b_compound(int argc,register char *argv[],Shbltin_t *context){}
-    int    b_float(int argc,register char *argv[],Shbltin_t *context){}
-    int    b_functions(int argc,register char *argv[],Shbltin_t *context){}
-    int    b_integer(int argc,register char *argv[],Shbltin_t *context){}
-    int    b_local(int argc,register char *argv[],Shbltin_t *context){}
-    int    b_nameref(int argc,register char *argv[],Shbltin_t *context){}
+    int    b_autoload(int argc,char *argv[],Shbltin_t *context){}
+    int    b_compound(int argc,char *argv[],Shbltin_t *context){}
+    int    b_float(int argc,char *argv[],Shbltin_t *context){}
+    int    b_functions(int argc,char *argv[],Shbltin_t *context){}
+    int    b_integer(int argc,char *argv[],Shbltin_t *context){}
+    int    b_local(int argc,char *argv[],Shbltin_t *context){}
+    int    b_nameref(int argc,char *argv[],Shbltin_t *context){}
 #endif
-int    b_typeset(int argc,register char *argv[],Shbltin_t *context)
+int    b_typeset(int argc,char *argv[],Shbltin_t *context)
 {
-	register int	n, flag = NV_VARNAME|NV_ASSIGN;
+	int		n, flag = NV_VARNAME|NV_ASSIGN;
 	struct tdata	tdata;
 	const char	*optstring = sh_opttypeset;
 	Namdecl_t 	*ntp = (Namdecl_t*)context->ptr;
 	Dt_t		*troot;
 	int		isfloat=0, isadjust=0, shortint=0, sflag=0;
 
-	memset((void*)&tdata,0,sizeof(tdata));
+	memset(&tdata,0,sizeof(tdata));
 	troot = sh.var_tree;
 	if(ntp)					/* custom declaration command added using enum */
 	{
@@ -233,7 +233,7 @@ int    b_typeset(int argc,register char *argv[],Shbltin_t *context)
 	}
 	else if(argv[0][0] != 't')		/* not <t>ypeset */
 	{
-		char **new_argv = (char **)stakalloc((argc + 2) * sizeof(char*));
+		char **new_argv = (char **)stkalloc(sh.stk, (argc + 2) * sizeof(char*));
 		error_info.id = new_argv[0] = SYSTYPESET->nvname;
 		if(argv[0][0] == 'a')		/* <a>utoload == typeset -fu */
 			new_argv[1] = "-fu";
@@ -265,7 +265,7 @@ int    b_typeset(int argc,register char *argv[],Shbltin_t *context)
 		{
 			case 'a':
 				flag |= NV_IARRAY;
-				if(opt_info.arg && *opt_info.arg!='[')
+				if(opt_info.arg && !(opt_info.arg[0]=='[' && opt_info.arg[strlen(opt_info.arg)-1]==']'))
 				{
 					opt_info.index--;
 					goto endargs;
@@ -348,7 +348,7 @@ int    b_typeset(int argc,register char *argv[],Shbltin_t *context)
 				}
 				break;
 			case 'M':
-				if((tdata.wctname = opt_info.arg) && !nv_mapchar((Namval_t*)0,tdata.wctname))
+				if((tdata.wctname = opt_info.arg) && !nv_mapchar(NULL,tdata.wctname))
 				{
 					errormsg(SH_DICT, ERROR_exit(1),e_unknownmap, tdata.wctname);
 					UNREACHABLE();
@@ -438,7 +438,7 @@ int    b_typeset(int argc,register char *argv[],Shbltin_t *context)
 			case '?':
 				errormsg(SH_DICT,ERROR_usage(0), "%s", opt_info.arg);
 				opt_info.disc = 0;
-				return(2);
+				return 2;
 		}
 	}
 endargs:
@@ -489,7 +489,7 @@ endargs:
 	}
 	if(error_info.errors)
 	{
-		errormsg(SH_DICT,ERROR_usage(2),"%s", optusage(NIL(char*)));
+		errormsg(SH_DICT,ERROR_usage(2),"%s", optusage(NULL));
 		UNREACHABLE();
 	}
 	if(sizeof(char*)<8 && tdata.argnum > SHRT_MAX)
@@ -520,7 +520,7 @@ endargs:
 		Stk_t *stkp = sh.stk;
 		int off=0,offset = stktell(stkp);
 		if(!tdata.prefix)
-			return(sh_outtype(sfstdout));
+			return sh_outtype(sfstdout);
 		sfputr(stkp,NV_CLASS,-1);
 #if SHOPT_NAMESPACE
 		if(sh.namespace)
@@ -554,7 +554,7 @@ endargs:
 		if(nv_isattr(tdata.tp,NV_TAGGED))
 		{
 			nv_offattr(tdata.tp,NV_TAGGED);
-			return(0);
+			return 0;
 		}
 	}
 	else if(tdata.aflag==0 && ntp && ntp->tp)
@@ -566,13 +566,13 @@ endargs:
 		errormsg(SH_DICT,ERROR_exit(1),e_nounattr);
 		UNREACHABLE();
 	}
-	return(setall(argv,flag,troot,&tdata));
+	return setall(argv,flag,troot,&tdata);
 }
 
 static void print_value(Sfio_t *iop, Namval_t *np, struct tdata *tp)
 {
-	char	 *name;
-	int	aflag=tp->aflag;
+	char		 *name;
+	int		aflag=tp->aflag;
 	Namval_t	*table;
 	if(nv_isnull(np))
 	{
@@ -641,9 +641,9 @@ static void print_value(Sfio_t *iop, Namval_t *np, struct tdata *tp)
 	}
 }
 
-static int     setall(char **argv,register int flag,Dt_t *troot,struct tdata *tp)
+static int     setall(char **argv,int flag,Dt_t *troot,struct tdata *tp)
 {
-	register char *name;
+	char *name;
 	char *last = 0;
 	int nvflags=(flag&(NV_ARRAY|NV_NOARRAY|NV_VARNAME|NV_IDENT|NV_ASSIGN|NV_STATIC|NV_MOVE));
 	int r=0, ref=0, comvar=(flag&NV_COMVAR),iarray=(flag&NV_IARRAY);
@@ -655,7 +655,7 @@ static int     setall(char **argv,register int flag,Dt_t *troot,struct tdata *tp
 		troot = sh.var_tree = sh.var_base;
 #if SHOPT_NAMESPACE
 		save_namespace = sh.namespace;
-		sh.namespace = NIL(Namval_t*);
+		sh.namespace = NULL;
 #endif
 	}
 	if(!sh.prefix)
@@ -681,8 +681,8 @@ static int     setall(char **argv,register int flag,Dt_t *troot,struct tdata *tp
 			nvflags |= (NV_NOREF|NV_NOADD|NV_NOFAIL);
 		while(name = *++argv)
 		{
-			register unsigned newflag;
-			register Namval_t *np;
+			unsigned newflag;
+			Namval_t *np;
 			Namarr_t	*ap=0;
 			Namval_t	*mp;
 			unsigned curflag;
@@ -712,7 +712,7 @@ static int     setall(char **argv,register int flag,Dt_t *troot,struct tdata *tp
 				{
 					if(sh.prefix)
 					{
-						sfprintf(sh.strbuf,"%s.%s%c",sh.prefix,name,0);
+						sfprintf(sh.strbuf,"%s.%s",sh.prefix,name);
 						name = sfstruse(sh.strbuf);
 					}
 #if SHOPT_NAMESPACE
@@ -765,8 +765,8 @@ static int     setall(char **argv,register int flag,Dt_t *troot,struct tdata *tp
 			/* tracked alias */
 			if(troot==sh.track_tree && tp->aflag=='-')
 			{
-				np = nv_search(name,troot,NV_ADD|NV_NOSCOPE);
-				path_alias(np,path_absolute(nv_name(np),NIL(Pathcomp_t*),0));
+				sh_offstate(SH_DEFPATH);  /* 'command -p hash foo' should work to create 'foo=/bin/foo' */
+				path_settrackedalias(name,path_absolute(name,NULL,0));
 				continue;
 			}
 			if(troot==sh.alias_tree && sh.subshell && !sh.subshare && strchr(name,'='))
@@ -849,7 +849,7 @@ static int     setall(char **argv,register int flag,Dt_t *troot,struct tdata *tp
 					{
 						if(ap && comvar)
 							ap->nelem |= ARRAY_TREE;
-						nv_putsub(np, (char*)0, 0);
+						nv_putsub(np, NULL, 0);
 					}
 				}
 				else if(nvflags&NV_ARRAY)
@@ -904,7 +904,7 @@ static int     setall(char **argv,register int flag,Dt_t *troot,struct tdata *tp
 						{
 							nv_disc(np,fp,NV_POP);
 							if(!(fp->nofree&1))
-								free((void*)fp);
+								free(fp);
 							nv_offattr(np,flag&(NV_LTOU|NV_UTOL));
 						}
 					}
@@ -939,7 +939,7 @@ static int     setall(char **argv,register int flag,Dt_t *troot,struct tdata *tp
 			if (tp->aflag && (tp->argnum || (curflag!=newflag)))
 			{
 				if(np==SH_LEVELNOD)
-					return(r);
+					return r;
 				if(sh.subshell)
 					sh_assignok(np,1);
 				if(troot!=sh.var_tree)
@@ -1038,7 +1038,7 @@ static int     setall(char **argv,register int flag,Dt_t *troot,struct tdata *tp
 		sh.namespace = save_namespace;
 #endif
 	}
-	return(r);
+	return r;
 }
 
 #if SHOPT_DYNAMIC
@@ -1047,7 +1047,7 @@ typedef void (*Libinit_f)(int,void*);
 
 #define GROWLIB	4
 
-Libcomp_t	*liblist;
+Libcomp_t		*liblist;
 static int		nlib;
 static int		maxlib;
 
@@ -1059,8 +1059,8 @@ static int		maxlib;
  */
 int sh_addlib(void* dll, char* name, Pathcomp_t* pp)
 {
-	register int	n;
-	register int	r;
+	int		n;
+	int		r;
 	Libinit_f	initfn;
 	Shbltin_t	*sp = &sh.bltindata;
 
@@ -1096,7 +1096,7 @@ int sh_addlib(void* dll, char* name, Pathcomp_t* pp)
 
 Shbltin_f sh_getlib(char* sym, Pathcomp_t* pp)
 {
-	register int	n;
+	int	n;
 
 	for (n = 0; n < nlib; n++)
 		if (liblist[n].ino == pp->ino && liblist[n].dev == pp->dev)
@@ -1112,9 +1112,9 @@ Shbltin_f sh_getlib(char* sym, Pathcomp_t* pp)
  */
 int	b_builtin(int argc,char *argv[],Shbltin_t *context)
 {
-	register char *arg=0, *name;
-	register int n, r=0, flag=0;
-	register Namval_t *np;
+	char *arg=0, *name;
+	int n, r=0, flag=0;
+	Namval_t *np;
 	long dlete=0;
 	struct tdata tdata;
 	Shbltin_f addr;
@@ -1127,7 +1127,7 @@ int	b_builtin(int argc,char *argv[],Shbltin_t *context)
 	memset(&tdata,0,sizeof(tdata));
 	stkp = sh.stk;
 	if(!sh.pathlist)
-		path_absolute(argv[0],NIL(Pathcomp_t*),0);
+		path_absolute(argv[0],NULL,0);
 	while (n = optget(argv,sh_optbuiltin)) switch (n)
 	{
 	    case 's':
@@ -1159,7 +1159,7 @@ int	b_builtin(int argc,char *argv[],Shbltin_t *context)
 	argv += opt_info.index;
 	if(error_info.errors)
 	{
-		errormsg(SH_DICT,ERROR_usage(2),"%s", optusage(NIL(char*)));
+		errormsg(SH_DICT,ERROR_usage(2),"%s", optusage(NULL));
 		UNREACHABLE();
 	}
 	if(arg || *argv)
@@ -1178,21 +1178,21 @@ int	b_builtin(int argc,char *argv[],Shbltin_t *context)
 		unsigned long ver;
 		char path[PATH_MAX];
 		void *library;
-		if(!(library = dllplugin(SH_ID, arg, NiL, SH_PLUGIN_VERSION, &ver, RTLD_LAZY, path, sizeof(path))))
+		if(!(library = dllplugin(SH_ID, arg, NULL, SH_PLUGIN_VERSION, &ver, RTLD_LAZY, path, sizeof(path))))
 		{
 			errormsg(SH_DICT,ERROR_exit(0),"%s: %s",arg,dllerror(0));
-			return(1);
+			return 1;
 		}
 		if(list)
 			sfprintf(sfstdout, "%s %08lu %s\n", arg, ver, path);
-		sh_addlib(library,arg,NiL);
+		sh_addlib(library,arg,NULL);
 	}
 	else
 #endif /* SHOPT_DYNAMIC */
 	if(*argv==0 && !dlete)
 	{
 		print_scan(sfstdout, flag, sh.bltin_tree, 1, &tdata);
-		return(0);
+		return 0;
 	}
 	r = 0;
 	flag = stktell(stkp);
@@ -1235,7 +1235,7 @@ int	b_builtin(int argc,char *argv[],Shbltin_t *context)
 				errmsg = "restricted name";
 			addr = funptr(np);
 		}
-		if(!dlete && !addr && !(np=sh_addbuiltin(arg,(Shbltin_f)0 ,0)))
+		if(!dlete && !addr && !(np=sh_addbuiltin(arg,NULL,0)))
 			errmsg = "not found";
 		if(errmsg)
 		{
@@ -1245,10 +1245,10 @@ int	b_builtin(int argc,char *argv[],Shbltin_t *context)
 		stkseek(stkp,flag);
 		argv++;
 	}
-	return(r);
+	return r;
 }
 
-int    b_set(int argc,register char *argv[],Shbltin_t *context)
+int    b_set(int argc,char *argv[],Shbltin_t *context)
 {
 	struct tdata tdata;
 	memset(&tdata,0,sizeof(tdata));
@@ -1256,7 +1256,7 @@ int    b_set(int argc,register char *argv[],Shbltin_t *context)
 	if(argv[1])
 	{
 		if(sh_argopts(argc,argv) < 0)
-			return(2);
+			return 2;
 		if(sh_isoption(SH_VERBOSE))
 			sh_onstate(SH_VERBOSE);
 		else
@@ -1265,7 +1265,7 @@ int    b_set(int argc,register char *argv[],Shbltin_t *context)
 	else
 		/* scan name chain and print */
 		print_scan(sfstdout,0,sh.var_tree,0,&tdata);
-	return(0);
+	return 0;
 }
 
 /*
@@ -1275,22 +1275,22 @@ int    b_set(int argc,register char *argv[],Shbltin_t *context)
  * Non-existent items being deleted give non-zero exit status
  */
 
-int    b_unalias(int argc,register char *argv[],Shbltin_t *context)
+int    b_unalias(int argc,char *argv[],Shbltin_t *context)
 {
 	NOT_USED(context);
-	return(unall(argc,argv,sh.alias_tree));
+	return unall(argc,argv,sh.alias_tree);
 }
 
-int    b_unset(int argc,register char *argv[],Shbltin_t *context)
+int    b_unset(int argc,char *argv[],Shbltin_t *context)
 {
 	NOT_USED(context);
-	return(unall(argc,argv,sh.var_tree));
+	return unall(argc,argv,sh.var_tree);
 }
 
-static int unall(int argc, char **argv, register Dt_t *troot)
+static int unall(int argc, char **argv, Dt_t *troot)
 {
-	register Namval_t *np;
-	register const char *name;
+	Namval_t *np;
+	const char *name;
 	volatile int r;
 	Dt_t	*dp;
 	int nflag=0,all=0,isfun,jmpval;
@@ -1319,16 +1319,16 @@ static int unall(int argc, char **argv, register Dt_t *troot)
 			break;
 		case '?':
 			errormsg(SH_DICT,ERROR_usage(0), "%s", opt_info.arg);
-			return(2);
+			return 2;
 	}
 	argv += opt_info.index;
 	if(error_info.errors || (*argv==0 &&!all))
 	{
-		errormsg(SH_DICT,ERROR_usage(2),"%s",optusage(NIL(char*)));
+		errormsg(SH_DICT,ERROR_usage(2),"%s",optusage(NULL));
 		UNREACHABLE();
 	}
 	if(!troot)
-		return(1);
+		return 1;
 	r = 0;
 	if(troot==sh.var_tree)
 		nflag |= NV_VARNAME;
@@ -1342,7 +1342,7 @@ static int unall(int argc, char **argv, register Dt_t *troot)
 				sh_subfork();	/* avoid affecting the parent shell's alias table */
 			dtclear(troot);
 		}
-		return(r);
+		return r;
 	}
 	while(name = *argv++)
 	{
@@ -1352,9 +1352,13 @@ static int unall(int argc, char **argv, register Dt_t *troot)
 		if(jmpval==0)
 		{
 #if SHOPT_NAMESPACE
-			if(sh.namespace && troot!=sh.var_tree)
-				np = sh_fsearch(name,nflag);
-			if(!np)
+			if(sh.namespace && troot==sh.fun_tree && *name!='.')
+			{
+				/* prefix the namespace name */
+				sfputr(sh.stk,nv_name(sh.namespace),'.');
+				sfputr(sh.stk,name,'\0');
+				name = stkfreeze(sh.stk,0);
+			}
 #endif /* SHOPT_NAMESPACE */
 			np=nv_open(name,troot,NV_NOADD|nflag);
 		}
@@ -1402,7 +1406,7 @@ static int unall(int argc, char **argv, register Dt_t *troot)
 			else if(isfun)
 			{
 				if(troot!=sh.fun_base)
-					nv_offattr(np,NV_FUNCTION);	/* invalidate */
+					np->nvflag = 0;	/* invalidate */
 				else if(!(np->nvalue.rp && np->nvalue.rp->running))
 					nv_delete(np,troot,0);
 			}
@@ -1417,37 +1421,37 @@ static int unall(int argc, char **argv, register Dt_t *troot)
 		}
 		else if(troot==sh.alias_tree)
 			r = 1;
-		else if(troot==sh.fun_tree && troot!=sh.fun_base && nv_search(name,sh.fun_tree,0))
+		else if(troot==sh.fun_tree && troot!=sh.fun_base && (np=nv_search(name,sh.fun_tree,0)) && is_afunction(np))
 			nv_open(name,troot,NV_NOSCOPE);	/* create dummy virtual subshell node without NV_FUNCTION attribute */
 	}
-	return(r);
+	return r;
 }
 
 /*
  * print out the name and value of a name-value pair <np>
  */
-static int print_namval(Sfio_t *file,register Namval_t *np,register int flag, struct tdata *tp)
+static int print_namval(Sfio_t *file,Namval_t *np,int flag, struct tdata *tp)
 {
-	register char *cp;
+	char	*cp;
 	int	indent=tp->indent, outname=0, isfun;
 	char	tempexport=0;
 	sh_sigcheck();
 	if(flag)
 		flag = '\n';
 	if(tp->noref && nv_isref(np))
-		return(0);
+		return 0;
 	if(sh.shcomp)
-		return(1);  /* print nothing while shcomp is compiling */
+		return 1;  /* print nothing while shcomp is compiling */
 	if(nv_isattr(np,NV_NOPRINT|NV_INTEGER)==NV_NOPRINT)
 	{
 		if(is_abuiltin(np))
 			sfputr(file,nv_name(np),'\n');
-		return(0);
+		return 0;
 	}
 	if(nv_istable(np))
 	{
 		print_value(file,np,tp);
-		return(0);
+		return 0;
 	}
 	isfun = is_afunction(np);
 	if(tp->prefix)
@@ -1473,7 +1477,7 @@ static int print_namval(Sfio_t *file,register Namval_t *np,register int flag, st
 		Sfio_t *iop=0;
 		char *fname=0;
 		if(nv_isattr(np,NV_NOFREE))
-			return(0);
+			return 0;
 		if(!flag && !np->nvalue.ip)
 			sfputr(file,"typeset -fu",' ');
 		else if(!flag && !nv_isattr(np,NV_FPOSIX))
@@ -1513,14 +1517,14 @@ static int print_namval(Sfio_t *file,register Namval_t *np,register int flag, st
 			if(fname)
 				sfclose(iop);
 		}
-		return(nv_size(np)+1);
+		return nv_size(np) + 1;
 	}
 	if(nv_arrayptr(np))
 	{
 		if(indent)
 			sfnputc(file,'\t',indent);
 		print_value(file,np,tp);
-		return(0);
+		return 0;
 	}
 	if(nv_isvtree(np) && !nv_isattr(np,NV_EXPORT))
 	{
@@ -1552,11 +1556,11 @@ static int print_namval(Sfio_t *file,register Namval_t *np,register int flag, st
 			else
 				sfputr(file,nv_isvtree(np)?cp:sh_fmtq(cp),'\n');
 		}
-		return(1);
+		return 1;
 	}
 	else if(outname || (tp->scanmask && tp->scanroot==sh.var_tree))
 		sfputr(file,nv_name(np),'\n');
-	return(0);
+	return 0;
 }
 
 /*
@@ -1565,15 +1569,15 @@ static int print_namval(Sfio_t *file,register Namval_t *np,register int flag, st
 static void	print_all(Sfio_t *file,Dt_t *root, struct tdata *tp)
 {
 	tp->outfile = file;
-	nv_scan(root, print_attribute, (void*)tp, 0, 0);
+	nv_scan(root, print_attribute, tp, 0, 0);
 }
 
 /*
  * print the attributes of name value pair give by <np>
  */
-static void	print_attribute(register Namval_t *np,void *data)
+static void	print_attribute(Namval_t *np,void *data)
 {
-	register struct tdata *dp = (struct tdata*)data;
+	struct tdata *dp = (struct tdata*)data;
 	nv_attribute(np,dp->outfile,dp->prefix,dp->aflag);
 }
 
@@ -1583,9 +1587,9 @@ static void	print_attribute(register Namval_t *np,void *data)
  */
 static void print_scan(Sfio_t *file, int flag, Dt_t *root, int option,struct tdata *tp)
 {
-	register char **argv;
-	register Namval_t *np;
-	register int namec;
+	char **argv;
+	Namval_t *np;
+	int namec;
 	Namval_t *onp = 0;
 	char	*name=0;
 	size_t	len;
@@ -1600,9 +1604,9 @@ static void print_scan(Sfio_t *file, int flag, Dt_t *root, int option,struct tda
 		tp->scanmask |= (NV_DOUBLE|NV_EXPNOTE);
 	if(flag==NV_LTOU || flag==NV_UTOL)
 		tp->scanmask |= NV_UTOL|NV_LTOU;
-	namec = nv_scan(root, nullscan, (void*)tp, tp->scanmask, flag&~NV_IARRAY);
+	namec = nv_scan(root, nullscan, tp, tp->scanmask, flag&~NV_IARRAY);
 	argv = tp->argnam  = (char**)stkalloc(sh.stk,(namec+1)*sizeof(char*));
-	namec = nv_scan(root, pushname, (void*)tp, tp->scanmask, flag&~NV_IARRAY);
+	namec = nv_scan(root, pushname, tp, tp->scanmask, flag&~NV_IARRAY);
 	if(mbcoll())
 		strsort(argv,namec,strcoll);
 	if(namec==0 && sh.namespace && nv_dict(sh.namespace)==root)
