@@ -2,7 +2,7 @@
 *                                                                      *
 *               This software is part of the ast package               *
 *          Copyright (c) 1982-2014 AT&T Intellectual Property          *
-*          Copyright (c) 2020-2022 Contributors to ksh 93u+m           *
+*          Copyright (c) 2020-2023 Contributors to ksh 93u+m           *
 *                      and is licensed under the                       *
 *                 Eclipse Public License, Version 2.0                  *
 *                                                                      *
@@ -14,6 +14,7 @@
 *                  Martijn Dekker <martijn@inlv.org>                   *
 *            Johnothan King <johnothanking@protonmail.com>             *
 *               Anuradha Weeraman <anuradha@debian.org>                *
+*               K. Eugene Carlson <kvngncrlsn@gmail.com>               *
 *                                                                      *
 ***********************************************************************/
 /*
@@ -30,9 +31,7 @@
 #include	<errno.h>
 #include	<ccode.h>
 #include	<fault.h>
-#include	"FEATURE/options"
 #include	"FEATURE/time"
-#include	"FEATURE/cmds"
 #ifdef _hdr_utime
 #   include	<utime.h>
 #   include	<ls.h>
@@ -46,17 +45,8 @@
 #include	"edit.h"
 #include	"shlex.h"
 
-static char *CURSOR_UP = Empty;  /* move cursor up one line */
-static char *ERASE_EOS = Empty;  /* erase to end of screen */
-#if _tput_terminfo
-#define TPUT_CURSOR_UP	"cuu1"
-#define TPUT_ERASE_EOS	"ed"
-#elif _tput_termcap
-#define TPUT_CURSOR_UP	"up"
-#define TPUT_ERASE_EOS	"cd"
-#else
-#undef _pth_tput
-#endif /* _tput_terminfo */
+static char *cursor_up;  /* move cursor up one line */
+static char *erase_eos;  /* erase to end of screen */
 
 #if SHOPT_MULTIBYTE
 #   define is_cntrl(c)	((c<=STRIP) && iscntrl(c))
@@ -75,42 +65,41 @@ static char *ERASE_EOS = Empty;  /* erase to end of screen */
     {
 	switch(c)
 	{
-	    case cntl('A'): return('A');
-	    case cntl('B'): return('B');
-	    case cntl('C'): return('C');
-	    case cntl('D'): return('D');
-	    case cntl('E'): return('E');
-	    case cntl('F'): return('F');
-	    case cntl('G'): return('G');
-	    case cntl('H'): return('H');
-	    case cntl('I'): return('I');
-	    case cntl('J'): return('J');
-	    case cntl('K'): return('K');
-	    case cntl('L'): return('L');
-	    case cntl('M'): return('M');
-	    case cntl('N'): return('N');
-	    case cntl('O'): return('O');
-	    case cntl('P'): return('P');
-	    case cntl('Q'): return('Q');
-	    case cntl('R'): return('R');
-	    case cntl('S'): return('S');
-	    case cntl('T'): return('T');
-	    case cntl('U'): return('U');
-	    case cntl('V'): return('V');
-	    case cntl('W'): return('W');
-	    case cntl('X'): return('X');
-	    case cntl('Y'): return('Y');
-	    case cntl('Z'): return('Z');
-	    case cntl(']'): return(']');
-	    case cntl('['): return('[');
+	    case cntl('A'): return 'A';
+	    case cntl('B'): return 'B';
+	    case cntl('C'): return 'C';
+	    case cntl('D'): return 'D';
+	    case cntl('E'): return 'E';
+	    case cntl('F'): return 'F';
+	    case cntl('G'): return 'G';
+	    case cntl('H'): return 'H';
+	    case cntl('I'): return 'I';
+	    case cntl('J'): return 'J';
+	    case cntl('K'): return 'K';
+	    case cntl('L'): return 'L';
+	    case cntl('M'): return 'M';
+	    case cntl('N'): return 'N';
+	    case cntl('O'): return 'O';
+	    case cntl('P'): return 'P';
+	    case cntl('Q'): return 'Q';
+	    case cntl('R'): return 'R';
+	    case cntl('S'): return 'S';
+	    case cntl('T'): return 'T';
+	    case cntl('U'): return 'U';
+	    case cntl('V'): return 'V';
+	    case cntl('W'): return 'W';
+	    case cntl('X'): return 'X';
+	    case cntl('Y'): return 'Y';
+	    case cntl('Z'): return 'Z';
+	    case cntl(']'): return ']';
+	    case cntl('['): return '[';
 	}
-	return('?');
+	return '?';
     }
 #endif
 #define MINWINDOW	15	/* minimum width window */
 #define DFLTWINDOW	80	/* default window width */
 #define RAWMODE		1
-#define ALTMODE		2
 #define ECHOMODE	3
 #define	SYSERR	-1
 
@@ -153,14 +142,14 @@ static const char bellchr[] = "\a";	/* bell char */
  */
 int tty_check(int fd)
 {
-	register Edit_t *ep = (Edit_t*)(sh.ed_context);
+	Edit_t *ep = (Edit_t*)(sh.ed_context);
 	struct termios tty;
 	Sfio_t *sp;
 	ep->e_savefd = -1;
 	if(fd < 0 || fd > sh.lim.open_max || sh.fdstatus[fd] == IOCLOSE
 	|| (sp = sh.sftable[fd]) && (sfset(sp,0,0) & SF_STRING))
-		return(0);
-	return(tty_get(fd,&tty)==0);
+		return 0;
+	return tty_get(fd,&tty)==0;
 }
 
 /*
@@ -169,9 +158,9 @@ int tty_check(int fd)
  *   is called again without an intervening tty_set()
  */
 
-int tty_get(register int fd, register struct termios *tty)
+int tty_get(int fd, struct termios *tty)
 {
-	register Edit_t *ep = (Edit_t*)(sh.ed_context);
+	Edit_t *ep = (Edit_t*)(sh.ed_context);
 	if(fd == ep->e_savefd)
 		*tty = ep->e_savetty;
 	else
@@ -179,7 +168,7 @@ int tty_get(register int fd, register struct termios *tty)
 		while(tcgetattr(fd,tty) == SYSERR)
 		{
 			if(errno !=EINTR)
-				return(SYSERR);
+				return SYSERR;
 			errno = 0;
 		}
 		/* save terminal settings if in canonical state */
@@ -189,7 +178,7 @@ int tty_get(register int fd, register struct termios *tty)
 			ep->e_savefd = fd;
 		}
 	}
-	return(0);
+	return 0;
 }
 
 /*
@@ -199,19 +188,19 @@ int tty_get(register int fd, register struct termios *tty)
 
 int tty_set(int fd, int action, struct termios *tty)
 {
-	register Edit_t *ep = (Edit_t*)(sh.ed_context);
+	Edit_t *ep = (Edit_t*)(sh.ed_context);
 	if(fd >=0)
 	{
 		while(tcsetattr(fd, action, tty) == SYSERR)
 		{
 			if(errno !=EINTR)
-				return(SYSERR);
+				return SYSERR;
 			errno = 0;
 		}
 		ep->e_savetty = *tty;
 	}
 	ep->e_savefd = fd;
-	return(0);
+	return 0;
 }
 
 /*{	TTY_COOKED( fd )
@@ -221,9 +210,9 @@ int tty_set(int fd, int action, struct termios *tty)
  *
 }*/
 
-void tty_cooked(register int fd)
+void tty_cooked(int fd)
 {
-	register Edit_t *ep = (Edit_t*)(sh.ed_context);
+	Edit_t *ep = (Edit_t*)(sh.ed_context);
 	ep->e_keytrap = 0;
 	if(ep->e_raw==0)
 		return;
@@ -254,31 +243,26 @@ void tty_cooked(register int fd)
  *
 }*/
 
-int tty_raw(register int fd, int echomode)
+int tty_raw(int fd, int echomode)
 {
 	int echo = echomode;
 #ifdef L_MASK
 	struct ltchars lchars;
 #endif	/* L_MASK */
-	register Edit_t *ep = (Edit_t*)(sh.ed_context);
+	Edit_t *ep = (Edit_t*)(sh.ed_context);
 	if(ep->e_raw==RAWMODE)
-		return(echo?-1:0);
+		return echo?-1:0;
 	else if(ep->e_raw==ECHOMODE)
-		return(echo?0:-1);
-#if !SHOPT_RAWONLY
-	if(ep->e_raw != ALTMODE)
-#endif /* SHOPT_RAWONLY */
-	{
-		if(tty_get(fd,&ttyparm) == SYSERR)
-			return(-1);
-	}
+		return echo?0:-1;
+	if(tty_get(fd,&ttyparm) == SYSERR)
+		return -1;
 #if  L_MASK || VENIX
 	if(ttyparm.sg_flags&LCASE)
-		return(-1);
+		return -1;
 	if(!(ttyparm.sg_flags&ECHO))
 	{
 		if(!echomode)
-			return(-1);
+			return -1;
 		echo = 0;
 	}
 	nttyparm = ttyparm;
@@ -295,7 +279,7 @@ int tty_raw(register int fd, int echomode)
 	ep->e_werase = cntl('W');
 	ep->e_lnext = cntl('V');
 	if( tty_set(fd, TCSADRAIN, &nttyparm) == SYSERR )
-		return(-1);
+		return -1;
 	ep->e_ttyspeed = (ttyparm.sg_ospeed>=B1200?FAST:SLOW);
 #   ifdef TIOCGLTC
 	/* try to remove effect of ^V and ^Y and ^O */
@@ -313,7 +297,7 @@ int tty_raw(register int fd, int echomode)
 	if (!(ttyparm.c_lflag & ECHO ))
 	{
 		if(!echomode)
-			return(-1);
+			return -1;
 		echo = 0;
 	}
 #   ifdef FLUSHO
@@ -366,138 +350,13 @@ int tty_raw(register int fd, int echomode)
 	ep->e_erase = ttyparm.c_cc[VERASE];
 	ep->e_kill = ttyparm.c_cc[VKILL];
 	if( tty_set(fd, TCSADRAIN, &nttyparm) == SYSERR )
-		return(-1);
+		return -1;
 	ep->e_ttyspeed = (cfgetospeed(&ttyparm)>=B1200?FAST:SLOW);
 #endif
 	ep->e_raw = (echomode?ECHOMODE:RAWMODE);
-	return(0);
+	return 0;
 }
 
-#if !SHOPT_RAWONLY
-
-/*
- *
- *	Get tty parameters and make ESC and '\r' wakeup characters.
- *
- */
-
-#   ifdef TIOCGETC
-int tty_alt(register int fd)
-{
-	register Edit_t *ep = (Edit_t*)(sh.ed_context);
-	int mask;
-	struct tchars ttychars;
-	switch(ep->e_raw)
-	{
-	    case ECHOMODE:
-		return(-1);
-	    case ALTMODE:
-		return(0);
-	    case RAWMODE:
-		tty_cooked(fd);
-	}
-	l_changed = 0;
-	if( ep->e_ttyspeed == 0)
-	{
-		if((tty_get(fd,&ttyparm) != SYSERR))
-			ep->e_ttyspeed = (ttyparm.sg_ospeed>=B1200?FAST:SLOW);
-		ep->e_raw = ALTMODE;
-	}
-	if(ioctl(fd,TIOCGETC,&l_ttychars) == SYSERR)
-		return(-1);
-	if(ioctl(fd,TIOCLGET,&l_mask)==SYSERR)
-		return(-1);
-	ttychars = l_ttychars;
-	mask =  LCRTBS|LCRTERA|LCTLECH|LPENDIN|LCRTKIL;
-	if((l_mask|mask) != l_mask)
-		l_changed = L_MASK;
-	if(ioctl(fd,TIOCLBIS,&mask)==SYSERR)
-		return(-1);
-	if(ttychars.t_brkc!=ESC)
-	{
-		ttychars.t_brkc = ESC;
-		l_changed |= T_CHARS;
-		if(ioctl(fd,TIOCSETC,&ttychars) == SYSERR)
-			return(-1);
-	}
-	return(0);
-}
-#   else
-#	ifndef PENDIN
-#	    define PENDIN	0
-#	endif /* PENDIN */
-#	ifndef IEXTEN
-#	    define IEXTEN	0
-#	endif /* IEXTEN */
-
-int tty_alt(register int fd)
-{
-	register Edit_t *ep = (Edit_t*)(sh.ed_context);
-	switch(ep->e_raw)
-	{
-	    case ECHOMODE:
-		return(-1);
-	    case ALTMODE:
-		return(0);
-	    case RAWMODE:
-		tty_cooked(fd);
-	}
-	if((tty_get(fd, &ttyparm)==SYSERR) || (!(ttyparm.c_lflag&ECHO)))
-		return(-1);
-#	ifdef FLUSHO
-	    ttyparm.c_lflag &= ~FLUSHO;
-#	endif /* FLUSHO */
-	nttyparm = ttyparm;
-	ep->e_eof = ttyparm.c_cc[VEOF];
-#	ifdef ECHOCTL
-	    /* escape character echos as ^[ */
-	    nttyparm.c_lflag |= (ECHOE|ECHOK|ECHOCTL|PENDIN|IEXTEN);
-	    nttyparm.c_cc[VEOL] = ESC;
-#	else
-	    /* switch VEOL2 and EOF, since EOF isn't echo'd by driver */
-	    nttyparm.c_lflag |= (ECHOE|ECHOK);
-	    nttyparm.c_cc[VEOF] = ESC;	/* make ESC the eof char */
-#	    ifdef VEOL2
-		nttyparm.c_iflag &= ~(IGNCR|ICRNL);
-		nttyparm.c_iflag |= INLCR;
-		nttyparm.c_cc[VEOL] = '\r';	/* make CR an eol char */
-		nttyparm.c_cc[VEOL2] = ep->e_eof; /* make EOF an eol char */
-#	    else
-		nttyparm.c_cc[VEOL] = ep->e_eof; /* make EOF an eol char */
-#	    endif /* VEOL2 */
-#	endif /* ECHOCTL */
-#	ifdef VREPRINT
-		nttyparm.c_cc[VREPRINT] = _POSIX_DISABLE;
-#	endif /* VREPRINT */
-#	ifdef VDISCARD
-		nttyparm.c_cc[VDISCARD] = _POSIX_DISABLE;
-#	endif /* VDISCARD */
-#	ifdef VWERASE
-	    if(ttyparm.c_cc[VWERASE] == _POSIX_DISABLE)
-		    nttyparm.c_cc[VWERASE] = cntl('W');
-	    ep->e_werase = nttyparm.c_cc[VWERASE];
-#	else
-	    ep->e_werase = cntl('W');
-#	endif /* VWERASE */
-#	ifdef VLNEXT
-	    if(ttyparm.c_cc[VLNEXT] == _POSIX_DISABLE )
-		    nttyparm.c_cc[VLNEXT] = cntl('V');
-	    ep->e_lnext = nttyparm.c_cc[VLNEXT];
-#	else
-	    ep->e_lnext = cntl('V');
-#	endif /* VLNEXT */
-	ep->e_erase = ttyparm.c_cc[VERASE];
-	ep->e_kill = ttyparm.c_cc[VKILL];
-	if( tty_set(fd, TCSADRAIN, &nttyparm) == SYSERR )
-		return(-1);
-	ep->e_ttyspeed = (cfgetospeed(&ttyparm)>=B1200?FAST:SLOW);
-	ep->e_raw = ALTMODE;
-	return(0);
-}
-
-#   endif /* TIOCGETC */
-#endif	/* SHOPT_RAWONLY */
-
 /*
  *	ED_WINDOW()
  *
@@ -506,14 +365,14 @@ int tty_alt(register int fd)
 int ed_window(void)
 {
 	int	cols;
-	sh_winsize(NIL(int*),&cols);
+	sh_winsize(NULL,&cols);
 	if(--cols < 0)
 		cols = DFLTWINDOW - 1;
 	else if(cols < MINWINDOW)
 		cols = MINWINDOW;
 	else if(cols > MAXWINDOW)
 		cols = MAXWINDOW;
-	return(cols);
+	return cols;
 }
 
 /*	E_FLUSH()
@@ -524,8 +383,8 @@ int ed_window(void)
 
 void ed_flush(Edit_t *ep)
 {
-	register int n = ep->e_outptr-ep->e_outbase;
-	register int fd = ERRIO;
+	int n = ep->e_outptr-ep->e_outbase;
+	int fd = ERRIO;
 	if(n<=0)
 		return;
 	write(fd,ep->e_outbase,(unsigned)n);
@@ -542,11 +401,7 @@ void ed_ringbell(void)
 }
 
 #if SHOPT_ESH || SHOPT_VSH
-/*
- * send a carriage return line feed to the terminal
- */
 
-#ifdef _pth_tput
 /*
  * Get or update a tput (terminfo or termcap) capability string.
  */
@@ -558,19 +413,24 @@ static void get_tput(char *tp, char **cpp)
 	sh_offoption(SH_RESTRICTED);
 	sh_offoption(SH_VERBOSE);
 	sh_offoption(SH_XTRACE);
-	sfprintf(sh.strbuf,".sh.value=$(" _pth_tput " %s 2>/dev/null)",tp);
+	sfprintf(sh.strbuf,".sh.value=${ \\command -p tput %s 2>/dev/null;}",tp);
 	sh_trap(sfstruse(sh.strbuf),0);
 	if((cp = nv_getval(SH_VALNOD)) && (!*cpp || strcmp(cp,*cpp)!=0))
 	{
-		if(*cpp && *cpp!=Empty)
+		if(*cpp)
 			free(*cpp);
-		*cpp = *cp ? sh_strdup(cp) : Empty;
+		*cpp = *cp ? sh_strdup(cp) : NULL;
+	}
+	else
+	{
+		if(*cpp)
+			free(*cpp);
+		*cpp = NULL;
 	}
 	nv_unset(SH_VALNOD);
 	sh.options = o;
 	sigrelease(SIGINT);
 }
-#endif /* _pth_tput */
 
 /*	ED_SETUP( max_prompt_size )
  *
@@ -584,14 +444,14 @@ static void get_tput(char *tp, char **cpp)
  *	    are not counted as part of the prompt length.
  */
 
-void	ed_setup(register Edit_t *ep, int fd, int reedit)
+void	ed_setup(Edit_t *ep, int fd, int reedit)
 {
-	register char *pp;
-	register char *last, *prev;
+	char *pp;
+	char *last, *prev;
 	char *ppmax;
 	int myquote = 0;
 	size_t n;
-	register int qlen = 1, qwid;
+	int qlen = 1, qwid;
 	char inquote = 0;
 	ep->e_fd = fd;
 	ep->e_multiline = sh_editor_active() && sh_isoption(SH_MULTILINE);
@@ -601,14 +461,17 @@ void	ed_setup(register Edit_t *ep, int fd, int reedit)
 	ep->nhlist = 0;
 	ep->hoff = 0;
 #endif /* SHOPT_EDPREDICT */
-	ep->e_stkoff = staktell();
-	ep->e_stkptr = stakfreeze(0);
+	ep->e_stkoff = stktell(sh.stk);
+	ep->e_stkptr = stkfreeze(sh.stk,0);
+#if SHOPT_MULTIBYTE
+	ep->e_savedwidth = 0;
+#endif /* SHOPT_MULTIBYTE */
 	if(!(last = sh.prompt))
 		last = "";
 	sh.prompt = 0;
 	if(sh.hist_ptr)
 	{
-		register History_t *hp = sh.hist_ptr;
+		History_t *hp = sh.hist_ptr;
 		ep->e_hismax = hist_max(hp);
 		ep->e_hismin = hist_min(hp);
 	}
@@ -629,7 +492,7 @@ void	ed_setup(register Edit_t *ep, int fd, int reedit)
 	ppmax = pp+PRSIZE-1;
 	*pp++ = '\r';
 	{
-		register int c;
+		int c;
 		while(prev = last, c = mbchar(last)) switch(c)
 		{
 			case ESC:
@@ -746,17 +609,41 @@ void	ed_setup(register Edit_t *ep, int fd, int reedit)
 	if(pp-ep->e_prompt > qlen)
 		ep->e_plen = pp - ep->e_prompt - qlen;
 	*pp = 0;
-#if SHOPT_ESH || SHOPT_VSH
+	if(ep->e_multiline)
+	{
+		static char *oldterm;
+		Namval_t *np = nv_search("TERM",sh.var_tree,0);
+		char *term = NULL;
+		if(nv_isattr(np,NV_EXPORT))
+			term = nv_getval(np);
+		if(!term)
+			term = "";
+		if(!oldterm || strcmp(term,oldterm))
+		{
+			get_tput(TINF_CURSOR_UP,&cursor_up);
+			get_tput(TINF_ERASE_EOS,&erase_eos);
+			if(!cursor_up)
+				get_tput(TCAP_CURSOR_UP,&cursor_up);
+			if(!erase_eos)
+				get_tput(TCAP_ERASE_EOS,&erase_eos);
+			if(oldterm)
+				free(oldterm);
+			oldterm = sh_strdup(term);
+		}
+		if(cursor_up && erase_eos)
+			ep->e_wsize = MAXLINE - (ep->e_plen + 1);
+		else
+			ep->e_multiline = 0;
+	}
 	if(!ep->e_multiline && (ep->e_wsize -= ep->e_plen) < 7)
 	{
-		register int shift = 7-ep->e_wsize;
+		int shift = 7-ep->e_wsize;
 		ep->e_wsize = 7;
 		pp = ep->e_prompt+1;
 		strcopy(pp,pp+shift);
 		ep->e_plen -= shift;
 		last[-ep->e_plen-2] = '\r';
 	}
-#endif /* SHOPT_ESH || SHOPT_VSH */
 	sfsync(sfstderr);
 	if(fd == sffileno(sfstderr))
 	{
@@ -776,23 +663,6 @@ void	ed_setup(register Edit_t *ep, int fd, int reedit)
 		sfset(sfstderr,SF_READ,1);
 	sfwrite(sfstderr,ep->e_outptr,0);
 	ep->e_eol = reedit;
-#if SHOPT_ESH || SHOPT_VSH
-	if(ep->e_multiline)
-	{
-#ifdef _pth_tput
-		char *term;
-		if(!ep->e_term)
-			ep->e_term = nv_search("TERM",sh.var_tree,0);
-		if(ep->e_term && (term=nv_getval(ep->e_term)) && strlen(term)<sizeof(ep->e_termname) && strcmp(term,ep->e_termname))
-		{
-			get_tput(TPUT_CURSOR_UP,&CURSOR_UP);
-			get_tput(TPUT_ERASE_EOS,&ERASE_EOS);
-			strcopy(ep->e_termname,term);
-		}
-#endif /* _pth_tput */
-		ep->e_wsize = MAXLINE - (ep->e_plen+1);
-	}
-#endif /* SHOPT_ESH || SHOPT_VSH */
 	if(ep->e_default && (pp = nv_getval(ep->e_default)))
 	{
 		n = strlen(pp);
@@ -804,20 +674,20 @@ void	ed_setup(register Edit_t *ep, int fd, int reedit)
 		ep->e_default = 0;
 	}
 }
-#endif /* SHOPT_ESH || SHOPT_VSH */
 
-static void ed_putstring(register Edit_t *ep, const char *str)
+static void ed_putstring(Edit_t *ep, const char *str)
 {
-	register int c;
+	int c;
 	while(c = *str++)
 		ed_putchar(ep,c);
 }
 
-static void ed_nputchar(register Edit_t *ep, int n, int c)
+static void ed_nputchar(Edit_t *ep, int n, int c)
 {
 	while(n-->0)
 		ed_putchar(ep,c);
 }
+#endif /* SHOPT_ESH || SHOPT_VSH */
 
 /*
  * Do read, restart on interrupt unless SH_SIGSET or SH_SIGTRAP is set
@@ -832,14 +702,12 @@ static void ed_nputchar(register Edit_t *ep, int n, int c)
  */
 int ed_read(void *context, int fd, char *buff, int size, int reedit)
 {
-	register Edit_t *ep = (Edit_t*)context;
-	register int rv= -1;
-	register int delim = ((ep->e_raw&RAWMODE)?nttyparm.c_cc[VEOL]:'\n');
+	Edit_t *ep = (Edit_t*)context;
+	int rv= -1;
+	int delim = ((ep->e_raw&RAWMODE)?nttyparm.c_cc[VEOL]:'\n');
 	int mode = -1;
 	int (*waitevent)(int,long,int) = sh.waitevent;
 	/* sfpkrd must use select(2) to intercept SIGWINCH for ed_read */
-	if(ep->e_raw==ALTMODE)
-		mode = 2;
 	if(size < 0)
 	{
 		mode = 2;
@@ -861,7 +729,8 @@ int ed_read(void *context, int fd, char *buff, int size, int reedit)
 		{
 			int	n, newsize;
 			char	*cp;
-			sh_winsize(NIL(int*),&newsize);
+			sh_winsize(NULL,&newsize);
+			ed_putchar(ep,'\r');
 			/*
 			 * Try to move cursor to start of first line and pray it works... it's very
 			 * failure-prone if the window size changed, especially on modern terminals
@@ -869,13 +738,17 @@ int ed_read(void *context, int fd, char *buff, int size, int reedit)
 			 */
 			if(ep->e_multiline)
 			{
-				n = (ep->e_plen + ep->e_cur) / newsize;
-				while(n--)
-					ed_putstring(ep,CURSOR_UP);
+				n = (ep->e_plen + ep->e_peol) / ep->e_winsz;
+				while(n-- > 0)
+					ed_putstring(ep,cursor_up);
+				/* clear the current command line */
+				ed_putstring(ep,erase_eos);
+			}
+			else
+			{
+				ed_nputchar(ep,newsize-1,' ');
+				ed_putchar(ep,'\r');
 			}
-			ed_putchar(ep,'\r');
-			/* clear the current command line */
-			ed_putstring(ep,ERASE_EOS);
 			ed_flush(ep);
 			/* show any buffered 'set -b' job notification(s) */
 			if(sh.notifybuf && (cp = sfstruse(sh.notifybuf)) && *cp)
@@ -884,8 +757,13 @@ int ed_read(void *context, int fd, char *buff, int size, int reedit)
 			ep->e_winsz = newsize-1;
 			if(ep->e_winsz < MINWINDOW)
 				ep->e_winsz = MINWINDOW;
-			if(!ep->e_multiline && ep->e_wsize < MAXLINE)
-				ep->e_wsize = ep->e_winsz-2;
+			if(!ep->e_multiline)
+			{
+				if(ep->e_wsize < MAXLINE)
+					ep->e_wsize = ep->e_winsz-2;
+				if(ep->e_wsize > ep->e_plen)
+					ep->e_wsize -= ep->e_plen;
+			}
 			/* redraw command line */
 #if SHOPT_ESH && SHOPT_VSH
 			if(sh_isoption(SH_VI))
@@ -945,7 +823,7 @@ int ed_read(void *context, int fd, char *buff, int size, int reedit)
 done:
 	sh.waitevent = waitevent;
 	sh_offstate(SH_TTYWAIT);
-	return(rv);
+	return rv;
 }
 
 
@@ -955,9 +833,9 @@ done:
  *    onto the stack so that it can be checked for KEYTRAP
  * putstack() returns 1 except when in the middle of a multi-byte char
  */
-static int putstack(Edit_t *ep,char string[], register int nbyte, int type) 
+static int putstack(Edit_t *ep,char string[], int nbyte, int type) 
 {
-	register int c;
+	int c;
 #if SHOPT_MULTIBYTE
 	char *endp, *p=string;
 	int size, offset = ep->e_lookahead + nbyte;
@@ -1001,7 +879,7 @@ static int putstack(Edit_t *ep,char string[], register int nbyte, int type)
 					*++endp = 0;
 					goto again;
 				}
-				return(c);
+				return c;
 			}
 			else
 			{
@@ -1037,7 +915,7 @@ static int putstack(Edit_t *ep,char string[], register int nbyte, int type)
 #   endif /* CBREAK */
 	}
 #endif /* SHOPT_MULTIBYTE */
-	return(1);
+	return 1;
 }
 
 /*
@@ -1049,9 +927,9 @@ static int putstack(Edit_t *ep,char string[], register int nbyte, int type)
  *   1		edit keys not mapped
  *   2		Next key is literal
  */
-int ed_getchar(register Edit_t *ep,int mode)
+int ed_getchar(Edit_t *ep,int mode)
 {
-	register int n, c;
+	int n, c;
 	char readin[LOOKAHEAD+1];
 	if(!ep->e_lookahead)
 	{
@@ -1123,11 +1001,11 @@ int ed_getchar(register Edit_t *ep,int mode)
 	}
 	else
 		siglongjmp(ep->e_env,(n==0?UEOF:UINTR));
-	return(c);
+	return c;
 }
 
 #if SHOPT_ESH || SHOPT_VSH
-void ed_ungetchar(Edit_t *ep,register int c)
+void ed_ungetchar(Edit_t *ep,int c)
 {
 	if (ep->e_lookahead < LOOKAHEAD)
 		ep->e_lbuf[ep->e_lookahead++] = c;
@@ -1140,11 +1018,11 @@ void ed_ungetchar(Edit_t *ep,register int c)
  * put a character into the output buffer
  */
 
-void	ed_putchar(register Edit_t *ep,register int c)
+void	ed_putchar(Edit_t *ep,int c)
 {
 	char buf[8];
-	register char *dp = ep->e_outptr;
-	register int i,size=1;
+	char *dp = ep->e_outptr;
+	int i,size=1;
 	if(!dp)
 		return;
 	buf[0] = c;
@@ -1159,16 +1037,8 @@ void	ed_putchar(register Edit_t *ep,register int c)
 		c = buf[i];
 	}
 	else
-	{
 		buf[0] = c;
-		size = 1;
-	}
 #endif	/* SHOPT_MULTIBYTE */
-	if (buf[0] == '_' && size==1)
-	{
-		*dp++ = ' ';
-		*dp++ = '\b';
-	}
 	*dp++ = c;
 	*dp = '\0';
 	if(dp >= ep->e_outlast)
@@ -1185,8 +1055,8 @@ void	ed_putchar(register Edit_t *ep,register int c)
  */
 Edpos_t ed_curpos(Edit_t *ep,genchar *phys, int off, int cur, Edpos_t curpos)
 {
-	register genchar *sp=phys;
-	register int c=1, col=ep->e_plen;
+	genchar *sp=phys;
+	int c=1, col=ep->e_plen;
 	Edpos_t pos;
 #if SHOPT_MULTIBYTE
 	char p[16];
@@ -1225,15 +1095,15 @@ Edpos_t ed_curpos(Edit_t *ep,genchar *phys, int off, int cur, Edpos_t curpos)
 			pos.line++;
 	}
 	pos.col = col;
-	return(pos);
+	return pos;
 }
 #endif /* SHOPT_ESH || SHOPT_VSH */
 
 #if SHOPT_ESH || SHOPT_VSH
-int ed_setcursor(register Edit_t *ep,genchar *physical,register int old,register int new,int first)
+int ed_setcursor(Edit_t *ep,genchar *physical,int old,int new,int first)
 {
 	static int oldline;
-	register int delta;
+	int delta;
 	int clear = 0;
 	Edpos_t newpos;
 
@@ -1244,7 +1114,7 @@ int ed_setcursor(register Edit_t *ep,genchar *physical,register int old,register
 		clear = 1;
 	}
 	if( delta == 0  &&  !clear)
-		return(new);
+		return new;
 	if(ep->e_multiline)
 	{
 		ep->e_curpos = ed_curpos(ep, physical, old,0,ep->e_curpos);
@@ -1252,7 +1122,7 @@ int ed_setcursor(register Edit_t *ep,genchar *physical,register int old,register
 		{
 			ed_nputchar(ep,clear,' ');
 			ed_nputchar(ep,clear,'\b');
-			return(new);
+			return new;
 		}
 		newpos =     ed_curpos(ep, physical, new,old,ep->e_curpos);
 		if(ep->e_curpos.col==0 && ep->e_curpos.line>0 && oldline<ep->e_curpos.line && delta<0)
@@ -1262,7 +1132,7 @@ int ed_setcursor(register Edit_t *ep,genchar *physical,register int old,register
 		{
 			int n,pline,plen=ep->e_plen;
 			for(;ep->e_curpos.line > newpos.line; ep->e_curpos.line--)
-				ed_putstring(ep,CURSOR_UP);
+				ed_putstring(ep,cursor_up);
 			pline = plen/(ep->e_winsz+1);
 			if(newpos.line <= pline)
 				plen -= pline*(ep->e_winsz+1);
@@ -1285,7 +1155,7 @@ int ed_setcursor(register Edit_t *ep,genchar *physical,register int old,register
 							ed_putchar(ep,physical[m++]);
 					}
 					ed_nputchar(ep,n,' ');
-					ed_putstring(ep,CURSOR_UP);
+					ed_putstring(ep,cursor_up);
 				}
 			}
 		}
@@ -1327,7 +1197,7 @@ int ed_setcursor(register Edit_t *ep,genchar *physical,register int old,register
 	}
 	while(delta-->0)
 		ed_putchar(ep,physical[old++]);
-	return(new);
+	return new;
 }
 #endif /* SHOPT_ESH || SHOPT_VSH */
 
@@ -1337,9 +1207,9 @@ int ed_setcursor(register Edit_t *ep,genchar *physical,register int old,register
  */
 int ed_virt_to_phys(Edit_t *ep,genchar *virt,genchar *phys,int cur,int voff,int poff)
 {
-	register genchar *sp = virt;
-	register genchar *dp = phys;
-	register int c;
+	genchar *sp = virt;
+	genchar *dp = phys;
+	int c;
 	genchar *curp = sp + cur;
 	genchar *dpmax = phys+MAXLINE;
 	int d, r;
@@ -1393,7 +1263,7 @@ int ed_virt_to_phys(Edit_t *ep,genchar *virt,genchar *phys,int cur,int voff,int
 	}
 	*dp = 0;
 	ep->e_peol = dp-phys;
-	return(r);
+	return r;
 }
 #endif /* SHOPT_ESH || SHOPT_VSH */
 
@@ -1406,20 +1276,20 @@ int ed_virt_to_phys(Edit_t *ep,genchar *virt,genchar *phys,int cur,int voff,int
 
 int	ed_internal(const char *src, genchar *dest)
 {
-	register const unsigned char *cp = (unsigned char *)src;
-	register int c;
-	register wchar_t *dp = (wchar_t*)dest;
+	const unsigned char *cp = (unsigned char *)src;
+	int c;
+	wchar_t *dp = (wchar_t*)dest;
 	if(dest == (genchar*)roundof(cp-(unsigned char*)0,sizeof(genchar)))
 	{
 		genchar buffer[MAXLINE];
 		c = ed_internal(src,buffer);
 		ed_gencpy((genchar*)dp,buffer);
-		return(c);
+		return c;
 	}
 	while(*cp)
 		*dp++ = mbchar(cp);
 	*dp = 0;
-	return(dp-(wchar_t*)dest);
+	return dp - (wchar_t*)dest;
 }
 #endif /* (SHOPT_ESH || SHOPT_VSH) && SHOPT_MULTIBYTE */
 
@@ -1432,8 +1302,8 @@ int	ed_internal(const char *src, genchar *dest)
 
 int	ed_external(const genchar *src, char *dest)
 {
-	register genchar wc;
-	register char *dp = dest;
+	genchar wc;
+	char *dp = dest;
 	char *dpmax = dp+sizeof(genchar)*MAXLINE-2;
 	if((char*)src == dp)
 	{
@@ -1446,7 +1316,7 @@ int	ed_external(const genchar *src, char *dest)
 #else
 		strcopy(dest,buffer);
 #endif
-		return(c);
+		return c;
 	}
 	while((wc = *src++) && dp<dpmax)
 	{
@@ -1460,7 +1330,7 @@ int	ed_external(const genchar *src, char *dest)
 		dp += size;
 	}
 	*dp = 0;
-	return(dp-dest);
+	return dp-dest;
 }
 #endif /* SHOPT_MULTIBYTE */
 
@@ -1482,7 +1352,7 @@ void	ed_gencpy(genchar *dp,const genchar *sp)
  * copy at most <n> items from <sp> to <dp>
  */
 
-void	ed_genncpy(register genchar *dp,register const genchar *sp, int n)
+void	ed_genncpy(genchar *dp,const genchar *sp, int n)
 {
 	dp = (genchar*)roundof((char*)dp-(char*)0,sizeof(genchar));
 	sp = (const genchar*)roundof((char*)sp-(char*)0,sizeof(genchar));
@@ -1495,12 +1365,12 @@ void	ed_genncpy(register genchar *dp,register const genchar *sp, int n)
  * find the string length of <str>
  */
 
-int	ed_genlen(register const genchar *str)
+int	ed_genlen(const genchar *str)
 {
-	register const genchar *sp = str;
+	const genchar *sp = str;
 	sp = (const genchar*)roundof((char*)sp-(char*)0,sizeof(genchar));
 	while(*sp++);
-	return(sp-str-1);
+	return sp-str-1;
 }
 #endif /* (SHOPT_ESH || SHOPT_VSH) && SHOPT_MULTIBYTE */
 
@@ -1520,12 +1390,12 @@ int	ed_genlen(register const genchar *str)
 
 int tcgetattr(int fd, struct termios *tt)
 {
-	register Edit_t *ep = (Edit_t*)(sh.ed_context);
-	register int r,i;
+	Edit_t *ep = (Edit_t*)(sh.ed_context);
+	int r,i;
 	ep->e_tcgeta = 0;
 	ep->e_echoctl = (ECHOCTL!=0);
 	if((r=ioctl(fd,TCGETS,tt))>=0 ||  errno!=EINVAL)
-		return(r);
+		return r;
 	if((r=ioctl(fd,TCGETA,&ott)) >= 0)
 	{
 		tt->c_lflag = ott.c_lflag;
@@ -1537,16 +1407,16 @@ int tcgetattr(int fd, struct termios *tt)
 		ep->e_tcgeta++;
 		ep->e_echoctl = 0;
 	}
-	return(r);
+	return r;
 }
 
 int tcsetattr(int fd,int mode,struct termios *tt)
 {
-	register Edit_t *ep = (Edit_t*)(sh.ed_context);
-	register int r;
+	Edit_t *ep = (Edit_t*)(sh.ed_context);
+	int r;
 	if(ep->e_tcgeta)
 	{
-		register int i;
+		int i;
 		ott.c_lflag = tt->c_lflag;
 		ott.c_oflag = tt->c_oflag;
 		ott.c_iflag = tt->c_iflag;
@@ -1573,9 +1443,9 @@ int tcsetattr(int fd,int mode,struct termios *tt)
 			case TCSAFLUSH:
 				mode = TCSETAF;
 		}
-		return(ioctl(fd,mode,&ott));
+		return ioctl(fd,mode,&ott);
 	}
-	return(ioctl(fd,mode,tt));
+	return ioctl(fd,mode,tt);
 }
 #endif /* SHOPT_OLDTERMIO */
 
@@ -1583,9 +1453,9 @@ int tcsetattr(int fd,int mode,struct termios *tt)
  * Execute keyboard trap on given buffer <inbuff> of given size <isize>
  * <mode> < 0 for vi insert mode
  */
-static int keytrap(Edit_t *ep,char *inbuff,register int insize, int bufsize, int mode)
+static int keytrap(Edit_t *ep,char *inbuff,int insize, int bufsize, int mode)
 {
-	register char *cp;
+	char *cp;
 	int savexit;
 	Lex_t *lexp = (Lex_t*)sh.lex_context, savelex;
 #if SHOPT_MULTIBYTE
@@ -1623,7 +1493,7 @@ static int keytrap(Edit_t *ep,char *inbuff,register int insize, int bufsize, int
 	else
 		insize = 0;
 	nv_unset(ED_TXTNOD);
-	return(insize);
+	return insize;
 }
 
 #if SHOPT_EDPREDICT
@@ -1847,7 +1717,7 @@ void	*ed_open(void)
 {
 	Edit_t *ed = sh_newof(0,Edit_t,1,0);
 	strcpy(ed->e_macro,"_??");
-	return((void*)ed);
+	return ed;
 }
 
 #undef ioctl
@@ -1873,7 +1743,7 @@ int	sh_ioctl(int fd, int cmd, void* val, int sz)
 				errno = err;
 		}
 	}
-	return(r);
+	return r;
 }
 
 #ifdef _lib_tcgetattr
@@ -1883,7 +1753,7 @@ int sh_tcgetattr(int fd, struct termios *tty)
 	int r,err = errno;
 	while((r=tcgetattr(fd,tty)) < 0 && errno==EINTR)
 		errno = err;
-	return(r);
+	return r;
     }
 
 #   undef tcsetattr
@@ -1892,6 +1762,6 @@ int sh_tcsetattr(int fd, int cmd, struct termios *tty)
 	int r,err = errno;
 	while((r=tcsetattr(fd,cmd,tty)) < 0 && errno==EINTR)
 		errno = err;
-	return(r);
+	return r;
     }
 #endif
