@@ -2,7 +2,7 @@
 *                                                                      *
 *               This software is part of the ast package               *
 *          Copyright (c) 1982-2012 AT&T Intellectual Property          *
-*          Copyright (c) 2020-2022 Contributors to ksh 93u+m           *
+*          Copyright (c) 2020-2023 Contributors to ksh 93u+m           *
 *                      and is licensed under the                       *
 *                 Eclipse Public License, Version 2.0                  *
 *                                                                      *
@@ -16,6 +16,7 @@
 *         hyenias <58673227+hyenias@users.noreply.github.com>          *
 *                   Marc Wilson <posguy99@gmail.com>                   *
 *                      Phi <phi.debian@gmail.com>                      *
+*               K. Eugene Carlson <kvngncrlsn@gmail.com>               *
 *                                                                      *
 ***********************************************************************/
 /*
@@ -66,6 +67,8 @@ typedef struct  _mac_
 	char		split;		/* set when word splitting is possible */
 	char		pattern;	/* set when glob pattern expansion or matching follows */
 	char		patfound;	/* set if pattern character found */
+	char		noextpat;	/* set to disallow extended patterns */
+	char		wasexpan;	/* set when word resulted from a $ or ` expansion */
 	char		assign;		/* set for assignments */
 	char		arith;		/* set for ((...)) */
 	char		arrayok;	/* $x[] ok for arrays */
@@ -113,14 +116,14 @@ static int	charlen(const char*,int);
 
 void *sh_macopen(void)
 {
-	return(sh_newof(0,Mac_t,1,0));
+	return sh_newof(0,Mac_t,1,0);
 }
 
 /*
  * perform only parameter substitution and catch failures
  * (also save lexer state to allow use while in here-docs)
  */
-char *sh_mactry(register char *string)
+char *sh_mactry(char *string)
 {
 	if(string)
 	{
@@ -135,9 +138,9 @@ char *sh_mactry(register char *string)
 		sh_popcontext(&buff);
 		*lexp = savelex;
 		sh.savexit = savexit;
-		return(string);
+		return string;
 	}
-	return("");
+	return "";
 }
 
 /*
@@ -147,12 +150,11 @@ char *sh_mactry(register char *string)
  * yields a single pathname.
  * If <mode> negative, then expansion rules for assignment are applied.
  */
-char *sh_mactrim(char *str, register int mode)
+char *sh_mactrim(char *str, int mode)
 {
-	register Mac_t	*mp = (Mac_t*)sh.mac_context;
-	Stk_t		*stkp = sh.stk;
-	Mac_t		savemac;
-	savemac = *mp;
+	Mac_t	*mp = (Mac_t*)sh.mac_context;
+	Stk_t	*stkp = sh.stk;
+	Mac_t	savemac = *mp;
 	stkseek(stkp,0);
 	mp->arith = (mode==3);
 	sh.argaddr = 0;
@@ -175,7 +177,7 @@ char *sh_mactrim(char *str, register int mode)
 	{
 		/* expand only if unique */
 		struct argnod *arglist=0;
-		if((mode=path_expand(str,&arglist))==1)
+		if((mode=path_expand(str,&arglist,0))==1)
 			str = arglist->argval;
 		else if(mode>1)
 		{
@@ -185,21 +187,20 @@ char *sh_mactrim(char *str, register int mode)
 		sh_trim(str);
 	}
 	*mp = savemac;
-	return(str);
+	return str;
 }
 
 /*
  * Perform all the expansions on the argument <argp>
  */
-int sh_macexpand(register struct argnod *argp, struct argnod **arghead,int flag)
+int sh_macexpand(struct argnod *argp, struct argnod **arghead,int flag)
 {
-	register int	flags = argp->argflag;
-	register char	*str = argp->argval;
-	register Mac_t  *mp = (Mac_t*)sh.mac_context;
-	char		**saveargaddr = sh.argaddr;
-	Mac_t		savemac;
-	Stk_t		*stkp = sh.stk;
-	savemac = *mp;
+	int	flags = argp->argflag;
+	char	*str = argp->argval;
+	Mac_t	*mp = (Mac_t*)sh.mac_context;
+	char	**saveargaddr = sh.argaddr;
+	Mac_t	savemac = *mp;
+	Stk_t	*stkp = sh.stk;
 	mp->sp = 0;
 	if(mp->ifsp=nv_getval(sh_scoped(IFSNOD)))
 		mp->ifs = *mp->ifsp;
@@ -250,7 +251,7 @@ int sh_macexpand(register struct argnod *argp, struct argnod **arghead,int flag)
 	}
 	sh.argaddr = saveargaddr;
 	*mp = savemac;
-	return(flags);
+	return flags;
 }
 
 /*
@@ -259,15 +260,14 @@ int sh_macexpand(register struct argnod *argp, struct argnod **arghead,int flag)
  */
 void sh_machere(Sfio_t *infile, Sfio_t *outfile, char *string)
 {
-	register int	c,n;
-	register const char	*state = sh_lexstates[ST_QUOTE];
-	register char	*cp;
-	register Mac_t	*mp = (Mac_t*)sh.mac_context;
+	int		c,n;
+	const char	*state = sh_lexstates[ST_QUOTE];
+	char		*cp;
+	Mac_t		*mp = (Mac_t*)sh.mac_context;
 	Lex_t		*lp = (Lex_t*)sh.lex_context;
 	Fcin_t		save;
-	Mac_t		savemac;
+	Mac_t		savemac = *mp;
 	Stk_t		*stkp = sh.stk;
-	savemac = *mp;
 	stkseek(stkp,0);
 	sh.argaddr = 0;
 	mp->sp = outfile;
@@ -335,7 +335,7 @@ void sh_machere(Sfio_t *infile, Sfio_t *outfile, char *string)
 				sfputc(outfile,ESCAPE);
 			continue;
 		    case S_GRAVE:
-			comsubst(mp,(Shnode_t*)0,0);
+			comsubst(mp,NULL,0);
 			break;
 		    case S_DOL:
 			c = fcget();
@@ -354,11 +354,10 @@ void sh_machere(Sfio_t *infile, Sfio_t *outfile, char *string)
 				sfputc(stkp,c);
 				if(n==S_LBRA)
 				{
-					c = fcget();
-					fcseek(-1);
+					c = fcpeek(0);
 					if(sh_lexstates[ST_NORM][c]==S_BREAK)
 					{
-						comsubst(mp,(Shnode_t*)0,2);
+						comsubst(mp,NULL,2);
 						break;
 					}
 					sh_lexskip(lp,RBRACE,1,ST_BRACE);
@@ -381,7 +380,7 @@ void sh_machere(Sfio_t *infile, Sfio_t *outfile, char *string)
 				break;
 			    }
 			    case S_PAR:
-				comsubst(mp,(Shnode_t*)0,3);
+				comsubst(mp,NULL,1);
 				break;
 			    case S_EOF:
 				if((c=fcfill()) > 0)
@@ -401,17 +400,17 @@ void sh_machere(Sfio_t *infile, Sfio_t *outfile, char *string)
 /*
  * expand argument but do not trim pattern characters
  */
-char *sh_macpat(register struct argnod *arg, int flags)
+char *sh_macpat(struct argnod *arg, int flags)
 {
-	register char *sp = arg->argval;
+	char *sp = arg->argval;
 	if((arg->argflag&ARG_RAW))
-		return(sp);
+		return sp;
 	sh_stats(STAT_ARGEXPAND);
 	if(flags&ARG_OPTIMIZE)
 		arg->argchn.ap=0;
 	if(!(sp=arg->argchn.cp))
 	{
-		sh_macexpand(arg,NIL(struct argnod**),flags|ARG_ARRAYOK);
+		sh_macexpand(arg,NULL,flags|ARG_ARRAYOK);
 		sp = arg->argchn.cp;
 		if(!(flags&ARG_OPTIMIZE) || !(arg->argflag&ARG_MAKE))
 			arg->argchn.cp = 0;
@@ -419,17 +418,17 @@ char *sh_macpat(register struct argnod *arg, int flags)
 	}
 	else
 		sh_stats(STAT_ARGHITS);
-	return(sp);
+	return sp;
 }
 
 /*
  * Process the characters up to <endch> or end of input string 
  */
-static void copyto(register Mac_t *mp,int endch, int newquote)
+static void copyto(Mac_t *mp,int endch, int newquote)
 {
-	register int	c,n;
-	register const char	*state = sh_lexstates[ST_MACRO];
-	register char	*cp,*first;
+	int		c,n;
+	const char	*state = sh_lexstates[ST_MACRO];
+	char		*cp,*first;
 	Lex_t		*lp = (Lex_t*)sh.lex_context;
 	int		tilde = -1;		/* offset for tilde expansion */
 	int		dotdot = 0;		/* offset for '..' in subscript */
@@ -437,19 +436,20 @@ static void copyto(register Mac_t *mp,int endch, int newquote)
 	int		brace = 0;		/* level of {braces} */
 	char		oldquote = mp->quote;	/* save "double quoted" state */
 	char		ansi_c = 0;		/* set when processing ANSI C escape codes */
-	char		ere = 0;		/* set when processing an extended regular expression */
+	int32_t		ere = 0;		/* bitmask of pattern options indicating an extended regular expression */
 	char		bracketexpr = 0; 	/* set when in [brackets] within a non-ERE glob pattern */
 	Sfio_t		*sp = mp->sp;
 	Stk_t		*stkp = sh.stk;
 	char		*resume = 0;
-	mp->sp = NIL(Sfio_t*);
+	mp->sp = NULL;
 	mp->quote = newquote;
 	first = cp = fcseek(0);
-	if(!mp->quote && *cp=='~' && cp[1]!=LPAREN)
+	if(!mp->quote && *cp=='~' && cp[1]!=LPAREN && !sh_isstate(SH_NOTILDEXP))
 		tilde = stktell(stkp);
 	/* handle // operator specially */
 	if(mp->pattern==2 && *cp=='/')
 		cp++;
+	mbinit();
 	while(1)
 	{
 		if(mbwide())
@@ -540,7 +540,7 @@ static void copyto(register Mac_t *mp,int endch, int newquote)
 				if(!mp->lit && !mp->quote)
 				{
 					int nc = *(unsigned char*)cp;
-					if((n==S_DIG || ((paren+ere) && (sh_lexstates[ST_DOL][nc]==S_ALP) || nc=='<' || nc=='>')))
+					if(n==S_DIG || ((paren || ere) && sh_lexstates[ST_DOL][nc]==S_ALP || nc=='<' || nc=='>'))
 						break;
 					if(ere && mp->pattern==1 && strchr(".[()*+?{|^$&!",*cp))
 						break;
@@ -591,7 +591,7 @@ static void copyto(register Mac_t *mp,int endch, int newquote)
 			first = fcseek(c+1);
 			c = mp->pattern;
 			if(n==S_GRAVE)
-				comsubst(mp,(Shnode_t*)0,0);
+				comsubst(mp,NULL,0);
 			else if((n= *cp) == '"' && !mp->quote)
 			{
 				int off = stktell(stkp);
@@ -755,13 +755,36 @@ static void copyto(register Mac_t *mp,int endch, int newquote)
 					paren++;
 					if((cp-first)>1 && cp[-2]=='~')
 					{
-						char *p = cp;
-						while((c=mbchar(p)) && c!=RPAREN)
-							if(c=='A'||c=='E'||c=='K'||c=='P'||c=='X')
-							{
-								ere = 1;
+						char *p = cp, minus = 0;
+						/* Parse expression options to know if the expression is an ERE */
+						while((c=mbchar(p)) && c!=RPAREN && c!=':') switch(c)
+						{
+							case 'A':  /* augmented regular expression (AST) */
+							case 'E':  /* extended regular expression */
+							case 'P':  /* Perl regular expression */
+							case 'X':  /* augmented regular expression (AST) */
+								if(minus)
+									ere &= ~(1 << c - 'A');
+								else
+									ere |= 1 << c - 'A';
 								break;
-							}
+							case 'B':  /* basic regular expression */
+							case 'F':  /* fixed pattern */
+							case 'G':  /* basic regular expression */
+							case 'K':  /* ksh glob pattern */
+							case 'L':  /* fixed pattern */
+							case 'S':  /* sh glob pattern */
+							case 'V':  /* System V regular expression */
+								if(!minus)
+									ere = 0;
+								break;
+							case '-':  /* disable the following options */
+								minus = 1;
+								break;
+							case '+':  /* enable the following options */
+								minus = 0;
+								break;
+						}
 					}
 				}
 				else if(n==RPAREN)
@@ -783,9 +806,8 @@ static void copyto(register Mac_t *mp,int endch, int newquote)
 			if(!(mp->quote || mp->lit))
 			{
 #if SHOPT_BRACEPAT
-				mp->patfound = mp->split && sh_isoption(SH_BRACEEXPAND);
-#else
-				mp->patfound = 0;
+				if(mp->split && sh_isoption(SH_BRACEEXPAND))
+					mp->patfound = 1;
 #endif
 				brace++;
 			}
@@ -811,7 +833,7 @@ static void copyto(register Mac_t *mp,int endch, int newquote)
 		    case S_EQ:
 			if(mp->assign==1)
 			{
-				if(*cp=='~' && !endch && !mp->quote && !mp->lit)
+				if(*cp=='~' && !endch && !mp->quote && !mp->lit && !sh_isstate(SH_NOTILDEXP))
 					tilde = stktell(stkp)+(c+1);
 				mp->assign = 2;
 			}
@@ -834,7 +856,7 @@ static void copyto(register Mac_t *mp,int endch, int newquote)
 				tilde = -1;
 				c=0;
 			}
-			if(n==S_COLON && mp->assign==2 && *cp=='~' && endch==0 && !mp->quote &&!mp->lit)
+			if(n==S_COLON && mp->assign==2 && *cp=='~' && endch==0 && !mp->quote && !mp->lit && !sh_isstate(SH_NOTILDEXP))
 				tilde = stktell(stkp)+(c+1);
 			else if(n==S_SLASH && mp->pattern==2)
 			{
@@ -879,12 +901,12 @@ done:
 /*
  * copy <str> to stack performing sub-expression substitutions
  */
-static void mac_substitute(Mac_t *mp, register char *cp,char *str,register int subexp[],int subsize)
+static void mac_substitute(Mac_t *mp, char *cp,char *str,int subexp[],int subsize)
 {
-	register int	c,n;
-	register char *first=fcseek(0);
-	char		*ptr;
-	Mac_t		savemac;
+	int	c,n;
+	char	*first=fcseek(0);
+	char	*ptr;
+	Mac_t	savemac;
 	n = stktell(sh.stk);
 	savemac = *mp;
 	mp->pattern = 3;
@@ -939,11 +961,11 @@ static void mac_substitute(Mac_t *mp, register char *cp,char *str,register int s
  */
 static char *getdolarg(int n, int *size)
 {
-	register int c=S_DELIM, d=sh.ifstable['\\'];
-	register unsigned char *first,*last,*cp = (unsigned char*)sh.cur_line;
-	register int m=sh.offsets[0],delim=0;
+	int c=S_DELIM, d=sh.ifstable['\\'];
+	unsigned char *first,*last,*cp = (unsigned char*)sh.cur_line;
+	int m=sh.offsets[0],delim=0;
 	if(m==0)
-		return(0);
+		return NULL;
 	if(m<0)
 		m = 0;
 	else if(n<=m)
@@ -985,7 +1007,7 @@ static char *getdolarg(int n, int *size)
 		first = last = 0;
 	if(size)
 		*size = last-first;
-	return((char*)first);
+	return (char*)first;
 }
 #endif /* SHOPT_FILESCAN */
 
@@ -995,7 +1017,7 @@ static char *getdolarg(int n, int *size)
 static char *prefix(char *id)
 {
 	Namval_t *np;
-	register char *sub=0, *cp = strchr(id,'.');
+	char *sub=0, *cp = strchr(id,'.');
 	if(cp)
 	{
 		*cp = 0;
@@ -1025,10 +1047,10 @@ static char *prefix(char *id)
 				id[n-1] = ']';
 			}
 			strcpy(&id[n],cp);
-			return(id);
+			return id;
 		}
 	}
-	return(sh_strdup(id));
+	return sh_strdup(id);
 }
 
 /*
@@ -1053,7 +1075,7 @@ static int subcopy(Mac_t *mp, int flag)
 	mp->split = split;
 	mp->arith = xarith;
 	mp->arrayok = arrayok;
-	return(loc);
+	return loc;
 }
 
 /*
@@ -1063,7 +1085,7 @@ static int subcopy(Mac_t *mp, int flag)
 int sh_macfun(const char *name, int offset)
 {
 	Namval_t	*np, *nq;
-	np = nv_bfsearch(name,sh.fun_tree,&nq,(char**)0);
+	np = nv_bfsearch(name,sh.fun_tree,&nq,NULL);
 	if(np)
 	{
 		/* treat ${x.foo} as ${x.foo;} */
@@ -1087,19 +1109,19 @@ int sh_macfun(const char *name, int offset)
 		stkseek(sh.stk,offset);
 		comsubst((Mac_t*)sh.mac_context,&t.node,2);
 		free(d.dol.dolval[0]);
-		return(1);
+		return 1;
 	}
-	return(0);
+	return 0;
 }
 
 static int namecount(Mac_t *mp,const char *prefix)
 {
 	int count = 0;
-	mp->nvwalk = nv_diropen((Namval_t*)0,prefix);
+	mp->nvwalk = nv_diropen(NULL,prefix);
 	while(nv_dirnext(mp->nvwalk))
 		count++;
 	nv_dirclose(mp->nvwalk);
-	return(count);
+	return count;
 }
 
 static char *nextname(Mac_t *mp,const char *prefix, int len)
@@ -1107,12 +1129,12 @@ static char *nextname(Mac_t *mp,const char *prefix, int len)
 	char *cp;
 	if(len==0)
 	{
-		mp->nvwalk = nv_diropen((Namval_t*)0,prefix);
-		return((char*)mp->nvwalk);
+		mp->nvwalk = nv_diropen(NULL,prefix);
+		return (char*)mp->nvwalk;
 	}
 	if(!(cp=nv_dirnext(mp->nvwalk)))
 		nv_dirclose(mp->nvwalk);
-	return(cp);
+	return cp;
 }
 
 /*
@@ -1121,11 +1143,11 @@ static char *nextname(Mac_t *mp,const char *prefix, int len)
  */
 static int varsub(Mac_t *mp)
 {
-	register int	c;
-	register int	type=0; /* M_xxx */
-	register char	*v,*argp=0;
-	register Namval_t	*np = NIL(Namval_t*);
-	register int 	dolg=0, mode=0;
+	int		c;
+	int		type=0; /* M_xxx */
+	char		*v,*argp=0;
+	Namval_t	*np = NULL;
+	int 		dolg=0, mode=0;
 	Lex_t		*lp = (Lex_t*)sh.lex_context;
 	Namarr_t	*ap=0;
 	int		dolmax=0, vsize= -1, offset= -1, nulflg, replen=0, bysub=0;
@@ -1133,6 +1155,7 @@ static int varsub(Mac_t *mp)
 	char		*idx = 0;
 	int		var=1,addsub=0,oldpat=mp->pattern,idnum=0,flag=0,d;
 	Stk_t		*stkp = sh.stk;
+	mp->wasexpan = 1;
 retry1:
 	idbuff[0] = 0;
 	idbuff[1] = 0;
@@ -1204,15 +1227,15 @@ retry1:
 	    case S_PAR:
 		if(type)
 			goto nosub;
-		comsubst(mp,(Shnode_t*)0,3);
-		return(1);
+		comsubst(mp,NULL,1);
+		return 1;
 	    case S_DIG:
 		var = 0;
 		c -= '0';
 		sh.argaddr = 0;
 		if(type)
 		{
-			register int d;
+			int d;
 			while((d=fcget()),isadigit(d))
 				c = 10*c + (d-'0');
 			fcseek(-1);
@@ -1237,8 +1260,7 @@ retry1:
 		/* Handle 'set -u'/'set -o nounset' for positional parameters */
 		if(!v && sh_isoption(SH_NOUNSET))
 		{
-			d=fcget();
-			fcseek(-1);
+			unsigned char d = fcpeek(0);
 			if(!(d && strchr(":+-?=",d)))
 			{
 				errormsg(SH_DICT,ERROR_exit(1),e_notset,ltos(c));
@@ -1254,7 +1276,7 @@ retry1:
 		offset = stktell(stkp);
 		do
 		{
-			register int d;
+			int d;
 			np = 0;
 			do
 			{
@@ -1376,7 +1398,7 @@ retry1:
 			if(sh_macfun(id,offset))
 			{
 				fcmbget(&LEN);
-				return(1);
+				return 1;
 			}
 		}
 		if(np && (flag&NV_NOADD) && nv_isnull(np))
@@ -1426,7 +1448,7 @@ retry1:
 				}
 			}
 			else if(ap && (isastchar(mode)||type==M_TREE)  && !(ap->nelem&ARRAY_SCAN) && type!=M_SIZE)
-				nv_putsub(np,NIL(char*),ARRAY_SCAN);
+				nv_putsub(np,NULL,ARRAY_SCAN);
 			if(!isbracechar(c))
 				goto nosub;
 			else
@@ -1493,7 +1515,7 @@ retry1:
 				v = sh.cur_line;
 #endif  /* SHOPT_FILESCAN */
 			else if(type==M_TREE)
-				v = nv_getvtree(np,(Namfun_t*)0);
+				v = nv_getvtree(np,NULL);
 			else
 			{
 				if(type && fcpeek(0)=='+')
@@ -1510,7 +1532,7 @@ retry1:
 					v = nv_getval(np);
 				mp->atmode = (v && mp->quoted && mode=='@');
 			}
-			if(savptr==stakptr(0))
+			if(savptr==stkptr(sh.stk,0))
 				stkseek(stkp,offset);
 			else
 				stkset(stkp,savptr,offset);
@@ -1610,7 +1632,7 @@ retry1:
 #if  SHOPT_FILESCAN
 				if(sh.cur_line)
 				{
-					getdolarg(MAX_ARGN,(int*)0);
+					getdolarg(MAX_ARGN,NULL);
 					c = sh.offsets[0];
 				}
 				else
@@ -1717,55 +1739,60 @@ retry1:
 		fcseek(-1);
 		c=0;
 	}
-	if(c==':')  /* ${name:expr1[:expr2]} */
+	/* check for slice operator: ${parameter:offset[:length]} */
+	if(c==':')
 	{
-		char *ptr;
-		type = (int)sh_strnum(argp,&ptr,1);
+		char *lastchar;
+		int sliceoffset;
+		sh_trim(argp);  /* remove internal backslash escapes */
+		sliceoffset = (int)sh_strnum(argp,&lastchar,1);
 		if(isastchar(mode))
 		{
 			if(id==idbuff)  /* ${@} or ${*} */
 			{
-				if(type<0 && (type+= dolmax)<0)
-					type = 0;
-				if(type==0)
+				if(sliceoffset < 0 && (sliceoffset += dolmax) < 0)
+					sliceoffset = 0;
+				if(sliceoffset==0)
 					v = special(dolg=0);
 #if  SHOPT_FILESCAN
 				else if(sh.cur_line)
 				{
-					v = getdolarg(dolg=type,&vsize);
+					v = getdolarg(dolg = sliceoffset, &vsize);
 					if(!v)
-						dolmax = type;
+						dolmax = sliceoffset;
 				}
 #endif  /* SHOPT_FILESCAN */
-				else if(type < dolmax)
-					v = sh.st.dolv[dolg=type];
+				else if(sliceoffset < dolmax)
+					v = sh.st.dolv[dolg = sliceoffset];
 				else
 					v =  0;
 			}
 			else if(ap)
 			{
+				/* Array slicing */
 				ap = nv_arrayptr(np); /* update */
-				if(type<0)
+				if(sliceoffset < 0)
 				{
 					if(array_assoc(ap))
-						type = -type;
+						sliceoffset = -sliceoffset;
 					else
-						type += array_maxindex(np);
+						sliceoffset += array_maxindex(np);
 				}
 				if(array_assoc(ap))
 				{
-					while(type-- >0 && (v=0,nv_nextsub(np)))
+					nv_putsub(np, NULL, ap->nelem&ARRAY_SCAN ? 0 : ARRAY_SCAN);
+					while(sliceoffset-- > 0 && (v=0,nv_nextsub(np)))
 						v = nv_getval(np);
 				}
-				else if(type > 0)
+				else if(sliceoffset >= 0)
 				{
-					if(nv_putsub(np,NIL(char*),type|ARRAY_SCAN))
+					if(nv_putsub(np, NULL, sliceoffset|ARRAY_SCAN))
 						v = nv_getval(np);
 					else
 						v = 0;
 				}
 			}
-			else if(type>0)
+			else if(sliceoffset > 0)
 				v = 0;
 			if(!v)
 				mp->atmode = 0;
@@ -1773,24 +1800,25 @@ retry1:
 		else if(v)
 		{
 			vsize = charlen(v,vsize);
-			if(type<0 && (type += vsize)<0)
-				type = 0;
-			if(vsize < type)
+			if(sliceoffset < 0 && (sliceoffset += vsize) < 0)
+				sliceoffset = 0;
+			if(vsize < sliceoffset)
 				v = 0;
 			else if(mbwide())
 			{
 				mbinit();
-				for(c=type;c;c--)
+				for(c = sliceoffset; c; c--)
 					mbchar(v);
 				c = ':';
 			}
 			else
-				v += type;
+				v += sliceoffset;
 			vsize = v?strlen(v):0;
 		}
-		if(*ptr==':')
+		if(*lastchar==':')
 		{
-			if((type = (int)sh_strnum(ptr+1,&ptr,1)) <=0)
+			int slicelength = (int)sh_strnum(lastchar+1,&lastchar,1);
+			if(slicelength <= 0)
 			{
 				v = 0;
 				mp->atmode = 0;
@@ -1799,33 +1827,33 @@ retry1:
 			{
 				if(dolg>=0)
 				{
-					if(dolg+type < dolmax)
-						dolmax = dolg+type;
+					if(dolg + slicelength < dolmax)
+						dolmax = dolg + slicelength;
 				}
 				else
-					dolmax = type;
+					dolmax = slicelength;
 			}
-			else if(type < vsize)
+			else if(slicelength < vsize)
 			{
 				if(mbwide())
 				{
 					char *vp = v;
 					mbinit();
-					while(type-->0)
+					while(slicelength-- > 0)
 					{
 						if((c=mbsize(vp))<1)
 							c = 1;
 						vp += c;
 					}
-					type = vp-v;
+					slicelength = vp - v;
 					c = ':';
 				}
-				vsize = type;
+				vsize = slicelength;
 			}
 			else
 				vsize = v?strlen(v):0;
 		}
-		if(*ptr)
+		if(*lastchar)
 			mac_error();
 		stkseek(stkp,offset);
 		argp = 0;
@@ -1866,7 +1894,7 @@ retry2:
 		int ofs_size = 0;
 		int match[2*(MATCH_MAX+1)],index;
 		int nmatch, nmatch_prev, vsize_last, tsize;
-		char *vlast = NIL(char*), *oldv;
+		char *vlast = NULL, *oldv;
 		while(1)
 		{
 			if(!v)
@@ -1884,9 +1912,16 @@ retry2:
 					oldv = v;
 					nmatch_prev = nmatch;
 					if(c=='%')
-						nmatch=substring(v,tsize,pattern,match,flag&STR_MAXIMAL);
+						nmatch = substring(v, tsize,
+							*pattern ? pattern : "~(E)$",
+							match,
+							flag & STR_MAXIMAL);
 					else
-						nmatch=strngrpmatch(v,vsize,pattern,(ssize_t*)match,elementsof(match)/2,flag|STR_INT);
+						nmatch = strngrpmatch(v, vsize,
+							*pattern ? pattern : "~(E)^",
+							(ssize_t*)match,
+							elementsof(match) / 2,
+							flag | STR_INT);
 					if(nmatch && repstr && !mp->macsub)
 						sh_setmatch(v,vsize,nmatch,match,index++);
 					if(nmatch)
@@ -1983,7 +2018,7 @@ retry2:
 			{
 				if(dolmax &&  --dolmax <=0)
 				{
-					nv_putsub(np,NIL(char*),ARRAY_UNDEF);
+					nv_putsub(np,NULL,ARRAY_UNDEF);
 					break;
 				}
 				if(ap)
@@ -2035,7 +2070,7 @@ retry2:
 			}
 		}
 		if(arrmax)
-			free((void*)arrmax);
+			free(arrmax);
 	}
 	else if(argp)
 	{
@@ -2093,35 +2128,37 @@ retry2:
 		free(pattern);
 	if(idx)
 		free(idx);
-	return(1);
+	return 1;
 nosub:
 	if(type==M_BRACE && sh_lexstates[ST_NORM][c]==S_BREAK)
 	{
 		fcseek(-1);
-		comsubst(mp,(Shnode_t*)0,2);
-		return(1);
+		comsubst(mp,NULL,2);
+		return 1;
 	}
 	if(type)
 		mac_error();
 	fcseek(-1);
-	return(0);
+	mp->wasexpan = 0;
+	return 0;
 }
 
 /*
  * This routine handles command substitution
+ * and arithmetic expansion.
  * <type> is 0 for older `...` version
  * 1 for $(...) or 2 for ${ subshare; }
  */
-static void comsubst(Mac_t *mp,register Shnode_t* t, int type)
+static void comsubst(Mac_t *mp,Shnode_t* t, int type)
 {
 	Sfdouble_t		num;
-	register int		c;
-	register char		*str;
+	int			c;
+	char			*str;
 	Sfio_t			*sp;
 	Stk_t			*stkp = sh.stk;
 	Fcin_t			save;
 	struct slnod            *saveslp = sh.st.staklist;
-	struct _mac_		savemac;
+	Mac_t			savemac = *mp;
 	int			savtop = stktell(stkp);
 	char			lastc=0, *savptr = stkfreeze(stkp,0);
 	int			was_history = sh_isstate(SH_HISTORY);
@@ -2130,8 +2167,8 @@ static void comsubst(Mac_t *mp,register Shnode_t* t, int type)
 	int			newlines,bufsize,nextnewlines;
 	Sfoff_t			foff;
 	Namval_t		*np;
+	savemac.wasexpan = 1;
 	sh.argaddr = 0;
-	savemac = *mp;
 	sh.st.staklist=0;
 	if(type)
 	{
@@ -2185,7 +2222,7 @@ static void comsubst(Mac_t *mp,register Shnode_t* t, int type)
 		sh_offstate(SH_VERBOSE);
 		if(mp->sp)
 			sfsync(mp->sp);	/* flush before executing command */
-		sp = sfnew(NIL(Sfio_t*),str,c,-1,SF_STRING|SF_READ);
+		sp = sfnew(NULL,str,c,-1,SF_STRING|SF_READ);
 		c = sh.inlineno;
 		sh.inlineno = error_info.line+sh.st.firstline;
 		t = (Shnode_t*)sh_parse(sp,SH_EOF|SH_NL);
@@ -2199,7 +2236,7 @@ static void comsubst(Mac_t *mp,register Shnode_t* t, int type)
 		if(t->tre.tretyp==0 && !t->com.comarg && !t->com.comset)
 		{
 			/* special case $(<file) and $(<#file) */
-			register int fd;
+			int fd;
 			int r=0;
 			struct checkpt buff;
 			struct ionod *ip=0;
@@ -2216,15 +2253,14 @@ static void comsubst(Mac_t *mp,register Shnode_t* t, int type)
 				if(sp=sh.sftable[fd])
 					num = sftell(sp);
 				else
-					num = lseek(fd, (off_t)0, SEEK_CUR);
+					num = lseek(fd, 0, SEEK_CUR);
 				goto out_offset;
 			}
 			if(!(sp=sh.sftable[fd]))
 			{
 				char *cp = (char*)sh_malloc(IOBSIZE+1);
-				sp = sfnew(NIL(Sfio_t*),cp,IOBSIZE,fd,SF_READ|SF_MALLOC);
+				sp = sfnew(NULL,cp,IOBSIZE,fd,SF_READ|SF_MALLOC);
 			}
-			type = 3;
 		}
 		else
 		{
@@ -2235,7 +2271,7 @@ static void comsubst(Mac_t *mp,register Shnode_t* t, int type)
 		fcrestore(&save);
 	}
 	else
-		sp = sfopen(NIL(Sfio_t*),"","sr");
+		sp = sfopen(NULL,"","sr");
 	sh_freeup();
 	sh.st.staklist = saveslp;
 	if(was_history)
@@ -2248,15 +2284,15 @@ static void comsubst(Mac_t *mp,register Shnode_t* t, int type)
 	mp->ifsp = nv_getval(np);
 	stkset(stkp,savptr,savtop);
 	newlines = 0;
-	sfsetbuf(sp,(void*)sp,0);
+	sfsetbuf(sp,sp,0);
 	bufsize = sfvalue(sp);
 	/* read command substitution output and put on stack or here-doc */
-	sfpool(sp, NIL(Sfio_t*), SF_WRITE);
+	sfpool(sp, NULL, SF_WRITE);
 	sh_offstate(SH_INTERACTIVE);
-	if((foff = sfseek(sp,(Sfoff_t)0,SEEK_END)) > 0)
+	if((foff = sfseek(sp,0,SEEK_END)) > 0)
 	{
 		size_t soff = stktell(stkp); 
-		sfseek(sp,(Sfoff_t)0,SEEK_SET);
+		sfseek(sp,0,SEEK_SET);
 		stkseek(stkp,soff+foff+64);
 		stkseek(stkp,soff);
 	}
@@ -2264,7 +2300,7 @@ static void comsubst(Mac_t *mp,register Shnode_t* t, int type)
 	{
 #if SHOPT_CRNL
 		/* eliminate <cr> */
-		register char *dp;
+		char *dp;
 		char *buff = str;
 		while(c>1 && (*str !='\r'|| str[1]!='\n'))
 		{
@@ -2344,13 +2380,13 @@ static void comsubst(Mac_t *mp,register Shnode_t* t, int type)
 /*
  * copy <str> onto the stack
  */
-static void mac_copy(register Mac_t *mp,register const char *str, register int size)
+static void mac_copy(Mac_t *mp,const char *str, int size)
 {
-	register char		*state;
-	register const char	*cp=str;
-	register int		c,n,nopat,len;
-	Stk_t			*stkp=sh.stk;
-	int			oldpat = mp->pattern;
+	char		*state;
+	const char	*cp=str;
+	int		c,n,nopat,len;
+	Stk_t		*stkp=sh.stk;
+	int		oldpat = mp->pattern;
 	nopat = (mp->quote||(mp->assign==1)||mp->arith);
 	if(mp->sp)
 		sfwrite(mp->sp,str,size);
@@ -2444,11 +2480,14 @@ static void mac_copy(register Mac_t *mp,register const char *str, register int s
 				size -= len;
 				continue;
 			}
-			if(n==S_ESC || n==S_EPAT)
+			if(n==S_ESC)
+				sfputc(stkp,ESCAPE);
+			else if(n==S_EPAT)
 			{
 				/* don't allow extended patterns in this case */
 				mp->patfound = mp->pattern;
 				sfputc(stkp,ESCAPE);
+				mp->noextpat = 1;
 			}
 			else if(n==S_PAT)
 				mp->patfound = mp->pattern;
@@ -2530,11 +2569,11 @@ static void mac_copy(register Mac_t *mp,register const char *str, register int s
  * If field is null count field if <split> is non-zero
  * Do filename expansion if required
  */
-static void endfield(register Mac_t *mp,int split)
+static void endfield(Mac_t *mp,int split)
 {
-	register struct argnod	*argp;
-	register int		count=0;
-	Stk_t			*stkp = sh.stk;
+	struct argnod	*argp;
+	int		count=0;
+	Stk_t		*stkp = sh.stk;
 	if(stktell(stkp) > ARGVAL || split)
 	{
 		argp = (struct argnod*)stkfreeze(stkp,1);
@@ -2543,14 +2582,15 @@ static void endfield(register Mac_t *mp,int split)
 		mp->atmode = 0;
 		if(mp->patfound)
 		{
+			int musttrim = mp->wasexpan && !mp->quoted && !mp->noextpat && strchr(argp->argval,'\\');
 			sh.argaddr = 0;
 #if SHOPT_BRACEPAT
 			/* in POSIX mode, disallow brace expansion for unquoted expansions */
 			if(sh_isoption(SH_BRACEEXPAND) && !(sh_isoption(SH_POSIX) && mp->pattern==1))
-				count = path_generate(argp,mp->arghead);
+				count = path_generate(argp,mp->arghead,musttrim);
 			else
 #endif /* SHOPT_BRACEPAT */
-				count = path_expand(argp->argval,mp->arghead);
+				count = path_expand(argp->argval,mp->arghead,musttrim);
 			if(count)
 				mp->fields += count;
 			else if(split)	/* pattern is null string */
@@ -2579,19 +2619,19 @@ static void endfield(register Mac_t *mp,int split)
  * Finds the right substring of STRING using the expression PAT
  * the longest substring is found when FLAG is set.
  */
-static int substring(register const char *string,size_t len,const char *pat,int match[], int flag)
+static int substring(const char *string,size_t len,const char *pat,int match[], int flag)
 {
-	register const char *sp=string;
-	register int size,nmatch,n;
+	const char *sp=string;
+	int size,nmatch,n;
 	int smatch[2*(MATCH_MAX+1)];
 	if(flag)
 	{
 		if(n=strngrpmatch(sp,len,pat,(ssize_t*)smatch,elementsof(smatch)/2,STR_RIGHT|STR_MAXIMAL|STR_INT))
 		{
 			memcpy(match,smatch,n*2*sizeof(smatch[0]));
-			return(n);
+			return n;
 		}
-		return(0);
+		return 0;
 	}
 	size = (int)len;
 	sp += size;
@@ -2609,21 +2649,21 @@ static int substring(register const char *string,size_t len,const char *pat,int
 		sp--;
 	}
 	if(size==len)
-		return(0);
+		return 0;
 	if(nmatch)
 	{
 		nmatch *=2;
 		while(--nmatch>=0)
 			match[nmatch] += size;
 	}
-	return(n);
+	return n;
 }
 
 #if SHOPT_MULTIBYTE
 	static char	*_lastchar(const char *string, const char *endstring)
 	{
-		register char *str = (char*)string;
-		register int c;
+		char *str = (char*)string;
+		int c;
 		mbinit();
 		while(*str)
 		{
@@ -2633,17 +2673,17 @@ static int substring(register const char *string,size_t len,const char *pat,int
 				break;
 			str += c;
 		}
-		return(str);
+		return str;
 	}
 #endif /* SHOPT_MULTIBYTE */
 static int	charlen(const char *string,int len)
 {
 	if(!string)
-		return(0);
+		return 0;
 	if(mbwide())
 	{
-		register const char *str = string, *strmax=string+len;
-		register int n=0;
+		const char *str = string, *strmax=string+len;
+		int n=0;
 		mbinit();
 		if(len>0)
 		{
@@ -2652,54 +2692,53 @@ static int	charlen(const char *string,int len)
 		}
 		else while(mbchar(str))
 			n++;
-		return(n);
+		return n;
 	}
 	else
 	{
 		if(len<0)
-			return(strlen(string));
-		return(len);
+			return strlen(string);
+		return len;
 	}
 }
 
 /*
  * <offset> is byte offset for beginning of tilde string
  */
-static void tilde_expand2(register int offset)
+static void tilde_expand2(int offset)
 {
-	char		*cp = NIL(char*);		/* character pointer for tilde expansion result */
-	char		*stakp = stakptr(0);		/* current stack object (&stakp[offset] is tilde string) */
-	int		curoff = staktell();		/* current offset of current stack object */
-	static char	block;				/* for disallowing tilde expansion in .get/.set to change ${.sh.tilde} */
+	char		*cp = NULL;			/* character pointer for tilde expansion result */
+	char		*stakp = stkptr(sh.stk,0);	/* current stack object (&stakp[offset] is tilde string) */
+	int		curoff = stktell(sh.stk);	/* current offset of current stack object */
 	/*
 	 * Allow overriding tilde expansion with a .sh.tilde.set or .get discipline function.
 	 */
-	if(!block && SH_TILDENOD->nvfun && SH_TILDENOD->nvfun->disc)
+	if(!sh.tilde_block && SH_TILDENOD->nvfun && SH_TILDENOD->nvfun->disc)
 	{
-		stakfreeze(1);				/* terminate current stack object to avoid data corruption */
-		block++;
+		stkfreeze(sh.stk,1);			/* terminate current stack object to avoid data corruption */
+		sh.tilde_block = 1;
 		nv_putval(SH_TILDENOD, &stakp[offset], 0);
 		cp = nv_getval(SH_TILDENOD);
-		block--;
+		sh.tilde_block = 0;
 		if(cp[0]=='\0' || cp[0]=='~')
-			cp = NIL(char*);		/* do not use empty or unexpanded result */
-		stakset(stakp,curoff);			/* restore stack to state on function entry */
+			cp = NULL;			/* do not use empty or unexpanded result */
+		stkset(sh.stk,stakp,curoff);		/* restore stack to state on function entry */
 	}
 	/*
 	 * Perform default tilde expansion unless overridden.
 	 * Write the result to the stack, if any.
 	 */
-	stakputc(0);
+	sfputc(sh.stk,0);
 	if(!cp)
 		cp = sh_tilde(&stakp[offset]);
 	if(cp)
 	{
-		stakseek(offset);
+		stkseek(sh.stk,offset);
 		if(!(cp[0]=='/' && !cp[1] && fcpeek(0)=='/'))
-			stakputs(cp);			/* for ~ == /, avoid ~/foo -> //foo */
+			sfputr(sh.stk,cp,-1);		/* for ~ == /, avoid ~/foo -> //foo */
 	}
 	else
-		stakseek(curoff);
+		stkseek(sh.stk,curoff);
 }
 
 /*
@@ -2710,24 +2749,24 @@ static void tilde_expand2(register int offset)
  * If ~name  is replaced with login directory of name.
  * If string doesn't start with ~ or ~... not found then 0 returned.
  */
-static char *sh_tilde(register const char *string)
+static char *sh_tilde(const char *string)
 {
-	register char		*cp;
-	register int		c;
-	register struct passwd	*pw = NIL(struct passwd*);
-	register Namval_t *np=0;
-	unsigned int save;
-	static Dt_t *logins_tree;
+	char		*cp;
+	int		c;
+	struct passwd	*pw = NULL;
+	Namval_t	*np=0;
+	unsigned int	save;
+	static Dt_t	*logins_tree;
 	if(*string++!='~')
-		return(NIL(char*));
+		return NULL;
 	if((c = *string)==0)
 	{
 		static char	*username;
 		if(cp = nv_getval(sh_scoped(HOME)))
-			return(cp);
+			return cp;
 		/* Fallback for unset HOME: get username and treat ~ like ~username */
 		if(!username && !((pw = getpwuid(getuid())) && (username = sh_strdup(pw->pw_name))))
-			return(NIL(char*));
+			return NULL;
 		string = username;
 	}
 	if((c=='-' || c=='+') && string[1]==0)
@@ -2736,27 +2775,27 @@ static char *sh_tilde(register const char *string)
 			cp = nv_getval(sh_scoped(PWDNOD));
 		else
 			cp = nv_getval(sh_scoped(OLDPWDNOD));
-		return(cp);
+		return cp;
 	}
 #if _WINIX
 	if(fcgetc(c)=='/')
 	{
 		char	*str;
-		int	n=0,offset=staktell();
-		stakputs(string);
+		int	n=0,offset=stktell(sh.stk);
+		sfputr(sh.stk,string,-1);
 		do
 		{
-			stakputc(c);
+			sfputc(sh.stk,c);
 			n++;
 		}
 		while (fcgetc(c) && c!='/');
-		stakputc(0);
+		sfputc(sh.stk,0);
 		if(c)
 			fcseek(-1);
-		str = stakseek(offset);
+		str = stkseek(sh.stk,offset);
 		Skip = n;
 		if(logins_tree && (np=nv_search(str,logins_tree,0)))
-			return(nv_getval(np));
+			return nv_getval(np);
 		if(pw = getpwnam(str))
 		{
 			string = str;
@@ -2766,9 +2805,9 @@ static char *sh_tilde(register const char *string)
 	}
 #endif /* _WINIX */
 	if(logins_tree && (np=nv_search(string,logins_tree,0)))
-		return(nv_getval(np));
+		return nv_getval(np);
 	if(!pw && !(pw = getpwnam(string)))
-		return(NIL(char*));
+		return NULL;
 #if _WINIX
 skip:
 #endif /* _WINIX */
@@ -2781,13 +2820,13 @@ skip:
 		nv_putval(np, pw->pw_dir,0);
 		sh.subshell = save;
 	}
-	return(pw->pw_dir);
+	return pw->pw_dir;
 }
 
 /*
  * return values for special macros
  */
-static char *special(register int c)
+static char *special(int c)
 {
 	if(c!='$')
 		sh.argaddr = 0;
@@ -2795,37 +2834,36 @@ static char *special(register int c)
 	{
 	    case '@':
 	    case '*':
-		return(sh.st.dolc>0?sh.st.dolv[1]:NIL(char*));
+		return sh.st.dolc>0?sh.st.dolv[1]:NULL;
 	    case '#':
 #if  SHOPT_FILESCAN
 		if(sh.cur_line)
 		{
-			getdolarg(MAX_ARGN,(int*)0);
-			return(ltos(sh.offsets[0]));
+			getdolarg(MAX_ARGN,NULL);
+			return ltos(sh.offsets[0]);
 		}
 #endif  /* SHOPT_FILESCAN */
-		return(ltos(sh.st.dolc));
+		return ltos(sh.st.dolc);
 	    case '!':
 		if(sh.bckpid)
-			return(ltos(sh.bckpid));
+			return ltos(sh.bckpid);
 		break;
 	    case '$':
-		return(ltos(sh.pid));
+		return ltos(sh.pid);
 	    case '-':
-		return(sh_argdolminus(sh.arg_context));
+		return sh_argdolminus(sh.arg_context);
 	    case '?':
-		return(ltos(sh.savexit));
+		return ltos(sh.savexit);
 	    case 0:
 		if(sh_isstate(SH_PROFILE) || sh.fn_depth==0 || !sh.st.cmdname)
-			return(sh.shname);
+			return sh.shname;
 		else
-			return(sh.st.cmdname);
+			return sh.st.cmdname;
 	}
 	/* Handle 'set -u'/'set -o nounset' for special parameters */
 	if(sh_isoption(SH_NOUNSET))
 	{
-		int d=fcget();
-		fcseek(-1);
+		unsigned char d = fcpeek(0);
 		if(!(d && strchr(":+-?=",d)))
 		{
 			char c_str[2];
@@ -2835,7 +2873,7 @@ static char *special(register int c)
 			UNREACHABLE();
 		}
 	}
-	return(NIL(char*));
+	return NULL;
 }
 
 /*
@@ -2854,8 +2892,8 @@ static noreturn void mac_error(void)
  */ 
 static char *mac_getstring(char *pattern)
 {
-	register char	*cp=pattern, *rep=0, *dp;
-	register int	c;
+	char	*cp=pattern, *rep=0, *dp;
+	int	c;
 	while(c = *cp++)
 	{
 		if(c==ESCAPE && (!rep || (*cp && strchr("&|()[]*?",*cp))))
@@ -2873,5 +2911,5 @@ static char *mac_getstring(char *pattern)
 	}
 	if(rep)
 		*dp = 0;
-	return(rep);
+	return rep;
 }
