@@ -2,7 +2,7 @@
 *                                                                      *
 *               This software is part of the ast package               *
 *          Copyright (c) 1982-2012 AT&T Intellectual Property          *
-*          Copyright (c) 2020-2022 Contributors to ksh 93u+m           *
+*          Copyright (c) 2020-2023 Contributors to ksh 93u+m           *
 *                      and is licensed under the                       *
 *                 Eclipse Public License, Version 2.0                  *
 *                                                                      *
@@ -28,7 +28,6 @@
 #include	"shopt.h"
 #include	<ast.h>
 #include	<sfio.h>
-#include	<stak.h>
 #include	<ls.h>
 #include	<fcin.h>
 #include	"defs.h"
@@ -69,13 +68,13 @@ static time_t	mailtime;
 static char	beenhere = 0;
 
 #ifdef _lib_sigvec
-    void clearsigmask(register int sig)
+    void clearsigmask(int sig)
     {
 	struct sigvec vec;
-	if(sigvec(sig,NIL(struct sigvec*),&vec)>=0 && vec.sv_mask)
+	if(sigvec(sig,NULL,&vec)>=0 && vec.sv_mask)
 	{
 		vec.sv_mask = 0;
-		sigvec(sig,&vec,NIL(struct sigvec*));
+		sigvec(sig,&vec,NULL);
 	}
     }
 #endif /* _lib_sigvec */
@@ -90,14 +89,14 @@ static int sh_source(Sfio_t *iop, const char *file)
 	char*	nid;
 	int	fd;
 
-	if (!file || !*file || (fd = path_open(file, NIL(Pathcomp_t*))) < 0)
+	if (!file || !*file || (fd = path_open(file, NULL)) < 0)
 	{
 		REGRESS(source, "sh_source", ("%s:ENOENT", file));
 		return 0;
 	}
 	oid = error_info.id;
 	nid = error_info.id = sh_strdup(file);
-	sh.st.filename = path_fullname(stakptr(PATH_OFFSET));
+	sh.st.filename = path_fullname(stkptr(sh.stk,PATH_OFFSET));
 	REGRESS(source, "sh_source", ("%s", file));
 	exfile(iop, fd);
 	error_info.id = oid;
@@ -113,12 +112,13 @@ static int sh_source(Sfio_t *iop, const char *file)
 
 int sh_main(int ac, char *av[], Shinit_f userinit)
 {
-	register char	*name;
-	register int	fdin;
-	register Sfio_t  *iop;
+	char		*name;
+	int		fdin;
+	Sfio_t		*iop;
 	struct stat	statb;
-	int i, rshflag;		/* set for restricted shell */
-	char *command;
+	int		i;
+	int		rshflag;	/* set for restricted shell */
+	char		*command;
 #ifdef _lib_sigvec
 	/* This is to clear mask that may be left on by rlogin */
 	clearsigmask(SIGALRM);
@@ -136,12 +136,12 @@ int sh_main(int ac, char *av[], Shinit_f userinit)
 	if(sigsetjmp(*((sigjmp_buf*)sh.jmpbuffer),0))
 	{
 		/* begin script execution here */
-		sh_reinit((char**)0);
+		sh_reinit(NULL);
 	}
 	sh.fn_depth = sh.dot_depth = 0;
 	command = error_info.id;
 	path_pwd();
-	iop = (Sfio_t*)0;
+	iop = NULL;
 	if(sh_isoption(SH_POSIX))
 		sh_onoption(SH_LETOCTAL);
 #if SHOPT_BRACEPAT
@@ -210,7 +210,7 @@ int sh_main(int ac, char *av[], Shinit_f userinit)
 			if(!sh_isoption(SH_NOUSRPROFILE) && !sh_isoption(SH_PRIVILEGED) && sh_isoption(SH_RC))
 			{
 				if(name = sh_mactry(nv_getval(ENVNOD)))
-					name = *name ? sh_strdup(name) : (char*)0;
+					name = *name ? sh_strdup(name) : NULL;
 #if SHOPT_SYSRC
 				if(!strmatch(name, "?(.)/./*"))
 					sh_source(iop, e_sysrc);
@@ -233,7 +233,7 @@ int sh_main(int ac, char *av[], Shinit_f userinit)
 		if(sh.comdiv)
 		{
 		shell_c:
-			iop = sfnew(NIL(Sfio_t*),sh.comdiv,strlen(sh.comdiv),0,SF_STRING|SF_READ);
+			iop = sfnew(NULL,sh.comdiv,strlen(sh.comdiv),0,SF_STRING|SF_READ);
 		}
 		else
 		{
@@ -251,7 +251,7 @@ int sh_main(int ac, char *av[], Shinit_f userinit)
 					char *cp;
 					int type;
 #endif
-					fdin = (int)strtol(name+8, (char**)0, 10);
+					fdin = (int)strtol(name+8, NULL, 10);
 					if(fstat(fdin,&statb)<0)
 					{
 						errormsg(SH_DICT,ERROR_system(1),e_open,name);
@@ -292,8 +292,8 @@ int sh_main(int ac, char *av[], Shinit_f userinit)
 					sp = 0;
 					if(fdin < 0 && !strchr(name,'/'))
 					{
-						if(path_absolute(name,NIL(Pathcomp_t*),0))
-							sp = stakptr(PATH_OFFSET);
+						if(path_absolute(name,NULL,0))
+							sp = stkptr(sh.stk,PATH_OFFSET);
 						if(sp)
 						{
 							if((fdin=sh_open(sp,O_RDONLY,0))>=0)
@@ -332,7 +332,7 @@ int sh_main(int ac, char *av[], Shinit_f userinit)
 		}
 		/* If the shell is initialised with std{in,out,err} closed, make the shell's FD state reflect that. */
 		for(i=0; i<=2; i++)
-			if(fcntl(i,F_GETFD,NiL)==-1 && errno==EBADF)	/* closed at OS level? */
+			if(fcntl(i,F_GETFD,NULL)==-1 && errno==EBADF)	/* closed at OS level? */
 				sh_close(i); 				/* update shell FD state */
 	}
 	else
@@ -353,7 +353,7 @@ int sh_main(int ac, char *av[], Shinit_f userinit)
 	else
 	{
 		/* keep $COLUMNS and $LINES up to date even for scripts that don't trap SIGWINCH */
-		sh_winsize(NIL(int*),NIL(int*));
+		sh_winsize(NULL,NULL);
 #ifdef SIGWINCH
 		signal(SIGWINCH,sh_fault);
 #endif /* SIGWINCH */
@@ -375,7 +375,7 @@ int sh_main(int ac, char *av[], Shinit_f userinit)
  * iop is not null when the input is a string
  * fdin is the input file descriptor 
  */
-static void	exfile(register Sfio_t *iop,register int fno)
+static void	exfile(Sfio_t *iop,int fno)
 {
 	time_t curtime;
 	Shnode_t *t;
@@ -457,9 +457,7 @@ static void	exfile(register Sfio_t *iop,register int fno)
 				sfclose(top);
 		}
 		/* make sure that we own the terminal */
-#ifdef SIGTSTP
 		tcsetpgrp(job.fd,sh.pid);
-#endif /* SIGTSTP */
 	}
 	/* error return here */
 	sfclrerr(iop);
@@ -479,7 +477,7 @@ static void	exfile(register Sfio_t *iop,register int fno)
 	{
 		sh.nextprompt = 1;
 		sh_freeup();
-		stakset(NIL(char*),0);
+		stkset(sh.stk,NULL,0);
 		sh_offstate(SH_STOPOK);
 		sh_offstate(SH_ERREXIT);
 		sh_offstate(SH_VERBOSE);
@@ -498,17 +496,15 @@ static void	exfile(register Sfio_t *iop,register int fno)
 		}
 		if(sh_isstate(SH_INTERACTIVE) && !tdone)
 		{
-			register char *mail;
-#ifdef JOBS
+			char *mail;
 			sh_offstate(SH_MONITOR);
 			if(sh_isoption(SH_MONITOR))
 				sh_onstate(SH_MONITOR);
 			if(job.pwlist)
 			{
-				job_walk(sfstderr,job_list,JOB_NFLAG,(char**)0);
-				job_wait((pid_t)0);
+				job_walk(sfstderr,job_list,JOB_NFLAG,NULL);
+				job_wait(0);
 			}
-#endif	/* JOBS */
 			if((mail=nv_getval(MAILPNOD)) || (mail=nv_getval(MAILNOD)))
 			{
 				time(&curtime);
@@ -557,13 +553,19 @@ static void	exfile(register Sfio_t *iop,register int fno)
 					else if(job_close()<0)
 						continue;
 				}
-				else if(sferr==SH_EXITSIG)
+				else if(errno)
 				{
 					/* Ctrl+C with SIGINT ignored */
 					sfputc(sfstderr,'\n');
 					continue;
 				}
 			}
+			else if(errno && sferr)
+			{
+				/* Error reading from running script; panic */
+				errormsg(SH_DICT,ERROR_SYSTEM|ERROR_PANIC,e_readscript);
+				UNREACHABLE();
+			}
 			if(errno==0 && sferr && --maxtry>0)
 			{
 				sfclrlock(iop);
@@ -576,7 +578,7 @@ static void	exfile(register Sfio_t *iop,register int fno)
 		maxtry = IOMAXTRY;
 		if(sh_isstate(SH_INTERACTIVE) && sh.hist_ptr)
 		{
-			job_wait((pid_t)0);
+			job_wait(0);
 			hist_eof(sh.hist_ptr);
 			sfsync(sfstderr);
 		}
@@ -628,7 +630,7 @@ done:
 	if(fno>0)
 		sh_close(fno);
 	if(sh.st.filename)
-		free((void*)sh.st.filename);
+		free(sh.st.filename);
 	sh.st.filename = 0;
 }
 
@@ -636,11 +638,11 @@ done:
 /* prints out messages if files in list have been modified since last call */
 static void chkmail(char *files)
 {
-	register char *cp,*sp,*qp;
-	register char save;
-	struct argnod *arglist=0;
-	int	offset = staktell();
-	char 	*savstak=stakptr(0);
+	char		*cp,*sp,*qp;
+	char		save;
+	struct argnod	*arglist=0;
+	int		offset = stktell(sh.stk);
+	char	 	*savstak = stkptr(sh.stk,0);
 	struct stat	statb;
 	if(*(cp=files) == 0)
 		return;
@@ -707,7 +709,7 @@ static void chkmail(char *files)
 		cp = sp;
 	}
 	while(save);
-	stakset(savstak,offset);
+	stkset(sh.stk,savstak,offset);
 }
 
 #undef EXECARGS
@@ -755,8 +757,8 @@ static void fixargs(char **argv, int mode)
 		command_len = st.command_length;
 		return;
 	}
-	stakseek(command_len+2);
-	buff = stakseek(0);
+	stkseek(sh.stk,command_len+2);
+	buff = stkseek(sh.stk,0);
 	if(command_len==0)
 		return;
 	while((cp = *argv++) && offset < command_len)
@@ -769,7 +771,7 @@ static void fixargs(char **argv, int mode)
 	}
 	offset--;
 	memset(&buff[offset], 0, command_len - offset + 1);
-	un.pst_command = stakptr(0);
+	un.pst_command = stkptr(sh.stk,0);
 	pstat(PSTAT_SETCMD,un,0,0,0);
 #   elif _lib_setproctitle
 #	define CMDMAXLEN 255
