@@ -2,7 +2,7 @@
 *                                                                      *
 *               This software is part of the ast package               *
 *          Copyright (c) 1982-2012 AT&T Intellectual Property          *
-*          Copyright (c) 2020-2022 Contributors to ksh 93u+m           *
+*          Copyright (c) 2020-2023 Contributors to ksh 93u+m           *
 *                      and is licensed under the                       *
 *                 Eclipse Public License, Version 2.0                  *
 *                                                                      *
@@ -34,18 +34,18 @@ static void assign(Namval_t*,const char*,int,Namfun_t*);
 int nv_compare(Dt_t* dict, void *sp, void *dp, Dtdisc_t *disc)
 {
 	if(sp==dp)
-		return(0);
-	return(strcmp((char*)sp,(char*)dp));
+		return 0;
+	return strcmp((char*)sp,(char*)dp);
 }
 
 /*
  * call the next getval function in the chain
  */
-char *nv_getv(Namval_t *np, register Namfun_t *nfp)
+char *nv_getv(Namval_t *np, Namfun_t *nfp)
 {
-	register Namfun_t	*fp;
-	register char *cp;
-	if((fp = nfp) != NIL(Namfun_t*) && !nv_local)
+	Namfun_t	*fp;
+	char		*cp;
+	if((fp = nfp) != NULL && !nv_local)
 		fp = nfp = nfp->next;
 	nv_local=0;
 	for(; fp; fp=fp->next)
@@ -67,18 +67,18 @@ char *nv_getv(Namval_t *np, register Namfun_t *nfp)
 		nv_local=1;
 		cp = nv_getval(np);
 	}
-	return(cp);
+	return cp;
 }
 
 /*
  * call the next getnum function in the chain
  */
-Sfdouble_t nv_getn(Namval_t *np, register Namfun_t *nfp)
+Sfdouble_t nv_getn(Namval_t *np, Namfun_t *nfp)
 {
-	register Namfun_t	*fp;
-	register Sfdouble_t	d=0;
+	Namfun_t	*fp;
+	Sfdouble_t	d=0;
 	char *str;
-	if((fp = nfp) != NIL(Namfun_t*) && !nv_local)
+	if((fp = nfp) != NULL && !nv_local)
 		fp = nfp = nfp->next;
 	nv_local=0;
 	for(; fp; fp=fp->next)
@@ -106,17 +106,17 @@ Sfdouble_t nv_getn(Namval_t *np, register Namfun_t *nfp)
 		if(str && *str)
 			d = sh_arith(str);
 	}
-	return(d);
+	return d;
 }
 
 /*
  * call the next assign function in the chain
  */
-void nv_putv(Namval_t *np, const char *value, int flags, register Namfun_t *nfp)
+void nv_putv(Namval_t *np, const char *value, int flags, Namfun_t *nfp)
 {
-	register Namfun_t	*fp, *fpnext;
-	Namarr_t		*ap;
-	if((fp=nfp) != NIL(Namfun_t*) && !nv_local)
+	Namfun_t	*fp, *fpnext;
+	Namarr_t	*ap;
+	if((fp=nfp) != NULL && !nv_local)
 		fp = nfp = nfp->next;
 	nv_local=0;
 	if(flags&NV_NODISC)
@@ -131,7 +131,7 @@ void nv_putv(Namval_t *np, const char *value, int flags, register Namfun_t *nfp)
 				if(fp->disc || !(fp->nofree&1))
 					nv_disc(np,fp,NV_POP);
 				if(!(fp->nofree&1))
-					free((void*)fp);
+					free(fp);
 			}
 			continue;
 		}
@@ -184,15 +184,15 @@ static struct blocked	*blist;
  */
 static struct blocked *block_info(Namval_t *np, struct blocked *pp)
 {
-	register struct blocked	*bp;
-	void			*sub=0;
-	int			isub=0;
+	struct blocked	*bp;
+	void		*sub=0;
+	int		isub=0;
 	if(nv_isarray(np) && (isub=nv_aindex(np)) < 0)
-		sub = nv_associative(np,(const char*)0,NV_ACURRENT);
+		sub = nv_associative(np,NULL,NV_ACURRENT);
 	for(bp=blist ; bp; bp=bp->next)
 	{
 		if(bp->np==np && bp->sub==sub && bp->isub==isub)
-			return(bp);
+			return bp;
 	}
 	if(pp)
 	{
@@ -203,7 +203,7 @@ static struct blocked *block_info(Namval_t *np, struct blocked *pp)
 		pp->next = blist;
 		blist = pp;
 	}
-	return(pp);
+	return pp;
 }
 
 static void block_done(struct blocked *bp)
@@ -216,9 +216,9 @@ static void block_done(struct blocked *bp)
 /*
  * free discipline if no more discipline functions
  */
-static void chktfree(register Namval_t *np, register struct vardisc *vp)
+static void chktfree(Namval_t *np, struct vardisc *vp)
 {
-	register int n;
+	int n;
 	for(n=0; n< sizeof(vp->disc)/sizeof(*vp->disc); n++)
 	{
 		if(vp->disc[n])
@@ -228,8 +228,8 @@ static void chktfree(register Namval_t *np, register struct vardisc *vp)
 	{
 		/* no disc left so pop */
 		Namfun_t *fp;
-		if((fp=nv_stack(np, NIL(Namfun_t*))) && !(fp->nofree&1))
-			free((void*)fp);
+		if((fp=nv_stack(np, NULL)) && !(fp->nofree&1))
+			free(fp);
 	}
 }
 
@@ -239,12 +239,13 @@ static void chktfree(register Namval_t *np, register struct vardisc *vp)
 static void	assign(Namval_t *np,const char* val,int flags,Namfun_t *handle)
 {
 	int		type = (flags&NV_APPEND)?APPEND:ASSIGN;
-	register	struct vardisc *vp = (struct vardisc*)handle;
-	register	Namval_t *nq =  vp->disc[type];
+	struct vardisc *vp = (struct vardisc*)handle;
+	Namval_t *nq =  vp->disc[type];
 	struct blocked	block, *bp = block_info(np, &block);
 	Namval_t	node;
 	union Value	*up = np->nvalue.up;
 	Namval_t	*tp, *nr;  /* for 'typeset -T' types */
+	int		jmpval = 0;
 	if(val && (tp=nv_type(np)) && (nr=nv_open(val,sh.var_tree,NV_VARNAME|NV_ARRAY|NV_NOADD|NV_NOFAIL)) && tp==nv_type(nr)) 
 	{
 		char *sub = nv_getsub(np);
@@ -280,7 +281,6 @@ static void	assign(Namval_t *np,const char* val,int flags,Namfun_t *handle)
 	if(nq && !isblocked(bp,type))
 	{
 		struct checkpt	checkpoint;
-		int		jmpval;
 		int		savexit = sh.savexit;
 		Lex_t		*lexp = (Lex_t*)sh.lex_context, savelex;
 		int		bflag;
@@ -290,10 +290,10 @@ static void	assign(Namval_t *np,const char* val,int flags,Namfun_t *handle)
 		block(bp,type);
 		if(bflag = (type==APPEND && !isblocked(bp,LOOKUPS)))
 			block(bp,LOOKUPS);
-		sh_pushcontext(&checkpoint, 1);
+		sh_pushcontext(&checkpoint, SH_JMPFUN);
 		jmpval = sigsetjmp(checkpoint.buff, 0);
 		if(!jmpval)
-			sh_fun(nq,np,(char**)0);
+			sh_fun(nq,np,NULL);
 		sh_popcontext(&checkpoint);
 		if(sh.topfd != checkpoint.topfd)
 			sh_iorestore(checkpoint.topfd, jmpval);
@@ -309,7 +309,7 @@ static void	assign(Namval_t *np,const char* val,int flags,Namfun_t *handle)
 		np->nvalue.up = up;
 	if(val)
 	{
-		register char *cp;
+		char *cp;
 		Sfdouble_t d;
 		if(nv_isnull(SH_VALNOD))
 			cp=0;
@@ -370,6 +370,8 @@ done:
 		nq->nvalue.rp->running=0;
 		_nv_unset(nq,0);
 	}
+	if(jmpval >= SH_JMPFUN)
+		siglongjmp(*sh.jmplist,jmpval);
 }
 
 /*
@@ -378,16 +380,16 @@ done:
  */
 static char*	lookup(Namval_t *np, int type, Sfdouble_t *dp,Namfun_t *handle)
 {
-	register struct vardisc	*vp = (struct vardisc*)handle;
-	struct blocked		block, *bp = block_info(np, &block);
-	register Namval_t	*nq = vp->disc[type];
-	register char		*cp=0;
-	Namval_t		node;
-	union Value		*up = np->nvalue.up;
+	struct vardisc	*vp = (struct vardisc*)handle;
+	struct blocked	block, *bp = block_info(np, &block);
+	Namval_t	*nq = vp->disc[type];
+	char		*cp=0;
+	Namval_t	node;
+	union Value	*up = np->nvalue.up;
+	int		jmpval = 0;
 	if(nq && !isblocked(bp,type))
 	{
 		struct checkpt	checkpoint;
-		int		jmpval;
 		int		savexit = sh.savexit;
 		Lex_t		*lexp = (Lex_t*)sh.lex_context, savelex;
 		/* disciplines like PS2 may run at parse time; save, reinit and restore the lexer state */
@@ -406,10 +408,10 @@ static char*	lookup(Namval_t *np, int type, Sfdouble_t *dp,Namfun_t *handle)
 		}
 		block(bp,type);
 		block(bp, UNASSIGN);   /* make sure nv_setdisc doesn't invalidate 'vp' by freeing it */
-		sh_pushcontext(&checkpoint, 1);
+		sh_pushcontext(&checkpoint, SH_JMPFUN);
 		jmpval = sigsetjmp(checkpoint.buff, 0);
 		if(!jmpval)
-			sh_fun(nq,np,(char**)0);
+			sh_fun(nq,np,NULL);
 		sh_popcontext(&checkpoint);
 		if(sh.topfd != checkpoint.topfd)
 			sh_iorestore(checkpoint.topfd, jmpval);
@@ -423,7 +425,7 @@ static char*	lookup(Namval_t *np, int type, Sfdouble_t *dp,Namfun_t *handle)
 			*dp = nv_getnum(SH_VALNOD);
 		}
 		else if(cp = nv_getval(SH_VALNOD))
-			cp = stkcopy(stkstd,cp);
+			cp = stkcopy(sh.stk,cp);
 		_nv_unset(SH_VALNOD,NV_RDONLY);
 		if(!nv_isnull(&node))
 		{
@@ -449,19 +451,22 @@ static char*	lookup(Namval_t *np, int type, Sfdouble_t *dp,Namfun_t *handle)
 		nq->nvalue.rp->running=0;
 		_nv_unset(nq,0);
 	}
-	return(cp);
+	if(jmpval >= SH_JMPFUN)
+		siglongjmp(*sh.jmplist,jmpval);
+	sh_sigcheck();
+	return cp;
 }
 
 static char*	lookups(Namval_t *np, Namfun_t *handle)
 {
-	return(lookup(np,LOOKUPS,(Sfdouble_t*)0,handle));
+	return lookup(np,LOOKUPS,NULL,handle);
 }
 
 static Sfdouble_t lookupn(Namval_t *np, Namfun_t *handle)
 {
 	Sfdouble_t	d;
 	lookup(np,LOOKUPN, &d ,handle);
-	return(d);
+	return d;
 }
 
 
@@ -472,10 +477,10 @@ static Sfdouble_t lookupn(Namval_t *np, Namfun_t *handle)
  * If <event> is NULL, then return the event name after <action>
  * If <event> is NULL, and <action> is NULL, return the first event
  */
-char *nv_setdisc(register Namval_t* np,register const char *event,Namval_t *action,register Namfun_t *fp)
+char *nv_setdisc(Namval_t* np,const char *event,Namval_t *action,Namfun_t *fp)
 {
-	register struct vardisc *vp = (struct vardisc*)np->nvfun;
-	register int type = -1;
+	struct vardisc *vp = (struct vardisc*)np->nvfun;
+	int type = -1;
 	char *empty = "";
 	while(vp)
 	{
@@ -487,13 +492,13 @@ char *nv_setdisc(register Namval_t* np,register const char *event,Namval_t *acti
 		vp = 0;
 	if(np == (Namval_t*)fp)
 	{
-		register const char *name;
-		register int getname=0;
+		const char *name;
+		int getname=0;
 		/* top level call, check for get/set */
 		if(!event)
 		{
 			if(!action)
-				return((char*)nv_discnames[0]);
+				return (char*)nv_discnames[0];
 			getname=1;
 			event = (char*)action;
 		}
@@ -513,44 +518,44 @@ char *nv_setdisc(register Namval_t* np,register const char *event,Namval_t *acti
 			for(fp=(Namfun_t*)vp; fp; fp=fp->next)
 			{
 				if(fp->disc && fp->disc->setdisc)
-					return((*fp->disc->setdisc)(np,event,action,fp));
+					return (*fp->disc->setdisc)(np,event,action,fp);
 			}
 		}
 		else if(getname)
-			return((char*)name);
+			return (char*)name;
 	}
 	if(!fp)
-		return(NIL(char*));
+		return NULL;
 	if(np != (Namval_t*)fp)
 	{
 		/* not the top level */
 		while(fp = fp->next)
 		{
 			if(fp->disc && fp->disc->setdisc)
-				return((*fp->disc->setdisc)(np,event,action,fp));
+				return (*fp->disc->setdisc)(np,event,action,fp);
 		}
-		return(NIL(char*));
+		return NULL;
 	}
 	if (type < 0)
-		return(NIL(char*));
+		return NULL;
 	/* Handle GET/SET/APPEND/UNSET disc */
 	if(np==SH_VALNOD || np==SH_LEVELNOD)
-		return(0);
+		return NULL;
 	if(vp && vp->fun.disc->putval!=assign)
 		vp = 0;
 	if(!vp)
 	{
 		Namdisc_t	*dp;
 		if(action==np)
-			return((char*)action);
-		vp = sh_newof(NIL(struct vardisc*),struct vardisc,1,sizeof(Namdisc_t));
+			return (char*)action;
+		vp = sh_newof(NULL,struct vardisc,1,sizeof(Namdisc_t));
 		dp = (Namdisc_t*)(vp+1);
 		vp->fun.disc = dp;
 		memset(dp,0,sizeof(*dp));
 		dp->dsize = sizeof(struct vardisc);
 		dp->putval = assign;
 		if(nv_isarray(np) && !nv_arrayptr(np))
-			nv_putsub(np,(char*)0, 1);
+			nv_putsub(np,NULL, 1);
 		nv_stack(np, (Namfun_t*)vp);
 	}
 	if(action==np)
@@ -566,16 +571,19 @@ char *nv_setdisc(register Namval_t* np,register const char *event,Namval_t *acti
 		else if(type==LOOKUPN)
 			dp->getnum = lookupn;
 		vp->disc[type] = action;
+#if SHOPT_OPTIMIZE
+		nv_optimize_clear(np);
+#endif
 	}
 	else
 	{
 		struct blocked *bp;
 		action = vp->disc[type];
 		vp->disc[type] = 0;
-		if(!(bp=block_info(np,(struct blocked*)0)) || !isblocked(bp,UNASSIGN))
+		if(!(bp=block_info(np,NULL)) || !isblocked(bp,UNASSIGN))
 			chktfree(np,vp);
 	}
-	return(action?(char*)action:empty);
+	return action ? (char*)action : empty;
 }
 
 /*
@@ -585,17 +593,17 @@ char *nv_setdisc(register Namval_t* np,register const char *event,Namval_t *acti
  * If <event> is NULL, then return the event name after <action>
  * If <event> is NULL, and <action> is NULL, return the first event
  */
-static char *setdisc(register Namval_t* np,register const char *event,Namval_t *action,register Namfun_t *fp)
+static char *setdisc(Namval_t* np,const char *event,Namval_t *action,Namfun_t *fp)
 {
-	register Nambfun_t *vp = (Nambfun_t*)fp;
-	register int type,getname=0;
-	register const char *name;
+	Nambfun_t *vp = (Nambfun_t*)fp;
+	int type,getname=0;
+	const char *name;
 	const char **discnames = vp->bnames;
 	/* top level call, check for discipline match */
 	if(!event)
 	{
 		if(!action)
-			return((char*)discnames[0]);
+			return (char*)discnames[0];
 		getname=1;
 		event = (char*)action;
 	}
@@ -611,9 +619,9 @@ static char *setdisc(register Namval_t* np,register const char *event,Namval_t *
 			action = 0;
 	}
 	if(!name)
-		return(nv_setdisc(np,event,action,fp));
+		return nv_setdisc(np,event,action,fp);
 	else if(getname)
-		return((char*)name);
+		return (char*)name;
 	/* Handle the disciplines */
 	if(action==np)
 		action = vp->bltins[type];
@@ -632,7 +640,7 @@ static char *setdisc(register Namval_t* np,register const char *event,Namval_t *
 		action = vp->bltins[type];
 		vp->bltins[type] = 0;
 	}
-	return((char*)action);
+	return (char*)action;
 }
 
 static void putdisc(Namval_t* np, const char* val, int flag, Namfun_t* fp)
@@ -640,67 +648,67 @@ static void putdisc(Namval_t* np, const char* val, int flag, Namfun_t* fp)
 	nv_putv(np,val,flag,fp);
 	if(!val && !(flag&NV_NOFREE))
 	{
-		register Nambfun_t *vp = (Nambfun_t*)fp;
-		register int i;
+		Nambfun_t *vp = (Nambfun_t*)fp;
+		int i;
 		for(i=0; vp->bnames[i]; i++)
 		{
-			register Namval_t *mp;
+			Namval_t *mp;
 			if((mp=vp->bltins[i]) && !nv_isattr(mp,NV_NOFREE))
 			{
 				if(is_abuiltin(mp))
 				{
 					if(mp->nvfun && !nv_isattr(mp,NV_NOFREE))
-						free((void*)mp->nvfun);
+						free(mp->nvfun);
 					dtdelete(sh.bltin_tree,mp);
-					free((void*)mp);
+					free(mp);
 				}
 			}
 		}
 		nv_disc(np,fp,NV_POP);
 		if(!(fp->nofree&1))
-			free((void*)fp);
+			free(fp);
 	}
 }
 
 static const Namdisc_t Nv_bdisc	= {   0, putdisc, 0, 0, setdisc };
 
-Namfun_t *nv_clone_disc(register Namfun_t *fp, int flags)
+Namfun_t *nv_clone_disc(Namfun_t *fp, int flags)
 {
-	register Namfun_t	*nfp;
-	register int		size;
+	Namfun_t	*nfp;
+	int		size;
 	if(!fp->disc && !fp->next && (fp->nofree&1))
-		return(fp);
+		return fp;
 	if(!(size=fp->dsize) && (!fp->disc || !(size=fp->disc->dsize)))
 		size = sizeof(Namfun_t);
-	nfp = sh_newof(NIL(Namfun_t*),Namfun_t,1,size-sizeof(Namfun_t));
+	nfp = sh_newof(NULL,Namfun_t,1,size-sizeof(Namfun_t));
 	memcpy(nfp,fp,size);
 	nfp->nofree &= ~1;
 	nfp->nofree |= (flags&NV_RDONLY)?1:0;
-	return(nfp);
+	return nfp;
 }
 
 int nv_adddisc(Namval_t *np, const char **names, Namval_t **funs)
 {
-	register Nambfun_t *vp;
-	register int n=0;
-	register const char **av=names;
+	Nambfun_t *vp;
+	int n=0;
+	const char **av=names;
 	if(av)
 	{
 		while(*av++)
 			n++;
 	}
-	vp = sh_newof(NIL(Nambfun_t*),Nambfun_t,1,n*sizeof(Namval_t*));
+	vp = sh_newof(NULL,Nambfun_t,1,n*sizeof(Namval_t*));
 	vp->fun.dsize = sizeof(Nambfun_t)+n*sizeof(Namval_t*);
 	vp->fun.nofree |= 2;
 	vp->num = n;
 	if(funs)
-		memcpy((void*)vp->bltins, (void*)funs,n*sizeof(Namval_t*));
+		memcpy(vp->bltins, funs,n*sizeof(Namval_t*));
 	else while(n>=0)
 		vp->bltins[n--] = 0;
 	vp->fun.disc = &Nv_bdisc;
 	vp->bnames = names; 
 	nv_stack(np,&vp->fun);
-	return(1);
+	return 1;
 }
 
 /*
@@ -711,13 +719,13 @@ int nv_adddisc(Namval_t *np, const char **names, Namval_t **funs)
  *    NV_POP:	 Delete <fp> from top of the stack
  *    NV_CLONE:  Replace fp with a copy created my malloc() and return it
  */
-Namfun_t *nv_disc(register Namval_t *np, register Namfun_t* fp, int mode)
+Namfun_t *nv_disc(Namval_t *np, Namfun_t* fp, int mode)
 {
 	Namfun_t *lp, **lpp;
 	if(nv_isref(np))
-		return(0);
+		return NULL;
 	if(mode==NV_CLONE && !fp)
-		return(0);
+		return NULL;
 	if(fp)
 	{
 		fp->subshell = sh.subshell;
@@ -726,15 +734,15 @@ Namfun_t *nv_disc(register Namval_t *np, register Namfun_t* fp, int mode)
 			if(mode==NV_CLONE)
 			{
 				lp = nv_clone_disc(fp,0);
-				return(np->nvfun=lp);
+				return np->nvfun=lp;
 			}
 			if(mode==NV_FIRST || mode==0)
-				return(fp);
+				return fp;
 			np->nvfun = lp->next;
 			if(mode==NV_POP)
-				return(fp);
+				return fp;
 			if(mode==NV_LAST && (lp->next==0 || lp->next->disc==0))
-				return(fp);
+				return fp;
 		}
 		/* see if <fp> is on the list already */
 		lpp = &np->nvfun;
@@ -745,16 +753,16 @@ Namfun_t *nv_disc(register Namval_t *np, register Namfun_t* fp, int mode)
 				if(lp->next==fp)
 				{
 					if(mode==NV_LAST && fp->next==0)
-						return(fp);
+						return fp;
 					if(mode==NV_CLONE)
 					{
 						fp = nv_clone_disc(fp,0);
 						lp->next = fp;
-						return(fp);
+						return fp;
 					}
 					lp->next = fp->next;
 					if(mode==NV_POP)
-						return(fp);
+						return fp;
 					if(mode!=NV_LAST)
 						break;
 				}
@@ -764,7 +772,7 @@ Namfun_t *nv_disc(register Namval_t *np, register Namfun_t* fp, int mode)
 				lpp = &lp->next;
 		}
 		if(mode==NV_POP)
-			return(0);
+			return NULL;
 		/* push */
 		nv_offattr(np,NV_NODISC);
 		if(mode==NV_LAST)
@@ -785,13 +793,13 @@ Namfun_t *nv_disc(register Namval_t *np, register Namfun_t* fp, int mode)
 	else
 	{
 		if(mode==NV_FIRST)
-			return(np->nvfun);
+			return np->nvfun;
 		else if(mode==NV_LAST)
 			for(lp=np->nvfun; lp; fp=lp,lp=lp->next);
 		else if(fp = np->nvfun)
 			np->nvfun = fp->next;
 	}
-	return(fp);
+	return fp;
 }
 
 /*
@@ -800,33 +808,33 @@ Namfun_t *nv_disc(register Namval_t *np, register Namfun_t* fp, int mode)
  */
 Namfun_t *nv_hasdisc(Namval_t *np, const Namdisc_t *dp)
 {
-	register Namfun_t *fp;
+	Namfun_t *fp;
 	for(fp=np->nvfun; fp; fp = fp->next)
 	{
 		if(fp->disc== dp)
-			return(fp);
+			return fp;
 	}
-	return(0);
+	return NULL;
 }
 
 static void *newnode(const char *name)
 {
-	register int s;
-	register Namval_t *np = sh_newof(0,Namval_t,1,s=strlen(name)+1);
+	int s;
+	Namval_t *np = sh_newof(0,Namval_t,1,s=strlen(name)+1);
 	np->nvname = (char*)np+sizeof(Namval_t);
 	memcpy(np->nvname,name,s);
-	return((void*)np);
+	return np;
 }
 
 /*
  * clone a numeric value
  */
-static void *num_clone(register Namval_t *np, void *val)
+static void *num_clone(Namval_t *np, void *val)
 {
-	register int size;
+	int size;
 	void *nval;
 	if(!val)
-		return(0);
+		return NULL;
 	if(nv_isattr(np,NV_DOUBLE)==NV_DOUBLE)
 	{
 		if(nv_isattr(np,NV_LONG))
@@ -845,19 +853,19 @@ static void *num_clone(register Namval_t *np, void *val)
 			if(nv_isattr(np,NV_INT16P|NV_DOUBLE)==NV_INT16P)
 				size = sizeof(short);
 			else
-				return((void*)np->nvalue.ip);
+				return np->nvalue.ip;
 		}
 		else
 			size = sizeof(int32_t);
 	}
 	nval = sh_malloc(size);
 	memcpy(nval,val,size);
-	return(nval);
+	return nval;
 }
 
 void clone_all_disc( Namval_t *np, Namval_t *mp, int flags)
 {
-	register Namfun_t *fp, **mfp = &mp->nvfun, *nfp, *fpnext;
+	Namfun_t *fp, **mfp = &mp->nvfun, *nfp, *fpnext;
 	for(fp=np->nvfun; fp;fp=fpnext)
 	{
 		fpnext = fp->next;
@@ -899,7 +907,7 @@ int nv_clone(Namval_t *np, Namval_t *mp, int flags)
 		if(!fpnext && (flags&NV_COMVAR) && fp->disc && fp->disc->namef)
 			break;
 		if(!(fp->nofree&1))
-			free((void*)fp);
+			free(fp);
 	}
 	mp->nvfun = fp;
 	if(fp=np->nvfun)
@@ -917,7 +925,7 @@ int nv_clone(Namval_t *np, Namval_t *mp, int flags)
 		clone_all_disc(np, mp, flags);
 	}
 	if(flags&NV_APPEND)
-		return(1);
+		return 1;
 	if(mp->nvsize == size)
 		nv_setsize(mp,nv_size(np));
 	if(mp->nvflag == flag)
@@ -957,18 +965,18 @@ int nv_clone(Namval_t *np, Namval_t *mp, int flags)
 		else
 			np->nvflag &= NV_MINIMAL;
 	        nv_setsize(np,0);
-		return(1);
+		return 1;
 	}
 	else if((flags&NV_ARRAY) && !nv_isattr(np,NV_MINIMAL))
 		mp->nvenv = np->nvenv;
 	if(nv_isattr(np,NV_INTEGER) && !nv_isarray(np) && mp->nvalue.ip!=np->nvalue.ip && np->nvalue.cp!=Empty)
 	{
-		mp->nvalue.ip = (int*)num_clone(np,(void*)np->nvalue.ip);
+		mp->nvalue.ip = (int*)num_clone(np,np->nvalue.ip);
 		nv_offattr(mp,NV_NOFREE);
 	}
 	else if((flags&NV_NOFREE) && !nv_arrayptr(np))
 	        nv_onattr(np,NV_NOFREE);
-	return(1);
+	return 1;
 }
 
 /*
@@ -984,8 +992,8 @@ int nv_clone(Namval_t *np, Namval_t *mp, int flags)
  */
 Namval_t *nv_search(const char *name, Dt_t *root, int mode)
 {
-	register Namval_t *np;
-	register Dt_t *dp = 0;
+	Namval_t *np;
+	Dt_t *dp = 0;
 	if(mode&NV_NOSCOPE)
 		dp = dtview(root,0);
 	if(mode&NV_REF)
@@ -998,38 +1006,38 @@ Namval_t *nv_search(const char *name, Dt_t *root, int mode)
 	{
 		if(*name=='.' && root==sh.var_tree && !dp)
 			root = sh.var_base;
-		np = dtmatch(root,(void*)name);
+		np = dtmatch(root,name);
 	}
+	/* skip dummy shell function node (function unset in virtual subshell) */
+	if(np && !np->nvflag && root==sh.fun_tree)
+		np = mode&NV_NOSCOPE ? NULL : dtmatch(sh.bltin_tree,name);
 	if(!np && (mode&NV_ADD))
 	{
 		if(sh.namespace && !(mode&NV_NOSCOPE) && root==sh.var_tree)
 			root = nv_dict(sh.namespace);
 		else if(!dp && !(mode&NV_NOSCOPE))
 		{
-			register Dt_t *next;
+			Dt_t *next;
 			while(next=dtvnext(root))
 				root = next;
 		}
-		np = (Namval_t*)dtinsert(root,newnode(name));
+		np = (Namval_t*)dtinstall(root,newnode(name));
 	}
 	if(dp)
 		dtview(root,dp);
-	return(np);
+	return np;
 }
 
 /*
- * finds function or builtin for given name and the discipline variable
- * if var!=0 the variable pointer is returned and the built-in name
- *    is put onto the stack at the current offset.
- * otherwise, a pointer to the builtin (variable or type) is returned
- * and var contains the pointer to the variable
- * if last==0 and first component of name is a reference, nv_bfsearch()
-	will return 0.
+ * Finds function or builtin for given name and the discipline variable.
+ * If var!=0, the variable pointer is returned and the built-in name is put onto the stack at the current offset.
+ * Otherwise, a pointer to the built-in (variable or type) is returned and var contains the pointer to the variable.
+ * If last==0 and first component of name is a reference, nv_bfsearch() will return NULL.
  */ 
 Namval_t *nv_bfsearch(const char *name, Dt_t *root, Namval_t **var, char **last)
 {
-	int		c,offset = staktell();
-	register char	*sp, *cp=0;
+	int		c,offset = stktell(sh.stk);
+	char		*sp, *cp=0;
 	Namval_t	*np, *nq;
 	char		*dname=0;
 	if(var)
@@ -1038,35 +1046,34 @@ Namval_t *nv_bfsearch(const char *name, Dt_t *root, Namval_t **var, char **last)
 	for(sp=(char*)name+1; *sp; sp++) 
 	{
 		if(*sp=='=')
-			return(0);
+			return NULL;
 		if(*sp=='[')
 		{
 			while(*sp=='[')
 			{
-				sp = nv_endsubscript((Namval_t*)0,(char*)sp,0);
+				sp = nv_endsubscript(NULL,(char*)sp,0);
 				if(sp[-1]!=']')
-					return(0);
+					return NULL;
 			}
 			if(*sp==0)
 				break;
 			if(*sp!='.')
-				return(0);
+				return NULL;
 			cp = sp;
 		}
 		else if(*sp=='.')
 			cp = sp; 
 	}
 	if(!cp)
-		return(var?nv_search(name,root,0):0);
-	stakputs(name);
-	stakputc(0);
+		return var ? nv_search(name,root,0) : NULL;
+	sfputr(sh.stk,name,0);
 	dname = cp+1;
-	cp = stakptr(offset) + (cp-name); 
+	cp = stkptr(sh.stk,offset) + (cp-name); 
 	if(last)
 		*last = cp;
 	c = *cp;
 	*cp = 0;
-	nq=nv_open(stakptr(offset),0,NV_VARNAME|NV_NOADD|NV_NOFAIL);
+	nq=nv_open(stkptr(sh.stk,offset),0,NV_VARNAME|NV_NOADD|NV_NOFAIL);
 	*cp = c;
 	if(!nq)
 	{
@@ -1081,29 +1088,28 @@ Namval_t *nv_bfsearch(const char *name, Dt_t *root, Namval_t **var, char **last)
 	*var = nq;
 	if(c=='[')
 		nv_endsubscript(nq, cp,NV_NOADD);
-	stakseek(offset);
+	stkseek(sh.stk,offset);
 #if SHOPT_NAMESPACE
 	if(nv_istable(nq))
 	{
 		Namval_t *nsp = sh.namespace;
 		if(last==0)
-			return(nv_search(name,root,0));
+			return nv_search(name,root,0);
 		sh.namespace = 0;
-		stakputs(nv_name(nq));
+		sfputr(sh.stk,nv_name(nq),-1);
 		sh.namespace = nsp;
-		stakputs(dname-1);
-		stakputc(0);
-		np = nv_search(stakptr(offset),root,0);
-		stakseek(offset);
-		return(np);
+		sfputr(sh.stk,dname-1,0);
+		np = nv_search(stkptr(sh.stk,offset),root,0);
+		stkseek(sh.stk,offset);
+		return np;
 	}
 #endif /* SHOPT_NAMESPACE */
 	while(nv_isarray(nq) && !nv_isattr(nq,NV_MINIMAL|NV_EXPORT) && nq->nvenv && nv_isarray((Namval_t*)nq->nvenv))
 		nq = (Namval_t*)nq->nvenv;
-	return((Namval_t*)nv_setdisc(nq,dname,nq,(Namfun_t*)nq));
+	return (Namval_t*)nv_setdisc(nq,dname,nq,(Namfun_t*)nq);
 done:
-	stakseek(offset);
-	return(np);
+	stkseek(sh.stk,offset);
+	return np;
 }
 
 /*
@@ -1117,25 +1123,24 @@ done:
  */
 Namval_t *sh_addbuiltin(const char *path, Shbltin_f bltin, void *extra)
 {
-	register const char	*name;
-	char			*cp;
-	register Namval_t	*np, *nq=0;
-	int			offset=staktell();
+	const char	*name;
+	char		*cp;
+	Namval_t	*np, *nq=0;
+	int		offset=stktell(sh.stk);
 	if(extra==(void*)1)
 		name = path;
-	else if((name = path_basename(path))==path && bltin!=b_typeset && (nq=nv_bfsearch(name,sh.bltin_tree,(Namval_t**)0,&cp)))
-		path = name = stakptr(offset);
+	else if((name = path_basename(path))==path && bltin!=b_typeset && (nq=nv_bfsearch(name,sh.bltin_tree,NULL,&cp)))
+		path = name = stkptr(sh.stk,offset);
 	else if(sh.bltin_dir && extra!=(void*)1)
 	{
-		stakputs(sh.bltin_dir);
-		stakputc('/');
-		stakputs(name);
-		path = stakptr(offset);
+		sfputr(sh.stk,sh.bltin_dir,'/');
+		sfputr(sh.stk,name,0);
+		path = stkptr(sh.stk,offset);
 	}
 	if(np = nv_search(name,sh.bltin_tree,0))
 	{
 		/* exists without a path */
-		stakseek(offset);
+		stkseek(sh.stk,offset);
 		if(extra == (void*)1)
 		{
 			if(nv_isattr(np,BLT_SPC))
@@ -1145,12 +1150,12 @@ Namval_t *sh_addbuiltin(const char *path, Shbltin_f bltin, void *extra)
 				UNREACHABLE();
 			}
 			if(np->nvfun && !nv_isattr(np,NV_NOFREE))
-				free((void*)np->nvfun);
+				free(np->nvfun);
 			dtdelete(sh.bltin_tree,np);
-			return(0);
+			return NULL;
 		}
 		if(!bltin)
-			return(np);
+			return np;
 	}
 	else for(np=(Namval_t*)dtfirst(sh.bltin_tree);np;np=(Namval_t*)dtnext(sh.bltin_tree,np))
 	{
@@ -1160,31 +1165,31 @@ Namval_t *sh_addbuiltin(const char *path, Shbltin_f bltin, void *extra)
 		if(strcmp(path,nv_name(np)))
 		{
 			if(nv_isattr(np,BLT_SPC))
-				return(np);
+				return np;
 			if(!bltin)
 				bltin = funptr(np);
 			if(np->nvenv)
 				dtdelete(sh.bltin_tree,np);
 			if(extra == (void*)1)
-				return(0);
-			np = 0;
+				return NULL;
+			np = NULL;
 		}
 		break;
 	}
 	if(!np && !(np = nv_search(path,sh.bltin_tree,bltin?NV_ADD:0)))
-		return(0);
-	stakseek(offset);
+		return NULL;
+	stkseek(sh.stk,offset);
 	if(nv_isattr(np,BLT_SPC))
 	{
 		if(extra)
 			np->nvfun = (Namfun_t*)extra;
-		return(np);
+		return np;
 	}
 	np->nvenv = 0;
 	np->nvfun = 0;
 	if(bltin)
 	{
-		np->nvalue.bfp = (void*)bltin;
+		np->nvalue.bfp = bltin;
 		nv_onattr(np,NV_BLTIN|NV_NOFREE);
 		np->nvfun = (Namfun_t*)extra;
 	}
@@ -1198,14 +1203,14 @@ Namval_t *sh_addbuiltin(const char *path, Shbltin_f bltin, void *extra)
 		}
 	}
 	if(extra == (void*)1)
-		return(0);
-	return(np);
+		return NULL;
+	return np;
 }
 
 #undef nv_stack
-extern Namfun_t *nv_stack(register Namval_t *np, register Namfun_t* fp)
+extern Namfun_t *nv_stack(Namval_t *np, Namfun_t* fp)
 {
-	return(nv_disc(np,fp,0));
+	return nv_disc(np,fp,0);
 }
 
 struct table
@@ -1215,20 +1220,20 @@ struct table
 	Dt_t		*dict;
 };
 
-static Namval_t *next_table(register Namval_t* np, Dt_t *root,Namfun_t *fp)
+static Namval_t *next_table(Namval_t* np, Dt_t *root,Namfun_t *fp)
 {
 	struct table *tp = (struct table *)fp;
 	if(root)
-		return((Namval_t*)dtnext(root,np));
+		return (Namval_t*)dtnext(root,np);
 	else
-		return((Namval_t*)dtfirst(tp->dict));
+		return (Namval_t*)dtfirst(tp->dict);
 }
 
 static Namval_t *create_table(Namval_t *np,const char *name,int flags,Namfun_t *fp)
 {
 	struct table *tp = (struct table *)fp;
 	sh.last_table = np;
-	return(nv_create(name, tp->dict, flags, fp));
+	return nv_create(name, tp->dict, flags, fp);
 }
 
 static Namfun_t *clone_table(Namval_t* np, Namval_t *mp, int flags, Namfun_t *fp)
@@ -1237,8 +1242,8 @@ static Namfun_t *clone_table(Namval_t* np, Namval_t *mp, int flags, Namfun_t *fp
 	struct table	*ntp = (struct table*)nv_clone_disc(fp,0);
 	Dt_t		*oroot=tp->dict,*nroot=dtopen(&_Nvdisc,Dtoset);
 	if(!nroot)
-		return(0);
-	memcpy((void*)ntp,(void*)fp,sizeof(struct table));
+		return NULL;
+	memcpy(ntp,fp,sizeof(struct table));
 	ntp->dict = nroot;
 	ntp->parent = nv_lastdict();
 	for(np=(Namval_t*)dtfirst(oroot);np;np=(Namval_t*)dtnext(oroot,np))
@@ -1246,7 +1251,7 @@ static Namfun_t *clone_table(Namval_t* np, Namval_t *mp, int flags, Namfun_t *fp
 		mp = (Namval_t*)dtinsert(nroot,newnode(np->nvname));
 		nv_clone(np,mp,flags);
 	}
-	return(&ntp->fun);
+	return &ntp->fun;
 }
 
 /*
@@ -1267,12 +1272,12 @@ static void delete_fun(Namval_t *np, void *data)
 	nv_delete(np,sh.fun_tree,NV_NOFREE);
 }
 
-static void put_table(register Namval_t* np, const char* val, int flags, Namfun_t* fp)
+static void put_table(Namval_t* np, const char* val, int flags, Namfun_t* fp)
 {
-	register Dt_t		*root = ((struct table*)fp)->dict;
-	register Namval_t	*nq, *mp;
-	Namarr_t		*ap;
-	struct adata		data;
+	Dt_t		*root = ((struct table*)fp)->dict;
+	Namval_t	*nq, *mp;
+	Namarr_t	*ap;
+	struct adata	data;
 	if(val)
 	{
 		nv_putv(np,val,flags,fp);
@@ -1282,18 +1287,18 @@ static void put_table(register Namval_t* np, const char* val, int flags, Namfun_
 		return;
 	memset(&data,0,sizeof(data));
 	data.mapname = nv_name(np);
-	nv_scan(sh.fun_tree,delete_fun,(void*)&data,NV_FUNCTION,NV_FUNCTION|NV_NOSCOPE);
+	nv_scan(sh.fun_tree,delete_fun,&data,NV_FUNCTION,NV_FUNCTION|NV_NOSCOPE);
 	dtview(root,0);
 	for(mp=(Namval_t*)dtfirst(root);mp;mp=nq)
 	{
 		_nv_unset(mp,flags);
 		nq = (Namval_t*)dtnext(root,mp);
 		dtdelete(root,mp);
-		free((void*)mp);
+		free(mp);
 	}
 	dtclose(root);
 	if(!(fp->nofree&1))
-		free((void*)fp);
+		free(fp);
 	np->nvfun = 0;
 }
 
@@ -1302,14 +1307,14 @@ static void put_table(register Namval_t* np, const char* val, int flags, Namfun_
  */
 static char *get_table(Namval_t *np, Namfun_t *fp)
 {
-	register Dt_t *root = ((struct table*)fp)->dict;
+	Dt_t *root = ((struct table*)fp)->dict;
 	static Sfio_t *out;
-	register int first=1;
-	register Dt_t *base = dtview(root,0);
+	int first=1;
+	Dt_t *base = dtview(root,0);
         if(out)
-                sfseek(out,(Sfoff_t)0,SEEK_SET);
+                sfseek(out,0,SEEK_SET);
         else
-                out =  sfnew((Sfio_t*)0,(char*)0,-1,-1,SF_WRITE|SF_STRING);
+                out =  sfnew(NULL,NULL,-1,-1,SF_WRITE|SF_STRING);
 	for(np=(Namval_t*)dtfirst(root);np;np=(Namval_t*)dtnext(root,np))
 	{
                 if(!nv_isnull(np) || np->nvfun || nv_isattr(np,~NV_NOFREE))
@@ -1324,7 +1329,7 @@ static char *get_table(Namval_t *np, Namfun_t *fp)
 	sfputc(out,0);
 	if(base)
 		dtview(root,base);
-	return((char*)out->_data);
+	return (char*)out->_data;
 }
 
 static const Namdisc_t table_disc =
@@ -1344,24 +1349,24 @@ Namval_t *nv_parent(Namval_t *np)
 {
 	struct table *tp = (struct table *)nv_hasdisc(np,&table_disc);
 	if(tp)
-		return(tp->parent);
-	return(0);
+		return tp->parent;
+	return NULL;
 }
 
 Dt_t *nv_dict(Namval_t* np)
 {
 	struct table *tp = (struct table*)nv_hasdisc(np,&table_disc);
 	if(tp)
-		return(tp->dict);
+		return tp->dict;
 	np = sh.last_table;
 	if(np && (tp = (struct table*)nv_hasdisc(np,&table_disc)))
-		return(tp->dict);
-	return(sh.var_tree);
+		return tp->dict;
+	return sh.var_tree;
 }
 
 int nv_istable(Namval_t *np)
 {
-	return(nv_hasdisc(np,&table_disc)!=0);
+	return nv_hasdisc(np,&table_disc)!=0;
 }
 
 /*
@@ -1375,7 +1380,7 @@ Namval_t *nv_mount(Namval_t *np, const char *name, Dt_t *dict)
 		pp = np;
 	else
 		pp = nv_lastdict();
-	tp = sh_newof((struct table*)0, struct table,1,0);
+	tp = sh_newof(NULL, struct table,1,0);
 	if(name)
 	{
 		Namfun_t *fp = pp->nvfun;
@@ -1390,7 +1395,7 @@ Namval_t *nv_mount(Namval_t *np, const char *name, Dt_t *dict)
 	tp->parent = pp;
 	tp->fun.disc = &table_disc;
 	nv_disc(mp, &tp->fun, NV_FIRST);
-	return(mp);
+	return mp;
 }
 
 const Namdisc_t *nv_discfun(int which)
@@ -1398,11 +1403,11 @@ const Namdisc_t *nv_discfun(int which)
 	switch(which)
 	{
 	    case NV_DCADD:
-		return(&Nv_bdisc);
+		return &Nv_bdisc;
 	    case NV_DCRESTRICT:
-		return(&RESTRICTED_disc);
+		return &RESTRICTED_disc;
 	}
-	return(0);
+	return NULL;
 }
 
 /*
@@ -1411,25 +1416,28 @@ const Namdisc_t *nv_discfun(int which)
  */
 int nv_hasget(Namval_t *np)
 {
-	register Namfun_t	*fp;
+	Namfun_t	*fp;
 	if(np==sh_scoped(IFSNOD))
-		return(0);	/* avoid BUG_IFSISSET: always return false for IFS */
+		return 0;	/* avoid BUG_IFSISSET: always return false for IFS */
 	for(fp=np->nvfun; fp; fp=fp->next)
 	{
 		if(!fp->disc || (!fp->disc->getnum && !fp->disc->getval))
 			continue;
-		return(1);
+		return 1;
 	}
-	return(0);
+	return 0;
 }
 
 #if SHOPT_NAMESPACE
 Namval_t *sh_fsearch(const char *fname, int add)
 {
-	int	offset = stktell(sh.stk);
-	sfputr(sh.stk,nv_name(sh.namespace),'.');
-	sfputr(sh.stk,fname,0);
-	fname = stkptr(sh.stk,offset);
-	return(nv_search(fname,sh_subfuntree(add&NV_ADD),add));
+	if(*fname!='.')
+	{
+		int	offset = stktell(sh.stk);
+		sfputr(sh.stk,nv_name(sh.namespace),'.');
+		sfputr(sh.stk,fname,0);
+		fname = stkptr(sh.stk,offset);
+	}
+	return nv_search(fname,sh_subfuntree(add&NV_ADD),add);
 }
 #endif /* SHOPT_NAMESPACE */
