@@ -2,7 +2,7 @@
 #                                                                      #
 #               This software is part of the ast package               #
 #          Copyright (c) 1982-2012 AT&T Intellectual Property          #
-#          Copyright (c) 2020-2022 Contributors to ksh 93u+m           #
+#          Copyright (c) 2020-2023 Contributors to ksh 93u+m           #
 #                      and is licensed under the                       #
 #                 Eclipse Public License, Version 2.0                  #
 #                                                                      #
@@ -457,9 +457,9 @@ s=SECONDS
 set -o pipefail
 for ((i=0; i < 30; i++))
 do	print hello
-	sleep .01
-done |  "$binsleep" .1
-(( (SECONDS-s) < .2 )) || err_exit 'early termination not causing broken pipe'
+	sleep .02
+done |  "$binsleep" .2
+(( (SECONDS-s) < .4 )) || err_exit 'early termination not causing broken pipe'
 
 got=$({ trap 'print trap' 0; print -n | "$bincat"; } & wait "$!")
 [[ $got == trap ]] || err_exit "trap on exit not correctly triggered (expected 'trap', got $(printf %q "$got"))"
@@ -990,5 +990,34 @@ do
 	|| err_exit "last command in script exec-optimized in spite of $sig trap ($pid1 == $pid2)"
 done
 
+# ======
+# Nested compound assignment misparsed in $(...) or ${ ...; } command substitution
+# https://github.com/ksh93/ksh/issues/269
+# TODO: one of the tests below crashes when actually executed; test lexing only by using noexec.
+for testcode in \
+	': $( typeset -a arr=((a b c) 1) )' \
+	': ${ typeset -a arr=((a b c) 1); }' \
+	': $( typeset -a arr=( ( ((a b c)1))) )' \
+	': ${ typeset -a arr=( ( ((a b c)1))); }' \
+	': $(( 1 << 2 ))' \
+	': $(: $(( 1 << 2 )) )' \
+	': $( (( 1 << 2 )) )' \
+	': $( : $( (( 1 << 2 )) ) )' \
+	': $( (( $( (( 1 << 2 )); echo 1 ) << 2 )) )' \
+	': $( typeset -a arr=((a $(( 1 << 2 )) c) 1) )' \
+	'typeset -Ca arr=((a=ah b=beh c=si))' \
+	': $( typeset -Ca arr=((a=ah b=beh c=si)) )' \
+	'r=${ typeset -Ca arr=((a=ah b=beh c=si)); }' \
+	'set --noexec; : $( typeset -a arr=((a $(( $( typeset -a barr=((a $(( 1 << 2 )) c) 1); echo 1 ) << $( typeset -a bazz=((a $(( 1 << 2 )) c) 1); echo 2 ) )) c) 1) )' \
+	'r=$(typeset -C arr=( (a=ah b=beh c=si) 1 (e f g)));'
+do
+	# fork comsub with 'ulimit' on old ksh to avoid a fixed lexer bug crashing the entire test script
+	got=$(let ".sh.version >= 20211209" || ulimit -c 0
+		eval "set +x; $testcode" 2>&1) \
+	|| err_exit "comsub/arithexp lexing test $(printf %q "$testcode"):" \
+		"got status $? and $(printf %q "$got")"
+done
+unset testcode
+
 # ======
 exit $((Errors<125?Errors:125))
