@@ -2,7 +2,7 @@
 #                                                                      #
 #               This software is part of the ast package               #
 #          Copyright (c) 1982-2012 AT&T Intellectual Property          #
-#          Copyright (c) 2020-2022 Contributors to ksh 93u+m           #
+#          Copyright (c) 2020-2023 Contributors to ksh 93u+m           #
 #                      and is licensed under the                       #
 #                 Eclipse Public License, Version 2.0                  #
 #                                                                      #
@@ -790,7 +790,9 @@ got=$({ "$SHELL" -c '
 	Job_t job
 	job.set_pos
 	job.set_pos
-'; } 2>&1) || err_exit 'crash involving short int as first type member' \
+'; } 2>&1)
+e=$?
+let "e==0" || err_exit 'crash involving short int as first type member' \
 	"(got status $e$( ((e>128)) && print -n /SIG && kill -l "$e"), $(printf %q "$got"))"
 
 # ======
