@@ -2,7 +2,7 @@
 *                                                                      *
 *               This software is part of the ast package               *
 *          Copyright (c) 1985-2011 AT&T Intellectual Property          *
-*          Copyright (c) 2020-2022 Contributors to ksh 93u+m           *
+*          Copyright (c) 2020-2023 Contributors to ksh 93u+m           *
 *                      and is licensed under the                       *
 *                 Eclipse Public License, Version 2.0                  *
 *                                                                      *
@@ -42,5 +42,5 @@ realpath(const char* file, char* path)
 	    	if (!(path = malloc (PATH_MAX)))
         		return NULL;
 	}
-	return resolvepath(file, path, PATH_MAX) > 0 ? path : (char*)0;
+	return resolvepath(file, path, PATH_MAX) > 0 ? path : NULL;
 }
