@@ -2,7 +2,7 @@
 *                                                                      *
 *               This software is part of the ast package               *
 *          Copyright (c) 1985-2011 AT&T Intellectual Property          *
-*          Copyright (c) 2020-2022 Contributors to ksh 93u+m           *
+*          Copyright (c) 2020-2023 Contributors to ksh 93u+m           *
 *                      and is licensed under the                       *
 *                 Eclipse Public License, Version 2.0                  *
 *                                                                      *
@@ -20,7 +20,7 @@
 
 #undef sfclrerr
 
-int sfclrerr(reg Sfio_t* f)
+int sfclrerr(Sfio_t* f)
 {
 	return __sf_clrerr(f);
 }
