@@ -2,7 +2,7 @@
 *                                                                      *
 *               This software is part of the ast package               *
 *          Copyright (c) 1985-2012 AT&T Intellectual Property          *
-*          Copyright (c) 2020-2022 Contributors to ksh 93u+m           *
+*          Copyright (c) 2020-2023 Contributors to ksh 93u+m           *
 *                      and is licensed under the                       *
 *                 Eclipse Public License, Version 2.0                  *
 *                                                                      *
@@ -22,5 +22,5 @@
 Sfio_t*
 fmemopen(void* buf, size_t size, const char* mode)
 {
-	return sfnew(NiL, buf, size, -1, SF_STRING|_sftype(mode, NiL, NiL));
+	return sfnew(NULL, buf, size, -1, SF_STRING|_sftype(mode, NULL, NULL));
 }
