@@ -2,7 +2,7 @@
 *                                                                      *
 *               This software is part of the ast package               *
 *          Copyright (c) 1985-2011 AT&T Intellectual Property          *
-*          Copyright (c) 2020-2022 Contributors to ksh 93u+m           *
+*          Copyright (c) 2020-2023 Contributors to ksh 93u+m           *
 *                      and is licensed under the                       *
 *                 Eclipse Public License, Version 2.0                  *
 *                                                                      *
@@ -22,6 +22,6 @@
 void
 rewind(Sfio_t* f)
 {
-	sfseek(f, (Sfoff_t)0, SEEK_SET|SF_PUBLIC);
+	sfseek(f, 0, SEEK_SET|SF_PUBLIC);
 	sfclrlock(f);
 }
