@@ -2,7 +2,7 @@
 *                                                                      *
 *               This software is part of the ast package               *
 *          Copyright (c) 1985-2012 AT&T Intellectual Property          *
-*          Copyright (c) 2020-2022 Contributors to ksh 93u+m           *
+*          Copyright (c) 2020-2023 Contributors to ksh 93u+m           *
 *                      and is licensed under the                       *
 *                 Eclipse Public License, Version 2.0                  *
 *                                                                      *
@@ -15,6 +15,7 @@
 *                     Phong Vo <phongvo@gmail.com>                     *
 *                  Martijn Dekker <martijn@inlv.org>                   *
 *            Johnothan King <johnothanking@protonmail.com>             *
+*               K. Eugene Carlson <kvngncrlsn@gmail.com>               *
 *                                                                      *
 ***********************************************************************/
 /*
@@ -69,6 +70,8 @@ static const Namval_t		options[] =
 static Tm_info_t	_tm_info_ = { 0 };
 Tm_info_t*		_tm_infop_ = &_tm_info_;
 
+static char*		tz_abbr;
+
 #if _tzset_environ
 
 static char	TZ[256];
@@ -112,11 +115,11 @@ _tm_localtime(const time_t* t)
 static int
 tzwest(time_t* clock, int* isdst)
 {
-	register struct tm*	tp;
-	register int		n;
-	register int		m;
-	int			h;
-	time_t			epoch;
+	struct tm*	tp;
+	int		n;
+	int		m;
+	int		h;
+	time_t		epoch;
 
 	/*
 	 * convert to GMT assuming local time
@@ -141,6 +144,10 @@ tzwest(time_t* clock, int* isdst)
 	 */
 
 	tp = tmlocaltime(clock);
+#if _mem_tm_zone_tm
+	if (tp->tm_zone && !tz_abbr)
+		tz_abbr = strdup(tp->tm_zone);
+#endif
 	if (n = tp->tm_yday - n)
 	{
 		if (n > 1)
@@ -169,7 +176,7 @@ tmopt(void* a, const void* p, int n, const char* v)
 			tm_info.deformat = (n && (n = strlen(v)) > 0 && (n < 2 || v[n-2] != '%' || v[n-1] != '?')) ? strdup(v) : tm_info.format[TM_DEFAULT];
 			break;
 		case TM_type:
-			tm_info.local->type = (n && *v) ? ((zp = tmtype(v, NiL)) ? zp->type : strdup(v)) : 0;
+			tm_info.local->type = (n && *v) ? ((zp = tmtype(v, NULL)) ? zp->type : strdup(v)) : 0;
 			break;
 		default:
 			if (n)
@@ -186,22 +193,24 @@ tmopt(void* a, const void* p, int n, const char* v)
  */
 
 static void
-tmlocal(void)
+tmlocal(time_t now)
 {
-	register Tm_zone_t*	zp;
-	register int		n;
-	register char*		s;
-	register char*		e;
+	Tm_zone_t*		zp;
+	int			n;
+	char*			s;
+	char*			e;
 	int			i;
 	int			m;
 	int			isdst;
 	char*			t;
 	struct tm*		tp;
-	time_t			now;
 	char			buf[16];
 
 	static Tm_zone_t	local;
 
+	local.standard = 0;
+	local.daylight = 0;
+
 #if _tzset_environ
 	{
 		char**	v = environ;
@@ -238,7 +247,6 @@ tmlocal(void)
 	 */
 
 	tm_info.zone = tm_info.local = &local;
-	time(&now);
 	n = tzwest(&now, &isdst);
 
 	/*
@@ -268,6 +276,13 @@ tmlocal(void)
 	 * now get the time zone names
 	 */
 
+	if (tz_abbr)
+	{
+		if (!isdst)
+			local.standard = strdup(tz_abbr);
+		else
+			local.daylight = strdup(tz_abbr);
+	}
 #if _dat_tzname
 	if (tzname[0])
 	{
@@ -275,8 +290,10 @@ tmlocal(void)
 		 * POSIX
 		 */
 
-		local.standard = strdup(tzname[0]);
-		local.daylight = strdup(tzname[1]);
+		if (!local.standard)
+			local.standard = strdup(tzname[0]);
+		if (!local.daylight)
+			local.daylight = strdup(tzname[1]);
 	}
 	else
 #endif
@@ -286,30 +303,16 @@ tmlocal(void)
 		 * BSD
 		 */
 
-		local.standard = s;
-		if (s = strchr(s, ','))
-			*s++ = 0;
-		else
-			s = "";
-		local.daylight = s;
-	}
-	else if ((s = getenv("TZ")) && *s && *s != ':' && (s = strdup(s)))
-	{
-		/*
-		 * POSIX style but skipped by tmlocaltime()
-		 */
-
-		local.standard = s;
-		if (*++s && *++s && *++s)
+		if (!local.standard)
+			local.standard = s;
+		if (!local.daylight)
 		{
-			*s++ = 0;
-			tmgoff(s, &t, 0);
-			for (s = t; isalpha(*t); t++);
-			*t = 0;
+			if (s = strchr(s, ','))
+				*s++ = 0;
+			else
+				s = "";
+			local.daylight = s;
 		}
-		else
-			s = "";
-		local.daylight = s;
 	}
 	else
 	{
@@ -325,7 +328,8 @@ tmlocal(void)
 			if (zp->west == n && zp->dst == m)
 			{
 				local.type = t;
-				local.standard = zp->standard;
+				if (!local.standard)
+					local.standard = zp->standard;
 				if (!(s = zp->daylight))
 				{
 					e = (s = buf) + sizeof(buf);
@@ -337,7 +341,8 @@ tmlocal(void)
 					}
 					s = strdup(buf);
 				}
-				local.daylight = s;
+				if (!local.daylight)
+					local.daylight = s;
 				break;
 			}
 		}
@@ -349,18 +354,20 @@ tmlocal(void)
 
 			e = (s = buf) + sizeof(buf);
 			s = tmpoff(s, e - s, tm_info.format[TM_UT], n, 0);
-			local.standard = strdup(buf);
+			if (!local.standard)
+				local.standard = strdup(buf);
 			if (s < e - 1)
 			{
 				*s++ = ' ';
 				tmpoff(s, e - s, tm_info.format[TM_UT], m, TM_DST);
-				local.daylight = strdup(buf);
+				if (!local.daylight)
+					local.daylight = strdup(buf);
 			}
 		}
 	}
 	if (!*local.standard && !local.west && !local.dst && (s = getenv("TZ")))
 	{
-		if ((zp = tmzone(s, &t, NiL, NiL)) && !*t)
+		if ((zp = tmzone(s, &t, NULL, NULL)) && !*t)
 		{
 			local.standard = strdup(zp->standard);
 			if (zp->daylight)
@@ -380,7 +387,7 @@ tmlocal(void)
 	 * set the options
 	 */
 
-	stropt(getenv("TM_OPTIONS"), options, sizeof(*options), tmopt, NiL);
+	stropt(getenv("TM_OPTIONS"), options, sizeof(*options), tmopt, NULL);
 
 	/*
 	 * the time zone type is probably related to the locale
@@ -394,7 +401,7 @@ tmlocal(void)
 		{
 			if (zp->type)
 				t = zp->type;
-			if (tmword(s, NiL, zp->standard, NiL, 0))
+			if (tmword(s, NULL, zp->standard, NULL, 0))
 			{
 				local.type = t;
 				break;
@@ -420,7 +427,7 @@ tmlocal(void)
 		if (local.daylight)
 			zp++;
 		for (; !zp->type && zp->standard; zp++)
-			if (tmword(s, NiL, zp->standard, NiL, 0))
+			if (tmword(s, NULL, zp->standard, NULL, 0))
 			{
 				tm_info.flags |= TM_UTC;
 				break;
@@ -433,10 +440,11 @@ tmlocal(void)
  */
 
 void
-tminit(register Tm_zone_t* zp)
+tminit(Tm_zone_t* zp, time_t now, const char newzone)
 {
 	static uint32_t		serial = ~(uint32_t)0;
 
+	tz_abbr = 0;
 	if (serial != ast.env_serial)
 	{
 		serial = ast.env_serial;
@@ -446,9 +454,9 @@ tminit(register Tm_zone_t* zp)
 			tm_info.local = 0;
 		}
 	}
-	if (!tm_info.local)
-		tmlocal();
-	if (!zp)
+	if (!tm_info.local || newzone)
+		tmlocal(now);
+	if (!zp || newzone)
 		zp = tm_info.local;
 	tm_info.zone = zp;
 }
